/*
 * 
 * ProFiDo - The Processes Fitting Toolkit Dortmund
 * Copyright (C) 2010 - 2012 LS Informatik IV, TU Dortmund
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

package core;

import java.io.Serializable;
import java.util.Vector;

import core.history.ChangeDynamicTextParameterValueHistoryObject;
import core.history.HistoryObject;

/**
 * This class represents dynamic text parameters. A dynamic text parameter
 * consists of an dynamic amount of values. The exact amount of values is
 * determined by the value of a specific other parameter which is identified by
 * its name.
 * 
 * In the configuration xml-file the tag &ltdynamicref&gt defines the parameter
 * which determines the number of values. The tag &ltdynamicpart&gt specifies
 * the part of this parameters name which will be enumerated when displaying the
 * parameters name.
 */

public class DynamicTextParameter extends TextParameter implements Serializable {
	private static final long serialVersionUID = 1L;

	/**
	 * Part of this parameters name which will be enumerated when displaying the
	 * parameters name as specified in the configuration xml-file.
	 */
	private String dynamicPart;

	/**
	 * Name of the parameter which determines the number of values of this
	 * parameter as specified in the configuration xml-file.
	 */
	private String dynamicReferenceString;

	/**
	 * Reference to the parameter which determines the number of values of this
	 * parameter.
	 */
	private TextParameter dynamicReference;

	/**
	 * Temporary values of this parameter. Used to allow safe parsing of
	 * potentially malformated user input without interfering with the current
	 * values.
	 */
	private Vector<String> temporaryValues;

	/**
	 * Current values of this parameter.
	 */
	private Vector<String> values;

	/**
	 * Creates the parameter group according to the passed values.
	 */
	public DynamicTextParameter(String name, String description,
			String successor, String dynamicpart, String dynamicref) {
		super(name, description, successor);
		this.dynamicPart = dynamicpart;
		this.dynamicReferenceString = dynamicref;

		this.temporaryValues = new Vector<String>();
		this.values = new Vector<String>();
	}

	@Override
	public String getFormat() {
		String staticpart = this.name.replaceAll(this.dynamicPart, "");
		return "<" + staticpart + "1> <" + staticpart + "2> ... <" + staticpart
				+ this.dynamicReferenceString + ">";
	}

	@Override
	public String parseInputToTemporaryValue(String text) {
		int expectedValueCount = 0;
		try {
			expectedValueCount = Integer.valueOf(this.dynamicReference
					.getTemporaryValue());
		} catch (NumberFormatException e) {}

 		if (expectedValueCount != 0) {
			this.temporaryValues.removeAllElements();

			while ((temporaryValues.size() != expectedValueCount) && (!text.equals(""))) {
				// Parse until specified amount of values are collected
				int spacePos = text.indexOf(" ");

				String curValue = "";
				if (spacePos != -1) {
					curValue = text.substring(0, spacePos);
					text = text.substring(spacePos + 1, text.length());
				} else {
					curValue = text;
					text = "";
				}

				this.temporaryValues.add(curValue);
			}
		}

		return text;
	}

	public String getDynamicReferenceString() {
		return this.dynamicReferenceString;
	}

	public void setDynamicReference(TextParameter referenceParameter) {
		this.dynamicReference = referenceParameter;
	}

	/**
	 * This method converts the delivered string vector to a single string.
	 */
	private String valuesToString(Vector<String> values) {
		String result = "";
		for (String curValue : values) {
			result = result + curValue + " ";
		}
		result = result.trim();
		return result;
	}

	@Override
	public String getTemporaryValue() {
		return this.valuesToString(this.temporaryValues);
	}

	@Override
	public String getValue() {
		return this.valuesToString(this.values);
	}

	public Vector<String> getValues() {
		return this.values;
	}

	@Override
	public HistoryObject createHistoryObjectToInheritTemporaryValues() {
		boolean changes = true;

		if (this.temporaryValues.size() == this.values.size()) {
			// Check if there were any changes
			changes = false;
			for (int curIndex = 0; curIndex < this.values.size(); curIndex++) {
				String curTempValue = this.temporaryValues.get(curIndex);
				String curValue = this.values.get(curIndex);

				if (!curValue.equals(curTempValue)) {
					changes = true;
					break;
				}
			}
		}

		if (changes)
			return new ChangeDynamicTextParameterValueHistoryObject(this,
					this.temporaryValues);
		else
			return null;
	}

	public void setValues(Vector<String> newValues) {
		this.values = newValues;
	}

}
