/*
 * 
 * ProFiDo - The Processes Fitting Toolkit Dortmund
 * Copyright (C) 2010 - 2012 LS Informatik IV, TU Dortmund
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

package core;

import helper.SmallHelpers;

import java.io.Serializable;
import java.util.HashMap;
import java.util.Vector;

import core.history.ChangeBooleanParameterGroupValuesHistoryObject;
import core.history.ChangeSelectParameterGroupValuesHistoryObject;
import core.history.ChangeTextParameterGroupValuesHistoryObject;
import core.history.HistoryObject;

/**
 * This class defines the properties of parameter groups. Each command line
 * parameter of a job is represented by a parameter group, consisting of one
 * keyword and zero to several values dedicated to this key.
 * 
 * The complete command line call of a job is represented by the binary name and
 * a list of parameters which consist of several parameter groups. A parameter
 * group might have the attributes visible, disabled and type, specifying the
 * visibility within the canvas and weather the parameter is employed when
 * creating the command line call or not. Possible types might be filename,
 * bool, text or select.
 * 
 * The possible parameters for each job type are specified in the xml-file by
 * the tags &ltname&gt, &ltdescription&gt, &ltkey&gt and &ltdefault&gt followed
 * by one or several parameters each specified within the tag &ltparameter&gt.
 * &ltname&gt contains an identifier for the attribute. The content of
 * &ltdescription&gt is used to display further information about the parameter
 * group. The tag &ltkey&gt is used for the command line call generation when
 * exporting the workflow. If a tool has several parameter groups the
 * &ltsuccessor&gt tag will give the name of the succeeding parameter group
 * within the command line call for the corresponding program.
 * 
 * 
 */

public abstract class ParameterGroup implements Serializable {
	private static final long serialVersionUID = 1L;

	/**
	 * Name of the corresponding command line parameter as specified within the
	 * configuration xml-file.
	 */
	protected String name;

	/**
	 * Description of the corresponding command line parameter as specified
	 * within the configuration xml-file.
	 */
	protected String description;

	/**
	 * Name of this parameters succeeding command line parameter as specified
	 * within the configuration xml-file.
	 */
	protected String successorName;

	/**
	 * Standard visibility of this parameter group as specified in this
	 * parameters description in the configuration xml-file.
	 */
	private boolean defaultVisibility;

	/**
	 * Standard value for enabled of this parameter group as specified in this
	 * parameters description in the configuration xml-file.
	 */
	private boolean defaultValueEnabled;

	/**
	 * Key used to start the corresponding parameter when generating the command
	 * line call as specified within the configuration xml-file.
	 */
	protected String key;

	/**
	 * Determine if the parameter group should be displayed on the canvas of the
	 * workflow.
	 */
	protected boolean visible;

	/**
	 * Determine if the parameter is employed when creating the workflow.
	 */
	protected boolean enabled;

	/**
	 * Reference to the succeeding parameter group object in accordance to the
	 * specified name from the configuration xml-file.
	 */
	private ParameterGroup successor;

	/**
	 * Determines if this parameter group will be shown in the edit job panel.
	 */
	private boolean fixed;

	/**
	 * Creates the parameter group according to the passed values.
	 */
	public ParameterGroup(String name, String key, String description,
			String successor, boolean visible, boolean enabled) {
		super();
		this.name = name;
		this.key = key;
		this.description = description;
		this.successorName = successor;
		this.defaultVisibility = visible;
		this.visible = visible;
		this.defaultValueEnabled = enabled;
		this.enabled = enabled;
	}

	/**
	 * Generates and returns the value part of this parameter groups command
	 * line call. If necessary due to the type of the parameter group the values
	 * are returned in a easy readable way.
	 */
	public abstract String getDisplayFriendlyValueString();

	/**
	 * Generates and returns the part of the command line call specified by this
	 * parameter group.
	 */
	public abstract String getCommandlineCallString();

	/**
	 * Generates (if necessary) and returns the description of this parameter
	 * group.
	 */
	public abstract String getDescription();

	/**
	 * This method set the parameters of the passed vector of parameter groups
	 * according to the passed hash map. A vector of all unchanged groups is
	 * returend.
	 */
	public static Vector<ParameterGroup> setParameterValues(
			Vector<ParameterGroup> inputGroups,
			HashMap<String, String> presetParameters) {
		Vector<ParameterGroup> changedParameterGroups = new Vector<ParameterGroup>();
		for (String curKey : presetParameters.keySet()) {
			HistoryObject curAction = null;
			ParameterGroup curGroup = SmallHelpers.findParameterGroupByName(
					curKey, inputGroups);
			if (curGroup instanceof BooleanParameterGroup) {
				boolean newValue = presetParameters.get(curKey)
						.equalsIgnoreCase("true");
				curAction = new ChangeBooleanParameterGroupValuesHistoryObject(
						"", (BooleanParameterGroup) curGroup, newValue);
			} else if (curGroup instanceof SelectParameterGroup) {
				String curChoice = ((SelectParameterGroup) curGroup)
						.getChoiceValue();
				curAction = new ChangeSelectParameterGroupValuesHistoryObject(
						null, (SelectParameterGroup) curGroup,
						presetParameters.get(curKey), curChoice);
			} else if (curGroup instanceof TextParameterGroup) {
				((TextParameterGroup) curGroup)
						.parseInputToTemporaryValues(presetParameters
								.get(curKey));
				Vector<HistoryObject> newValaues = ((TextParameterGroup) curGroup)
						.createHistoryObjectsToAdoptNewValues();
				curAction = new ChangeTextParameterGroupValuesHistoryObject("",
						(TextParameterGroup) curGroup, newValaues);
			}
			if (curAction != null) {
				curAction.performAction();
				changedParameterGroups.add(curGroup);
			}
		}

		// Create result containing all changed parameter groups
		Vector<ParameterGroup> result = new Vector<ParameterGroup>();
		for (ParameterGroup curGroup : inputGroups) {
			if (changedParameterGroups.indexOf(curGroup) == -1) {
				result.add(curGroup);
			}
		}

		return result;
	}
	
	/**
	 * This method changes the "fixed"-attribute of all parameter groups 
	 * in the "inputGroups" vector whose name is listed in the "fixedParameters"
	 * vector.
	 */
	public static void setFixedAttribute(
			Vector<ParameterGroup> inputGroups, Vector<String> fixedParameters) {
		for (String curName : fixedParameters) {
			ParameterGroup curGroup = SmallHelpers.findParameterGroupByName(
					curName, inputGroups);
			if (curGroup != null) {
				curGroup.setFixed(true);
			}
		}
	}

	public boolean getDefaultVisibility() {
		return this.defaultVisibility;
	}

	public boolean getDefaultValueEnabled() {
		return this.defaultValueEnabled;
	}

	public String getName() {
		return this.name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public boolean isVisible() {
		return this.visible;
	}

	public void setVisible(boolean visible) {
		this.visible = visible;
	}

	public void setEnable(boolean enable) {
		this.enabled = enable;
	}

	public boolean isEnabled() {
		return this.enabled;
	}

	public String getKey() {
		return this.key;
	}

	public String getSuccessorName() {
		return this.successorName;
	}

	public void setSuccessor(ParameterGroup successor) {
		this.successor = successor;
	}

	public ParameterGroup getSuccessor() {
		return this.successor;
	}

	public void setFixed(boolean isFixed) {
		this.fixed = isFixed;
	}

	/**
	 * This method returns true if the parameter group is fixed. 
	 * Otherwise it returns false.
	 */
	public boolean isFixed() {
		return this.fixed;
	}

}