/*
 * ProFiDo - The Processes Fitting Toolkit Dortmund
 * Copyright (C) 2010 - 2012 LS Informatik IV, TU Dortmund
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

package core.graph;

import gui.helper.Grid;
import gui.helper.Line;
import gui.helper.TextManager;
import helper.SmallHelpers;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Point;
import java.awt.Polygon;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.io.File;
import java.io.Serializable;
import java.util.Vector;

import core.TypedFile;

/**
 * This class defines the properties of the edges connection nodes in the
 * workflow graph. Edges are directed and thus determine the order of the
 * execution of the different jobs and the propagation of intermediate results.
 * The output of a source node is the input of a destination node. There are
 * restrictions in connecting nodes. The output of a source node must be
 * compatible with the expected/accepted inputs of a destination node to connect
 * these nodes. Each edge contains a label with the model type and the file name
 * for the model description that is propagated between two nodes.
 * 
 * This class stores and controls all information relevant for managing the
 * edge within the graph and for drawing it on the canvas. To allow more
 * flexible alignment of edges, each edge has several points on which an edge
 * can be dragged in order to avoid crossings and thus to enable a more readable
 * graph layout.
 */

public class Edge implements Serializable, MouseListener {
	private static final long serialVersionUID = 1L;
	protected static final int EDGE_POINT_RADIUS = 4;
	protected static Color EDGE_DEFAULT_COLOR = Color.BLACK;

	/**
	 * Stores the two nodes which are connected by this edge.
	 */
	private GraphNode from, to;

	/**
	 * Color of this edge.
	 */
	private Color color;

	/**
	 * X-coordinate of this edges center point.
	 */
	private int centerX;

	/**
	 * Y-coordinate of this edges center point.
	 */
	private int centerY;

	/**
	 * Pointer to the first point node of this edge.
	 */
	private PointNode firstPointNode;

	/**
	 * Selected file type of input node as represented by this edge.
	 */
	private String selectedType;

	/**
	 * Position within the target jobs parameter list of the file represented by
	 * this edge.
	 */
	private String orderKey;

	/**
	 * Indicates if this edges file name is visible within the workflow.
	 */
	private boolean fileNameVisible = false;

	/**
	 * Indicates if this edges file type is visible within the workflow.
	 */
	private boolean fileTypeVisible = true;

	/**
	 * Indicates if this edges order key is visible within the workflow.
	 */
	private boolean orderKeyVisible = false;
	private TypedFile fileOfEdge;

	/**
	 * Creates the edge according to the passed values.
	 */
	public Edge(GraphNode pFrom, GraphNode pTo, String orderKey) {
		this.from = pFrom;
		this.to = pTo;
		this.color = Edge.EDGE_DEFAULT_COLOR;
		this.orderKey = orderKey;

		this.refreshType();

		// Initialize point nodes of this edge
		this.createNewPointNodeAfterLastPointNode(null);
	}

	/**
	 * Creates a temporary edge without any source node. Handle with care, only
	 * use for edges that are used temporary.
	 */
	public Edge(TypedFile fileOfEdge, String selectedType, GraphNode pTo,
			String orderKey) {
		this.from = null;
		this.fileOfEdge = fileOfEdge;
		this.selectedType = selectedType;
		this.to = pTo;
		this.color = Edge.EDGE_DEFAULT_COLOR;
		this.orderKey = orderKey;
	}

	/**
	 * Refreshes the type of this edge according to the possible types as
	 * defined by the nodes connected by this node.
	 */
	public void refreshType() {
		// Check if any types of the nodes connected by this edge match
		Vector<String> intersectingTypes = this.getPossibleTypes();

		// Select first matching file type if several are possible
		if (intersectingTypes.size() > 0) {
			this.selectedType = intersectingTypes.get(0);
		}
	}

	/**
	 * Returns all types that can be assigned to this edge. The possible types
	 * are determined by the intersection of this edges source nodes output
	 * types and this edges destination nodes input types.
	 */
	public Vector<String> getPossibleTypes() {
		return this.from.intersectOutputTypesWithNodesInputTypes(this.to);
	}

	/**
	 * Adds a new point node after the passed point node to this edge and
	 * returns a reference to it. If no point node was specified, the new node
	 * is created after the last existing node. If any coordinates are passed to
	 * this method, the newly created node is moved there.
	 */
	public PointNode createNewPointNodeAfterSelected(PointNode curNode,
			Point gridPosition) {
		if (curNode == null) {
			curNode = this.firstPointNode;
		}

		PointNode newNode;
		// If curNode is still null, there is no first node currently
		if (curNode == null) {
			newNode = new PointNode(this.from.getGrid(), this.from, this.to,
					false, this);
			this.firstPointNode = newNode;
		} else {
			BasicNode rightNeighbour = curNode.getRightNeighbour();

			newNode = new PointNode(this.from.getGrid(), curNode,
					rightNeighbour, true, this);

			if (rightNeighbour instanceof PointNode) {
				((PointNode) rightNeighbour).setLeftNeighbour(newNode);
			}
			curNode.setRightNeighbour(newNode);
		}

		// Move to position if any was specified
		if (gridPosition == null) {
			this.notifiyThatNodeHasMoved();
		} else {
			newNode.moveToGridCoordinates(gridPosition);
		}
		return newNode;
	}

	/**
	 * Adds a new point node with the passed coordinates after the last point
	 * node to this edge.
	 */
	public PointNode createNewPointNodeAfterLastPointNode(Point gridPosition) {
		return this.createNewPointNodeAfterSelected(this.getLastPointNode(),
				gridPosition);
	}

	/**
	 * This methods removes the passed point node from this edge.
	 */
	public void deletePointNode(PointNode nodeToDelete) {
		BasicNode leftNeighbour = nodeToDelete.getLeftNeighbour();
		BasicNode rightNeighbour = nodeToDelete.getRightNeighbour();

		if (leftNeighbour instanceof PointNode) {
			((PointNode) leftNeighbour).setRightNeighbour(rightNeighbour);
		}

		if (rightNeighbour instanceof PointNode) {
			((PointNode) rightNeighbour).setLeftNeighbour(leftNeighbour);
		}

		if (nodeToDelete == this.firstPointNode) {
			this.firstPointNode = (PointNode) rightNeighbour;
		}
	}

	public void setColor(Color pColor) {
		this.color = pColor;
	}

	public GraphNode getFrom() {
		return this.from;
	}

	public GraphNode getTo() {
		return this.to;
	}

	public int getCenterX() {
		return this.centerX;
	}

	public int getCenterY() {
		return this.centerY;
	}

	public Color getColor() {
		return this.color;
	}

	/**
	 * This method draws the entire edge.
	 */
	public void draw(Graphics pGraphics) {
		pGraphics.setColor(this.getColor());

		BasicNode start = this.from;
		BasicNode end = this.to;

		// Draw line from start to first point node
		BasicNode curPoint = this.firstPointNode;
		pGraphics.drawLine(start.getX(), start.getY(), curPoint.getX(),
				curPoint.getY());

		// Draw line between point nodes
		while (((PointNode) curPoint).getRightNeighbour() instanceof PointNode) {
			PointNode lastPoint = (PointNode) curPoint;
			curPoint = ((PointNode) curPoint).getRightNeighbour();

			pGraphics.drawLine(lastPoint.getX(), lastPoint.getY(),
					curPoint.getX(), curPoint.getY());
		}

		// Draw linbe from last point to end node
		pGraphics.drawLine(curPoint.getX(), curPoint.getY(), end.getX(),
				end.getY());

		// Draw point nodes
		BasicNode p = this.firstPointNode;
		while (p instanceof PointNode) {
			p.draw(pGraphics);
			p = ((PointNode) p).getRightNeighbour();
		}

		// Draw arrow head
		Point intersection = this.to.calculateIntersection(this);
		int xPos = intersection.x;
		int yPos = intersection.y;
		pGraphics.setColor(Color.RED);

		Point lastPointNodePosition = this.getLastPoint();
		double curAngle = new Line(lastPointNodePosition.x,
				lastPointNodePosition.y, this.to.getX(), this.to.getY())
				.getAngle();
		double headLineLength = 14;

		int[] xPoints = new int[3];
		int[] yPoints = new int[3];

		double winkelR = curAngle - Math.PI / 3;// + (Math.PI / 14);
		double winkelL = winkelR - Math.PI / 3;// - (Math.PI / 7);

		// Right part
		xPoints[0] = (int) Math
				.round(xPos + Math.sin(winkelR) * headLineLength);
		yPoints[0] = (int) Math
				.round(yPos - Math.cos(winkelR) * headLineLength);
		xPoints[1] = xPos;
		yPoints[1] = yPos;
		// Left part
		xPoints[2] = (int) Math
				.round(xPos + Math.sin(winkelL) * headLineLength);
		yPoints[2] = (int) Math
				.round(yPos - Math.cos(winkelL) * headLineLength);

		Polygon poli = new Polygon(xPoints, yPoints, 3);
		pGraphics.fillPolygon(poli);

		// Print information on arrow head
		boolean displayFileName = Graph.DISPLAY_ALL_EDGE_INFORMATION
				|| this.fileNameVisible && Graph.DISPLAY_EDGE_INFORMATION;
		boolean displayFileType = Graph.DISPLAY_ALL_EDGE_INFORMATION
				|| this.fileTypeVisible && Graph.DISPLAY_EDGE_INFORMATION;
		boolean displayOrderKey = Graph.DISPLAY_ALL_EDGE_INFORMATION
				|| Graph.ALWAYS_DISPLAY_ORDER_KEYS || this.orderKeyVisible
				&& Graph.DISPLAY_EDGE_INFORMATION;

		String text = this.getCaptionText(displayFileName, displayFileType,
				displayOrderKey);

		if (!text.equals("")) {
			pGraphics.setColor(Color.black);
			PointNode informationPointNode = this.firstPointNode;
			TextManager.writeSmallText(text, informationPointNode.getX(),
					informationPointNode.getY() - 3 * Edge.EDGE_POINT_RADIUS,
					pGraphics, true);
		}
		pGraphics.setColor(Color.black);

	}

	/**
	 * Returns an information string representing details about this edge.
	 */
	public String getTooltipText() {
		return this.getCaptionText(true, true, true);
	}

	/**
	 * Returns an information string containing the file type of this edge.
	 */
	public String getCaptionText(boolean displayFileName,
			boolean displayFileType, boolean displayOrderKey) {
		String result = "";
		if (displayFileType) {
			result = result + this.selectedType;
		}

		if (displayFileName) {
			if (displayFileType) {
				result = result + " : ";
			}
			result = result + this.getFileName().getName();
		}

		if (displayOrderKey) {
			if (displayFileType || displayFileName) {
				result = result + " ";
			}
			result = result + "(" + this.orderKey + ")";
		}

		return result;
	}

	/**
	 * This method determines if there is a point node of this edge at the
	 * passed coordinates.
	 */
	public boolean isInsidePointNodeOfEdge(int pX, int pY, Grid pGrid) {
		boolean isinside = false;
		BasicNode p = this.firstPointNode;
		while (p instanceof PointNode) {

			if (pX <= p.getX() + 4 & pX >= p.getX() & pY <= p.getY() + 4
					& pY >= p.getY() | pX <= p.getX() + 4 & pX >= p.getX()
					& pY <= p.getY() - 4 & pY >= p.getY() | pX >= p.getX() - 4
					& pX <= p.getX() & pY >= p.getY() + 4 & pY <= p.getY()
					| pX >= p.getX() - 4 & pX <= p.getX() & pY >= p.getY() - 4
					& pY <= p.getY()) {
				isinside = true;
			}
			p = ((PointNode) p).getRightNeighbour();
		}
		return isinside;
	}

	/**
	 * Creates and return line object representing this edge.
	 */
	public Line getLine() {
		Point lastPoint = this.getLastPoint();
		return new Line(lastPoint.x, lastPoint.y, this.to.getX(),
				this.to.getY());
	}

	public PointNode getLastPointNode() {
		BasicNode p = this.firstPointNode;
		while (p != null
				&& ((PointNode) p).getRightNeighbour() instanceof PointNode) {
			p = ((PointNode) p).getRightNeighbour();
		}
		return (PointNode) p;
	}

	public Point getLastPoint() {
		return this.getLastPointNode().getPosition();
	}

	public TypedFile getFileName() {
		if (this.from != null)
			return this.from.getXMLOutputFile();
		else
			return this.fileOfEdge;
	}

	/**
	 * This method generates and returns a string representing all information
	 * of this edge needed to store this node in an xml-file.
	 */
	public String getXMLRepresentationOfEdge(int fromID, int toID) {
		String savefile = "<edge>" + "\n";
		savefile += "<from> " + fromID + " </from>" + "\n";
		savefile += "<to> " + toID + " </to>" + "\n";
		savefile += "<orderkey visible=\"" + this.orderKeyVisible + "\"> "
				+ this.orderKey + " </orderkey>" + "\n";
		savefile += "<points>" + "\n";
		BasicNode p = this.firstPointNode;
		while (p instanceof PointNode) {
			savefile += "<point> " + p.getX() + " " + p.getY() + " </point>"
					+ "\n";
			savefile += "<gridpoint> " + Math.round(p.getGridPosition().getX())
					+ " " + Math.round(p.getGridPosition().getY())
					+ " </gridpoint>" + "\n";
			p = ((PointNode) p).getRightNeighbour();
		}
		savefile += "</points>" + "\n";
		savefile += "<file>" + "\n";
		savefile += "<name visible=\"" + this.fileNameVisible + "\"> "
				+ this.getFileName().getName() + " </name>" + "\n";
		if (this.getFileName().getParent() != null
				&& !this.getFileName().getParent().equals("")) {
			savefile += "<path> " + this.getFileName().getParent()
					+ File.separator + " </path>" + "\n";
		}
		savefile += "<type visible=\"" + this.fileTypeVisible + "\"> "
				+ this.selectedType + " </type>" + "\n";
		savefile += "</file>" + "\n";
		savefile += "</edge>" + "\n";

		return savefile;
	}

	public void setEdgeFileType(String newType) {
		this.selectedType = newType;
	}

	public String getEdgeFileType() {
		return this.selectedType;
	}

	/**
	 * Defined in the MouseListener interface. Is currently not used by this
	 * class.
	 */
	@Override
	public void mouseClicked(MouseEvent e) {
	}

	/**
	 * Defined in the MouseListener interface. Is currently not used by this
	 * class.
	 */
	@Override
	public void mouseEntered(MouseEvent e) {
	}

	/**
	 * Defined in the MouseListener interface. Is currently not used by this
	 * class.
	 */
	@Override
	public void mouseExited(MouseEvent e) {
	}

	/**
	 * Defined in the MouseListener interface. Is currently not used by this
	 * class.
	 */
	@Override
	public void mousePressed(MouseEvent e) {
	}

	/**
	 * Defined in the MouseListener interface. Is currently not used by this
	 * class.
	 */
	@Override
	public void mouseReleased(MouseEvent e) {
	}

	/**
	 * This method notifies the edge that the node it starts at has moved and
	 * the positions of the edges PointNodes have to be recalculated.
	 */
	public void notifiyThatNodeHasMoved() {
		BasicNode curPointNode = this.firstPointNode;
		while (curPointNode instanceof PointNode) {
			((PointNode) curPointNode).notifiyThatNodeOrEdgeHasMoved();
			curPointNode = ((PointNode) curPointNode).getRightNeighbour();
		}
	}

	/**
	 * This method determines the point node which is located at the passed
	 * coordinates and returns it.
	 */
	public PointNode getPointNodeAtPosition(int x, int y, Grid pGrid) {
		BasicNode curPointNode = this.firstPointNode;
		while (curPointNode instanceof PointNode) {
			boolean inside = curPointNode.isInside(x, y);
			if (inside == true)
				return (PointNode) curPointNode;
			curPointNode = ((PointNode) curPointNode).getRightNeighbour();
		}
		return null;
	}

	/**
	 * Realigns all point nodes of this edge to the grid.
	 */
	public void realignToGrid() {
		BasicNode curPointNode = this.firstPointNode;
		while (curPointNode instanceof PointNode) {
			curPointNode.realignToGrid();
			curPointNode = ((PointNode) curPointNode).getRightNeighbour();
		}

		this.notifiyThatNodeHasMoved();
	}

	/**
	 * Removes all point nodes of this edge. Be aware that such an edge is not
	 * valid and drawable anymore. This should only be used for loading edges
	 * from a xml-file.
	 */
	public void deleteAllPointNodes() {
		this.firstPointNode = null;
	}

	public String getOrderKey() {
		return this.orderKey;
	}

	/**
	 * Sets the order key of this edges and triggers the resorting of this edges
	 * to-nodes incoming edges.
	 */
	public void setOrderKey(String newKey) {
		this.orderKey = newKey;
		// Whenever the order key is changed,
		// the sorting of the incoming edges of the
		// from edge have to be sorted
		this.to.sortIncomingEdgesAccordingToOrderKey();
	}

	public boolean isFileNameVisible() {
		return this.fileNameVisible;
	}

	public boolean isFileTypeVisible() {
		return this.fileTypeVisible;
	}

	public boolean isOrderKeyVisible() {
		return this.orderKeyVisible;
	}

	public void setVisibilty(boolean fileNameVisible, boolean fileTypeVisible,
			boolean orderKeyVisible) {
		this.fileNameVisible = fileNameVisible;
		this.fileTypeVisible = fileTypeVisible;
		this.orderKeyVisible = orderKeyVisible;
	}

	public PointNode getPointNodeAt(int x, int y) {
		BasicNode curPointNode = this.firstPointNode;
		while (curPointNode instanceof PointNode) {
			if (curPointNode.isInside(x, y))
				return (PointNode) curPointNode;
			curPointNode = ((PointNode) curPointNode).getRightNeighbour();
		}
		return null;
	}

	public PointNode getFirstPointNode() {
		return this.firstPointNode;
	}

	public String getDisplayFriendlyName() {
		return this.from.getName() + " -> " + this.to.getName();
	}

	/**
	 * This Method returns a Vector that contains all PointNodes of an Edge.
	 * 
	 * @return A Vector that contains all PointNodes on the Edge.
	 */
	public Vector<PointNode> getAllPointNodesOnEdge() {
		Vector<PointNode> result = new Vector<PointNode>();

		result.add(this.getFirstPointNode());
		BasicNode p = this.getFirstPointNode();
		while (p != null
				&& ((PointNode) p).getRightNeighbour() instanceof PointNode) {
			p = ((PointNode) p).getRightNeighbour();
			result.add((PointNode) p);
		}
		return result;
	}

	/**
	 * This method creates the TikZ representation for the current Edge. So TikZ
	 * will draw the Edge from the "from-GraphNode" to the "to-GraphNode". The
	 * method will check different scenarios and and creates the code
	 * accordingly.
	 * 
	 * @return The String that represents the Edge.
	 */
	public String getTikzRepresentation() {
		String result = "";
		Vector<PointNode> allPoints = this.getAllPointNodesOnEdge();

		// There is at least one PointNodes
		if (!allPoints.isEmpty()) {

			// There is only one PointNode on the Edge and it isn't user
			// positioned,(maybe it would be better to use the
			// "isUserPositioned" boolean
			// of a PointNode
			// to check it).
			if (this.getFirstPointNode().isOnlyPointNodeOfEdge()
					&& this.getFirstPointNode().getgridX() == -1
					&& this.getFirstPointNode().getgridY() == -1) {
				String label = this.getTikZPointNodeCaptionText();
				result = result
						+ "\\draw[-myarrow,fill=red] ("
						+ SmallHelpers.removeForLatexId(this.getFrom().getName()
						+ this.getFrom().getOrderPosition())
						+ ") -- ("
						+ SmallHelpers.removeForLatexId(this.getTo().getName()
						+ this.getTo().getOrderPosition())
						+ ") node[edgenode,  label={above:"
						+ SmallHelpers.escapeLatex(label.replace("<br>",
								" \\\\ ")) + "}] [pos=0.48] {};\n";

				// There is only one PointNode on the Edge and it is user
				// positioned
			} else if (this.getFirstPointNode().isOnlyPointNodeOfEdge()
					&& !(this.getFirstPointNode().getgridX() == -1)
					&& !(this.getFirstPointNode().getgridY() == -1)) {

				result = result + "\\draw[-myarrow,fill=red] ("
						+ SmallHelpers.removeForLatexId(this.getFrom().getName()
						+ this.getFrom().getOrderPosition()) + ") -- ("
						+ this.getFirstPointNode().getID() + ") -- ("
						+ SmallHelpers.removeForLatexId(this.getTo().getName()
						+ this.getTo().getOrderPosition()) + ");\n";

			}

			else {
				// There is more then one PointNode on the Edge and the
				// first PointNode is not user positioned
				if (!this.getFirstPointNode().isOnlyPointNodeOfEdge()
						&& this.getFirstPointNode().getgridX() == -1
						&& this.getFirstPointNode().getgridY() == -1) {

					String label = this.getTikZPointNodeCaptionText();
					result = result
							+ "\\draw[to2] ("
							+ SmallHelpers.removeForLatexId(this.getFrom().getName()
							+ this.getFrom().getOrderPosition())
							+ ") -- ("
							+ this.getFirstPointNode()
									.getRightNeighbourAsPointNode().getID()
							+ ") node[edgenode, label={above:"
							+ label.replace("<br>", " \\\\ ")
							+ "}] [pos=0.48] {};\n";

					for (PointNode curPointNode : allPoints) {
						if (curPointNode.equals(this.getFirstPointNode())
								&& curPointNode.getRightNeighbour() instanceof PointNode) {

							curPointNode = curPointNode
									.getRightNeighbourAsPointNode();

							result = result + "\\draw[to2] ("
									+ curPointNode.getID() + ")";
						}

						if (!curPointNode.equals(this.getFirstPointNode()
								.getRightNeighbourAsPointNode())) {
							result = result + "-- (" + curPointNode.getID()
									+ ")";

						} else {
							result = result + "";
						}
						if (curPointNode instanceof PointNode
								&& curPointNode.equals(this.getLastPointNode())) {
							result = result + "; \n";
							break;

						}

					}
					result = result + "\\draw[-myarrow,fill=red] ("
							+ this.getLastPointNode().getID() + ") -- " + "("
							+ SmallHelpers.removeForLatexId(this.getTo().getName()
							+ this.getTo().getOrderPosition()) + "); \n";
				}
				// There is more then one PointNode on the Edge and all
				// PointNodes are user positioned
				else {
					result = result + "\\draw[-myarrow,fill=red] ("
							+ SmallHelpers.removeForLatexId(this.getFrom().getName()
							+ this.getFrom().getOrderPosition()) + ")";
					for (PointNode curPointNode : allPoints) {

						result = result + "-- (" + curPointNode.getID() + ")";

					}

					result = result + "-- (" + SmallHelpers.removeForLatexId(this.getTo().getName()
							+ this.getTo().getOrderPosition()) + "); \n";

				}
			}
		}

		return result;
	}

	/**
	 * This method will create you a TikZ representation for all PointNodes on
	 * the current Edge.
	 * 
	 * @return The String that represents all PointNodes on the Edge.
	 */
	public String getTikzRepresentationOfPointNodes() {
		String result = "";
		Vector<PointNode> allPoints = this.getAllPointNodesOnEdge();

		for (PointNode curPoint : allPoints) {
			result = result + curPoint.getTikzRepresentation();
		}
		return result;

	}

	/**
	 * This method will return the PointNodeCaptionText, if it should be
	 * displayed on the PointNode (only for TikZ graphics).
	 * 
	 * @return will return the PointNodeCaptionText, if it should be displayed,
	 *         otherwise result will be a empty String.
	 */
	public String getTikZPointNodeCaptionText() {
		boolean displayFileName = Graph.DISPLAY_ALL_EDGE_INFORMATION
				|| this.fileNameVisible && Graph.DISPLAY_EDGE_INFORMATION;
		boolean displayFileType = Graph.DISPLAY_ALL_EDGE_INFORMATION
				|| this.fileTypeVisible && Graph.DISPLAY_EDGE_INFORMATION;
		boolean displayOrderKey = Graph.DISPLAY_ALL_EDGE_INFORMATION
				|| Graph.ALWAYS_DISPLAY_ORDER_KEYS || this.orderKeyVisible
				&& Graph.DISPLAY_EDGE_INFORMATION;

		String result = this.getCaptionText(displayFileName, displayFileType,
				displayOrderKey);
		return result;
	}

}
