/*
 * 
 * ProFiDo - The Processes Fitting Toolkit Dortmund
 * Copyright (C) 2010 - 2012 LS Informatik IV, TU Dortmund
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

package core.graph;

import gui.helper.Grid;
import gui.helper.Line;

import java.awt.Graphics;
import java.awt.Point;
import java.awt.Rectangle;
import java.io.Serializable;
import java.util.Vector;

import core.TypedFile;
import core.storage.ConfigurationXMLReader;

/**
 * This abstract class defines the properties of a file node. File nodes
 * represent external files in the file system in order to manage input and
 * output of the entire workflow.
 */

public abstract class FileNode extends GraphNode implements Serializable {
	private static final long serialVersionUID = 1L;

	/**
	 * FileName of the external file in the file system.
	 */
	protected TypedFile externalFileName;

	@Override
	protected String getCaptionText(boolean ignoreVisibilty,
			boolean markEnabled, boolean sortEnabled, boolean showTypes,
			boolean hideComment) {
		String result = "";
		if (this.externalFileName != null) {
			if (ignoreVisibilty) {
				result = result + this.externalFileName.getName() + " ("
						+ this.externalFileName.getTypes().get(0) + ")<br>";
			} else {
				result = result + this.externalFileName.getName() + "<br>";
			}
		}

		if (this instanceof InputFileNode && showTypes) {
			result = result + "<b>Output</b>: "
					+ this.getOutputFileTypes().get(0) + "<br>";
		}
		if (this instanceof OutputFileNode && showTypes) {
			result = result + "<b>Input</b>: "
					+ this.getInputFileTypes().get(0) + "<br>";
		}

		// Print comment
		if (!hideComment && (this.commentVisibilty || ignoreVisibilty)) {
			result = result + "<br>";

			String commentLines[] = this.comment.split("\n");
			for (String curLine : commentLines) {
				result = result + curLine + "<br>";
			}
		}

		return result;
	}

	/**
	 * Creates the node according to the passed values.
	 */
	public FileNode(int pX, int pY, ConfigurationXMLReader reader, Grid pGrid,
			int id, Graph graph) {
		super(pX, pY, reader, pGrid, id, graph);
	}

	@Override
	public boolean isInside(int pX, int pY) {
		Rectangle shape = this.getBoundingBox();

		double xdist = pX - this.x;
		double ydist = pY - this.y;

		double xdist2 = Math.pow(xdist, 2);
		double ydist2 = Math.pow(ydist, 2);

		return Math.sqrt(xdist2 + ydist2) <= shape.width / 2;
	}

	@Override
	protected Rectangle getBoundingBox() {
		Rectangle maxSpace = this.grid.getDrawingSpace(this.x, this.y);
		int r = 0;
		if (maxSpace.width < maxSpace.height) {
			r = 2 * maxSpace.width / 3;
		} else {
			r = 2 * maxSpace.height / 3;
		}

		Rectangle space = new Rectangle(this.x - r / 2, this.y - r / 2, r, r);
		return space;
	}

	@Override
	public Point calculateIntersection(Edge edge) {
		Line edgeLine = edge.getLine();
		Rectangle outerShape = this.getBoundingBox();

		// Prepare calculations
		double x = this.x; // X-Position of circle
		double y = this.y; // Y-Position of circle
		double r = outerShape.width / 2; // Radius of circle
		double m = edgeLine.getM(); // Value of line
		double n = edgeLine.getN(); // Value of line

		// Sqaures used for further calculations
		double m2 = Math.pow(m, 2);
		double n2 = Math.pow(n, 2);
		double r2 = Math.pow(r, 2);
		double x2 = Math.pow(x, 2);
		double y2 = Math.pow(y, 2);

		// For every point p on the circle the following equation holds:
		// (1) (x - px)^2 + (y - py)^2 = r^2
		// <=> x^2 - 2 * x * px + px^2 + y^2 - 2 * y * py + py^2 - r^2 = 0

		// For every point on the line the following equation holds:
		// (2) py = m * px + n

		// Together we get:
		// (3) x^2 - 2 * x * px + px^2 + y^2 - 2 * y * (m * px + n) + (m * px +
		// n)^2 - r^2 = 0
		// <=> x^2 - 2 * x * px + px^2 + y^2 - 2 * y * m * px - 2 * y * n + (m *
		// px)^2 + 2 * m * px * n + n^2 - r^2 = 0
		// <=> (m * px)^2 + px^2 + 2 * m * px * n - 2 * x * px - 2 * y * m * px
		// + (x^2 + y^2 - 2 * y * n + n^2 - r^2) = 0
		// <=> (m^2 + 1) * px^2 + ((2 * m * n) - (2 * x) - (2 * y * m)) * px +
		// (x^2 + y^2 - 2 * y * n + n^2 - r^2) = 0
		// <=> px^2 + (2 * m * n - 2 * x - 2 * y * m) * (px / (m^2 + 1)) + (x^2
		// + y^2 - 2 * y * n + n^2 - r^2) / (m^2 + 1) = 0
		// Therefore we get for the P/Q-Formula (x^2 + p * x + q = 0):
		double p = (2 * m * n - 2 * x - 2 * y * m) / (m2 + 1);
		double q = (x2 + y2 - 2 * y * n + n2 - r2) / (m2 + 1);

		// Calculate intersection x:
		// x1/2 = -(p / 2)^2 +- ((p / 2)^2 - q)^0.5
		double intersectionX1 = -(p / 2) + Math.sqrt(Math.pow(p / 2, 2) - q);
		double intersectionX2 = -(p / 2) - Math.sqrt(Math.pow(p / 2, 2) - q);

		Vector<Point> intersections = new Vector<Point>();
		if (edgeLine.getStartX() != edgeLine.getEndX()) {
			intersections.add(new Point((int) Math.round(intersectionX1),
					(int) Math.round(edgeLine.f(intersectionX1))));
			intersections.add(new Point((int) Math.round(intersectionX2),
					(int) Math.round(edgeLine.f(intersectionX2))));
		} else {
			double intersectionY1 = edgeLine.getEndY() + r;
			double intersectionY2 = edgeLine.getEndY() - r;
			double iX = edgeLine.getEndX();
			intersections.add(new Point((int) Math.round(iX), (int) Math
					.round(intersectionY1)));
			intersections.add(new Point((int) Math.round(iX), (int) Math
					.round(intersectionY2)));

		}
		double px = edge.getLastPoint().getX();
		double py = edge.getLastPoint().getY();
		Point closestIntersection = this.getClosestIntersection(intersections,
				px, py);
		return closestIntersection;
	}

	@Override
	protected void drawOutline(Graphics pGraphics, Rectangle pShape) {
		pGraphics.fillOval(pShape.x, pShape.y, pShape.width, pShape.height);
	}

	@Override
	public String getName() {
		return this.name;
	}

	public String getInformativeName() {
		return this.externalFileName.getName();
	}

	// public abstract String getCommandLineCall(String softwarePath, String
	// resultPath);

	public TypedFile getExternalFileName() {
		return this.externalFileName;
	}

	public int getConverterID() {
		return this.referenceID;
	}

	public void setExternalFileName(String newFileName) {
		Vector<String> types = this.externalFileName.getTypes();
		this.externalFileName = new TypedFile(newFileName, types);

	}

	public abstract void setConverterID(int newConverterID);

	public String getParameterTableLines() {
		return "";
	}

	/**
	 * This method will return the FileNode caption text if the caption is
	 * painted in the workflow. This method is needed to draw the TikZ graph
	 * exactly like the original workflow in ProFiDo.
	 * 
	 * @return The string that contains the caption text
	 */
	public String getFileNodeCaptionText() {
		String caption = "";
		if (Graph.DISPLAY_ALL_NODE_INFORMATION
				|| Graph.DISPLAY_NODE_INFORMATION) {
			caption = this.getCaptionText(Graph.DISPLAY_ALL_NODE_INFORMATION,
					Graph.DISPLAY_ALL_NODE_INFORMATION,
					Graph.DISPLAY_ALL_NODE_INFORMATION, false, false);

		}
		return caption;
	}
}
