/*
 * 
 * ProFiDo - The Processes Fitting Toolkit Dortmund
 * Copyright (C) 2010 - 2012 LS Informatik IV, TU Dortmund
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

package core.graph;

import gui.AboutFrame;
import gui.MainWindowFrame;
import gui.MainWindowPanel;
import gui.ShowHistoryFrame;
import gui.StatusDialog;
import gui.WorkflowPanel;
import gui.helper.Grid;
import gui.helper.InstantViewManager;
import gui.helper.ReportObject;

import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.image.BufferedImage;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.Serializable;
import java.util.Map;
import java.util.Vector;

import javax.imageio.ImageIO;
import javax.net.ssl.SSLEngineResult.Status;
import javax.swing.JOptionPane;

import core.TypedFile;
import core.history.History;
import core.history.HistoryObject;
import core.storage.ConfigurationXMLReader;

/**
 * This class represents the graph which shapes the workflow with all its
 * components.
 * 
 * A workflow consists of nodes and edges, where each node represents different
 * calculation steps and edges determine the data flow. Nodes correspond to to
 * different tools whilst edges determine the order of the execution. The
 * workflow can be exported into a Shell script in order to be executed. This
 * shell script consists of one setup script which prepares the workspace and
 * the workflow script which executes the actual workflow.
 * 
 */

public class Graph implements Serializable {
	private static final long serialVersionUID = 1L;

	/**
	 * Indicates if edge information should be displayed.
	 */
	public static boolean DISPLAY_EDGE_INFORMATION;

	/**
	 * Indicates if Node information should be displayed.
	 */
	public static boolean DISPLAY_NODE_INFORMATION;

	/**
	 * Indicates if order keys should be displayed.
	 */
	public static boolean ALWAYS_DISPLAY_ORDER_KEYS;

	/**
	 * Indicates if all edge information should be displayed.
	 */
	public static boolean DISPLAY_ALL_EDGE_INFORMATION;

	/**
	 * Indicates if all node information should be displayed.
	 */
	public static boolean DISPLAY_ALL_NODE_INFORMATION;

	/**
	 * Used to get unique Id's.
	 */
	private static int NEXT_FREE_ID;

	/**
	 * Nodes of this graph.
	 */
	private Vector<GraphNode> nodes;

	/**
	 * Edges of this graph.
	 */
	private Vector<Edge> edges;

	/**
	 * Name of this graph.
	 */
	private String name;

	/**
	 * Default location of all dialogs if set.
	 */
	private String workingDirektory;

	/**
	 * Stores the location of the workflow script if it was exported already.
	 */
	private String locationOfWorkflowScipt;

	/**
	 * Reference to the xmlReader used for reading the configuration xml-file.
	 */
	private ConfigurationXMLReader reader;

	/**
	 * History to keep track of changes to the graph.
	 */
	private History history;

	/**
	 * Indicates if this graph has been saved since the last changes.
	 */
	private boolean saved;

	/**
	 * Creates the graph according to the passed values.
	 */
	public Graph(String name, String outputFolder, String expFolder,
			ConfigurationXMLReader reader, WorkflowPanel workflowPanel,
			ShowHistoryFrame historyFrame) {
		this.nodes = new Vector<GraphNode>();
		this.edges = new Vector<Edge>();
		// this.workflowPanel = workflowPanel;
		this.reader = reader;
		this.name = name;
		this.workingDirektory = outputFolder;
		this.locationOfWorkflowScipt = expFolder;
		this.history = new History(workflowPanel, historyFrame);
		this.saved = true;
	}

	public String getWorkingDirectory() {
		return this.workingDirektory;
	}

	public void setWorkingDirectory(String outputFolder) {
		this.saved = false;
		this.workingDirektory = outputFolder;
	}

	public String getExpFolder() {
		return this.locationOfWorkflowScipt;
	}

	/**
	 * Adds the passed node to the graph.
	 */
	public void addNode(GraphNode k) {
		this.nodes.add(k);
	}

	/**
	 * Adds the passed edge to the graph.
	 */
	public void addEdge(Edge k) {
		// Check if edge is already existent in the graph
		if (this.getSameEdge(k) == null) {
			this.edges.add(k);
			k.getFrom().addOutgoingEdge(k);
			k.getTo().addIncomingEdge(k);
		}
	}

	/**
	 * This method returns the list of all edges which are connected to the
	 * passed node.
	 */
	public Vector<Edge> getAllEdgesConnectedToNode(BasicNode pAbstractNode) {
		Vector<Edge> result = new Vector<Edge>();

		if (!(pAbstractNode instanceof PointNode)) {
			result.addAll(((GraphNode) pAbstractNode).getIncomingEdges());
			result.addAll(((GraphNode) pAbstractNode).getOutgoingEdges());
		}
		return result;
	}

	/**
	 * Removes the passed node from the graph.
	 */
	public void removeNode(BasicNode pAbstractNode) {
		if (pAbstractNode instanceof PointNode) {
			((PointNode) pAbstractNode).getEdge().deletePointNode(
					(PointNode) pAbstractNode);
		} else {
			this.nodes.remove(pAbstractNode);
			// Hide edit windows if it was visible
			if (pAbstractNode instanceof JobNode) {
				JobNode curNode = (JobNode) pAbstractNode;
				if (curNode.hasEditDialog()) {
					curNode.hideEditDialog();
				}
			}
		}
	}

	/**
	 * Removes all edges connecting the two passed nodes.
	 */
	public void removeEdge(GraphNode pFrom, GraphNode pTo) {
		Edge curEdge = this.getSameEdge(new Edge(pFrom, pTo, ""));
		while (curEdge != null) {
			this.removeEdge(curEdge);

			curEdge = this.getSameEdge(new Edge(pFrom, pTo, ""));
		}
	}

	/**
	 * Removes the passed edge from the graph and deletes all references to it
	 * in all nodes connected to it.
	 */
	public void removeEdge(Edge pEdge) {
		this.edges.remove(pEdge);

		// Remove references in nodes
		pEdge.getFrom().removeEdge(pEdge);
		pEdge.getTo().removeEdge(pEdge);
	}

	/**
	 * Searches and returns edge with the same start and end point as the passed
	 * one.
	 */
	public Edge getSameEdge(Edge k) {
		for (Edge curEdge : this.edges) {
			if (curEdge.getFrom().equals(k.getFrom())
					&& curEdge.getTo().equals(k.getTo()))
				return curEdge;
		}
		return null;
	}

	/**
	 * This method removes all edges from the graph.
	 */
	public void removeAllEdges() {
		for (GraphNode curEdge : this.nodes) {
			curEdge.removeAllEdges();
		}
		this.edges = new Vector<Edge>();
		this.saved = false;
	}

	public Vector<Edge> getEdges() {
		return this.edges;
	}

	public Vector<GraphNode> getAbstractNodes() {
		return this.nodes;
	}

	/**
	 * This method draws the entire workflow
	 */
	public void draw(Graphics pGraphics, Grid pGrid) {
		// Edges
		for (Edge curEdge : this.edges) {
			curEdge.draw(pGraphics);
		}

		// Nodes
		for (GraphNode curAbstractNode : this.nodes) {
			curAbstractNode.draw(pGraphics);
		}
	}

	/**
	 * Realigns all nodes and edges to the grid after resizing of the gui.
	 */
	public void realignToGrid(Grid pGrid) {
		for (GraphNode curAbstractNode : this.nodes) {
			curAbstractNode.realignToGrid();
		}
		for (Edge curEdge : this.edges) {
			curEdge.realignToGrid();
		}
	}

	/**
	 * This method determines the node which is located at the passed
	 * coordinates and returns it.
	 */
	public BasicNode getBasicNodeAtPosition(int x, int y, Grid pGrid) {
		for (GraphNode curAbstractNode : this.nodes) {
			if (curAbstractNode.isInside(x, y))
				return curAbstractNode;
		}
		for (Edge curEdge : this.edges) {
			PointNode curEdgePointNode = curEdge.getPointNodeAt(x, y);
			if (curEdgePointNode != null)
				return curEdgePointNode;
		}
		return null;

	}

	/**
	 * This method determines the edge which is located at the passed
	 * coordinates and returns it.
	 */
	public Edge getEdgeAtPosition(int x, int y, Grid pGrid) {
		for (Edge curEdge : this.edges) {
			boolean inside = curEdge.isInsidePointNodeOfEdge(x, y, pGrid);
			if (inside == true)
				return curEdge;
		}
		return null;
	}

	public ConfigurationXMLReader getReader() {
		return this.reader;
	}

	public Grid getGrid() {
		for (GraphNode curAbstractNode : this.nodes)
			return curAbstractNode.getGrid();
		return null;
	}

	/**
	 * Removes any mark (marked, edited, visible) from all node in the graph.
	 */
	/*
	 * public void unmarkEverything() { for (GraphNode curAbstractNode : nodes)
	 * { curAbstractNode.setCurrentlyEdited(false);
	 * curAbstractNode.setEditDialogVisible(false);
	 * curAbstractNode.setMarked(false); }
	 * 
	 * }
	 */

	public String getName() {
		return this.name;
	}

	/**
	 * Returns the next free id and increments it.
	 */
	public static int getNextFreeID() {
		Graph.NEXT_FREE_ID++;
		return Graph.NEXT_FREE_ID - 1;
	}

	public void setName(String newName) {
		this.saved = false;
		this.name = newName;
	}

	/**
	 * Generate a file name which is not used yet.
	 */
	public String generateNewFileName(String center) {
		return this.getName() + "_" + center + Graph.getNextFreeID();
	}

	/**
	 * This method creates the two shell scripts allowing the workflow
	 * represented by this graph to be executet.
	 * 
	 * @param workflowPanel
	 * @param createReport
	 */
	public void createScripts(String targetFileName,
			WorkflowPanel workflowPanel, boolean createReport)
			throws IOException {
		this.locationOfWorkflowScipt = new TypedFile(targetFileName)
				.getParent();
		this.saved = false;

		// Determine which binaries and scripts are used
		Vector<Integer> usedProgrammIDs = new Vector<Integer>();
		Vector<Integer> usedInputConverterIDs = new Vector<Integer>();
		Vector<Integer> usedOutputConverterIDs = new Vector<Integer>();

		for (GraphNode curAbstractNode : this.nodes) {
			int curID = curAbstractNode.getReferenceID();
			if (curAbstractNode instanceof JobNode) {
				if (usedProgrammIDs.indexOf(curID) == -1) {
					usedProgrammIDs.add(curID);
				}
			} else if (curAbstractNode instanceof InputFileNode) {
				if (usedInputConverterIDs.indexOf(curID) == -1) {
					usedInputConverterIDs.add(curID);
				}
			} else if (curAbstractNode instanceof OutputFileNode) {
				if (usedOutputConverterIDs.indexOf(curID) == -1) {
					usedOutputConverterIDs.add(curID);
				}
			}
		}

		// Determine all requirements
		Vector<String> requirements = this.reader.getRequirements(
				usedProgrammIDs, usedInputConverterIDs, usedOutputConverterIDs);

		this.createWorkflowScript(targetFileName, requirements, createReport);
		// Ensure proper extension
		if (targetFileName.indexOf(".sh") == -1) {
			targetFileName = targetFileName + ".sh";
		}

		String setupFileName = targetFileName.substring(0,
				targetFileName.lastIndexOf("."))
				+ "_setup.sh";
		this.createSetupScript(setupFileName, usedProgrammIDs, requirements);

		if (createReport) {
			this.createReport(workflowPanel);
		}
	}

	/**
	 * This Method creates a partial script that checks whether all the
	 * necessary programs are available.
	 */
	public String createRequirmentsCheck(Vector<String> requirements,
			boolean exitOnError) {
		String result = "";
		if (requirements != null) {
			result = result
					+ "# Before any further steps, check if all required external binaries are present in the path.\n";
			result = result + "requirements=\"";
			for (String curRequirement : requirements) {
				result = result + curRequirement.trim() + " ";
			}
			result = result.trim();
			result = result + "\"\n";
			result = result
					+ "# Further requirements can be added to the above\n";
			result = result
					+ "# variable separated by blanks. Example: \"ls cat head\"\n";
			result = result + "missingCommands=\"\"\n";
			result = result + "for curCmd in $requirements; do\n";
			result = result + "\tif ! [ `command -v $curCmd` ]; then\n";
			result = result
					+ "\t\tmissingCommands=\"$missingCommands\\n$curCmd\"\n";
			result = result + "\tfi\n";
			result = result + "done\n";
			result = result + "if ! [ -z \"$missingCommands\" ]; then\n";
			result = result
					+ "\techo -e \"Warning, the following required external commands were not found on your system.\"\n";
			result = result
					+ "\techo -e \"Please ensure their presence before executing the workflow. $missingCommands\"\n";
			if (exitOnError) {
				result = result + "\texit 1\n";
			}
			result = result + "fi\n";

		}

		return result;
	}

	/**
	 * This method creates a script to create the working environment for the
	 * actual workflow execution. This script setups all the necessary tools for
	 * the workflow. It creates all needed folders and copies all used the
	 * binaries.
	 * 
	 * @param requirements
	 * @param usedProgrammIDs
	 */
	public void createSetupScript(String targetFileName,
			Vector<Integer> usedProgrammIDs, Vector<String> requirements)
			throws IOException {
		String result = this.createScriptHeader(false);

		result = result + this.createRequirmentsCheck(requirements, false)
				+ "\n";
		result = result
				+ "# Before the workflow can be used, this script sets up the environment.\n";
		result = result + "\n";
		result = result + "# First, all folders are set.\n";

		result = result + "export SOFTWARESOURCE=\""
				+ this.reader.getConfigValue("softwaresource").get(0) + "\"\n";
		result = result + "export TARGETDIR=\""
				+ new TypedFile(targetFileName).getParent() + "\"\n";

		result = result + "\n";
		result = result + "\n";

		// result = result +
		// "# If no parameter was passed, remove old Folders\n";
		// result = result + "if [ -z \"$1\" ]; then \n";
		// result = result + "		\\rm -f -R ${TARGETDIR}/Results/\n";
		// result = result + "		\\rm -f -R ${TARGETDIR}/Software/\n";
		// result = result + "fi\n";

		result = result + "\n";
		result = result + "\n";
		result = result + "# Creater Folders\n";
		// result = result + "\\mkdir -p \"${TARGETDIR}/Results/\"\n";
		result = result + "\\mkdir -p \"${TARGETDIR}/Results/Tmp/\"\n";
		result = result + "\\mkdir -p \"${TARGETDIR}/Software/\"\n";
		result = result + "\n";
		
		

		result = result + "\n";
		result = result + "\n";
		
		result = result + "# Copy Software\n";

		
		// Copy all used binaries
		for (int curID : usedProgrammIDs) {
			String binarayPath = this.reader.getProgrammBinaryPathByID(curID);
			if (!binarayPath.equals("")) {
				binarayPath = binarayPath
						.substring(0, binarayPath.length() - 1);
			}
			String binarayName = this.reader.getProgrammBinaryNameByID(curID);

			if (binarayPath.equals("")) {
				result = result + "\\cp -f -R \"${SOFTWARESOURCE}/"
						+ binarayName + "\"" + " \"${TARGETDIR}/Software/"
						+ "\"" + "\n";
			} else {
				result = result + "\\cp -f -R \"${SOFTWARESOURCE}/"
						+ binarayPath + "\"" + " \"${TARGETDIR}/Software/"
						+ "\"" + "\n";
			}
		}

		// Copy all other needed folders
		Vector<String> auxiliaryfolders = this.reader
				.getConfigValue("auxiliaryfolder");
		for (String auxiliaryfolder : auxiliaryfolders) {

			result = result + "\\cp -f -R " + "\"" + auxiliaryfolder + "\""
					+ " \"${TARGETDIR}/Software\"\n";
		}

		this.writeFile(targetFileName, result, true);
	}

	public String createScriptInitialization(String baseDir,
			Vector<String> requirements, boolean forceRequirments,
			boolean exportBasedir, boolean exitOnError) {
		String result = this.createScriptHeader(exitOnError);

		result = result
				+ this.createRequirmentsCheck(requirements, forceRequirments);
		result = result
				+ "# Before the actual execution, variables are set. To adapt the\n";
		result = result
				+ "# batch to another location just change the BASEDIR variable.\n";

		if (exportBasedir) {
			result = result + "export BASEDIR=\"" + baseDir + "\"\n\n";
		}

		result = result + "# Derived folders: \n";
		result = result + "export RESULTDIR=\"${BASEDIR}/Results\"\n";
		result = result + "export SOFTWAREDIR=\"${BASEDIR}/Software\"\n";
		return result;
	}

	protected String getLatexHeader() {
		String result = "";

		result = result + "\\documentclass[11pt]{article}\n";
		result = result + "\\usepackage{a4}\n";
		result = result + "\\usepackage[latin1]{inputenc}\n";
		result = result + "\\usepackage{graphicx,color}\n";
		result = result + "\\usepackage{times}\n";
		result = result + "\\usepackage{amstext}\n";
		result = result + "\\usepackage{amsfonts}\n";
		result = result + "\\usepackage[T1]{fontenc}\n";
		result = result + "\\usepackage{multicol}\n";
		result = result + "\\usepackage{listings}\n";
		result = result + "\\usepackage{float}\n";
		result = result + "\\usepackage{caption}\n";
		result = result + "\\usepackage{subcaption}\n";
		result = result + "\\usepackage{amssymb}\n";
		result = result + "\\usepackage{amsthm}\n";
		result = result + "\\usepackage{textcomp} \n";
		result = result + "\\usepackage{tikz}\n";
		result = result + "\\usepackage{datetime}\n";
		result = result + "\\usetikzlibrary{shapes}\n";
		result = result + "\\usetikzlibrary{arrows,backgrounds,automata}\n";
		result = result + "\\usetikzlibrary{fit}\n";
		result = result + "\\pgfdeclarelayer{bg}\n";
		result = result + "\\pgfsetlayers{bg,main}\n";

		result = result + "\\definecolor{lightgray}{rgb}{.9, .9, .9}\n";

		result = result
				+ "\\newcommand{\\profidoincludegraphics}[2][width=0.8\\textwidth]{\\IfFileExists{#2}{\\includegraphics[{#1}]{#2}}{}}\n";
		result = result
				+ "\\newcommand{\\profidoinput}[1]{\\IfFileExists{#1}{\\input{#1}}{}}\n";
		result = result
				+ "\\newcommand{\\profidoinclude}[1]{\\IfFileExists{#1}{\\include{#1}}{}}\n";

		result = result + "\\usepackage{grffile}\n";

		result = result + "\\lstset{\n";
		result = result + "\tbackgroundcolor=\\color{lightgray},\n";
		result = result + "\textendedchars=true,\n";
		result = result + "\tbasicstyle=\\footnotesize\\ttfamily,\n";
		result = result + "\tshowstringspaces=false,\n";
		result = result + "\tshowspaces=false,\n";
		result = result + "\tnumberstyle=\\footnotesize\\ttfamily,\n";
		result = result + "\tnumbersep=9pt,\n";
		result = result + "\ttabsize=2,\n";
		result = result + "\tbreaklines=true,\n";
		result = result + "\tshowtabs=false}\n";

		result = result + "\\setcounter{tocdepth}{3}\n";

		result = result + "\\newsavebox{\\tmpbox}\n";
		result = result
				+ "\\newenvironment{tabwrap}{\\begin{lrbox}{\\tmpbox}}\n";
		result = result + "{\\end{lrbox}%\n";
		result = result + "\\setbox0\\hbox{\\usebox\\tmpbox}%\n";
		result = result + "\\ifdim\\wd0<\\textwidth\n";
		result = result + "\t\\usebox\\tmpbox%\n";
		result = result + "\\else\n";
		result = result + "\t\\resizebox{\\textwidth}{!}{\\usebox\\tmpbox}%\n";
		result = result + "\\fi\n";
		result = result + "}\n";

		result = result + "\\begin{document}\n";

		result = result + "\\profidoinclude{Files/settings}\n";
		result = result + "\\newcommand{\\HRule}{\\rule{\\linewidth}{0.5mm}}\n";

		result = result + "\\begin{titlepage}\n";
		result = result + "\\vspace{10.0cm}\n";
		result = result + "\\begin{center}\n";
		result = result + "\\HRule  \\\\[0.4cm]\n";
		result = result + "{\\Huge\\bf Report for Workflow\\\\[0.4cm]"
				+ this.getName() + "} \\\\[0.4cm]\n";
		result = result + "\\HRule \\\\[5ex]\n";
		result = result + "{Generated with ProFiDo}\\\\[5ex]\n";
		result = result + "\\today \\\\\n";
		result = result + "\\currenttime \\\\[8ex]\n";
		result = result + "\\end{center} \n";
		result = result + "\\end{titlepage}\n";
		result = result + "\\newpage\n";

		result = result + "\\tableofcontents\n";
		result = result + "\\newpage\n";
		result = result + "\\listoffigures\n";
		result = result + "\\newpage\n";

		result = result + "\n";
		result = result + "\n";
		result = result
				+ "% This latex file provides a report for the workflow "
				+ this.getName() + ".\n";
		result = result
				+ "% To change which representation of which file type should be included\n";
		result = result
				+ "% modify the settings.tex accordingly. Please note that the settings.tex\n";
		result = result
				+ "% file will be overriden each time the report.sh scripts is executed.\n";
		result = result + "\n";
		result = result + "\n";

		result = result + "\n" + GraphNode.getInitializationForLatexVariables()
				+ "\n\n";

		return result;
	}

	public String getLatexFooter() {
		return "\\end{document}\n";
	}

	private String getLatexRepresentationOfGraph(Grid grid) {
		// Create image
		// this.saveScreenshot("Overview.png", grid);

		String tikzCode = this.createTiKzGraphic();
		this.writeFile(this.locationOfWorkflowScipt
				+ "/Results/Report/Files/overview.tex", tikzCode, false);

		// Create latex
		String result = "";
		result = result + "% -----------------------------\n";
		result = result + "% Overview for Workflow " + this.getName() + "\n";
		result = result + "% -----------------------------\n";

		result = result + "\\section{Workflow Overview}\n";
		result = result + "\\subsection{Graph}\n";

		result = result + "\\profidoinput{" + this.locationOfWorkflowScipt
				+ "/Results/Report/Files/overview.tex" + "}\n";

		// result = result + "\\begin{figure}[h!]\\label{GlobalOverview}\n";
		// result = result + "\\centering \n";
		// result = result
		// + "\\includegraphics[width=0.9\\textwidth]{Files/Overview}\n";
		// result = result + "\\caption{Overview of Workflow " + this.getName()
		// + "}\n";
		// result = result + "\\end{figure}\n";

		return result;
	}

	private void createReport(String expFolder, WorkflowPanel workPanel) {
		
		String exportFolder = expFolder;
		WorkflowPanel workflowPanel = workPanel;

		// Show status frame
		StatusDialog statusDialog = new StatusDialog(
				workflowPanel.getMainWindowFrame());
		
		
		statusDialog.setTitle("Generating Report Framework");
		statusDialog.setLocationRelativeTo(workflowPanel);
		statusDialog.setProgressBarMinimum(0);
		statusDialog.setVisibleAll(true);
		statusDialog.setProgressBarValue(0);
		
		// Create folders
		new File(exportFolder + File.separator + "Results").mkdir();
		new File(exportFolder + File.separator + "Results" + File.separator
				+ "Report").mkdir();
		new File(exportFolder + File.separator + "Results" + File.separator
				+ "Report" + File.separator + "Files").mkdir();
		String relativeTargetFileName = "/" + "Report" + "/" + "report";

		// Prepare storage and helper
		Vector<ReportObject> reportObjects = new Vector<ReportObject>();
		InstantViewManager manager = new InstantViewManager(workflowPanel,
				this.reader);

		// Create latex and script file
		String latexFile = this.getLatexHeader();
		String scriptFile = "";

		// Create global part
		latexFile = latexFile
				+ this.getLatexRepresentationOfGraph(workflowPanel.getGrid());

		// Iterate through nodes and create report objects
		Vector<GraphNode> visitedAbstractNodes = new Vector<GraphNode>();
		Vector<GraphNode> unvisitedAbstractNodes = new Vector<GraphNode>();

		this.sortNodes();

		for (GraphNode curAbstractNode : this.nodes) {
			if (curAbstractNode instanceof InputFileNode) {
				visitedAbstractNodes.add(curAbstractNode);

				Vector<ReportObject> curNodeReportObjects = manager
						.createReportObjects(curAbstractNode.outgoingEdges
								.get(0));
				reportObjects.addAll(curNodeReportObjects);
				latexFile = latexFile
						+ "\n\n"
						+ curAbstractNode
								.getLatexRepresentation(curNodeReportObjects);
				scriptFile = scriptFile
						+ "\n\n"
						+ curAbstractNode
								.getScriptForLatexRepresentation(curNodeReportObjects);

			} else if (curAbstractNode instanceof OutputFileNode) {
				// Ignore output file nodes
			} else {
				unvisitedAbstractNodes.add(curAbstractNode);
			}

		}

		int totalJobNodes = unvisitedAbstractNodes.size();
		statusDialog.setProgressBarMaximum(totalJobNodes);
		
		// Iterate jobs
		while (unvisitedAbstractNodes.size() > 0) {
			/*
			 * System.out.println("Done " + (totalJobNodes -
			 * unvisitedAbstractNodes.size() + 1) + "/" + totalJobNodes);
			 */

			statusDialog.setProgressBarValue(totalJobNodes
					- unvisitedAbstractNodes.size() + 1);

			GraphNode curAbstractNode = this
					.findAbstractNodeWithAllPredecessorsVisited(
							visitedAbstractNodes, unvisitedAbstractNodes);
			visitedAbstractNodes.add(curAbstractNode);
			unvisitedAbstractNodes.remove(curAbstractNode);

			Vector<ReportObject> curNodeReportObjects = manager
					.createReportObjects(curAbstractNode.outgoingEdges.get(0));
			reportObjects.addAll(curNodeReportObjects);
			latexFile = latexFile
					+ "\n\n"
					+ curAbstractNode
							.getLatexRepresentation(curNodeReportObjects);
			scriptFile = scriptFile
					+ "\n\n"
					+ curAbstractNode
							.getScriptForLatexRepresentation(curNodeReportObjects);
		}

		// End document
		latexFile = latexFile + this.getLatexFooter();
		this.writeFile(exportFolder + File.separator + "Results"
				+ relativeTargetFileName + ".tex", latexFile, false);

		// Collect all control variables
		Vector<String> allControlVariables = new Vector<String>();
		Vector<String> controlVariablesDescriptions = new Vector<String>();

		for (ReportObject curReportObject : reportObjects) {
			String curControlVariable = curReportObject
					.getControlVariableName();
			if (allControlVariables.indexOf(curControlVariable) == -1) {
				allControlVariables.add(curControlVariable);
				controlVariablesDescriptions.add(curReportObject
						.getViewerDescription());
			}
		}

		// Create corresponding latex and bash variables
		String bashVariables = "";
		bashVariables = bashVariables
				+ "# The following variables control which representation of which file type\n";
		bashVariables = bashVariables
				+ "# will be generated and included in the report file. Set to 1 to include\n";
		bashVariables = bashVariables
				+ "# the corresponding type. The settings.tex file will be generated based\n";
		bashVariables = bashVariables + "# upon these settings.\n";

		String latexVariables = "# Generate settings.tex\n";
		latexVariables = latexVariables + "rm -rf Files/settings.tex\n";

		for (int curPos = 0; curPos < allControlVariables.size(); curPos++) {
			String curControlVariable = allControlVariables.elementAt(curPos);
			bashVariables = bashVariables + curControlVariable + "=1 # "
					+ controlVariablesDescriptions.elementAt(curPos) + "\n";
			latexVariables = latexVariables + "if [ \"$" + curControlVariable
					+ "\" == 1 ] ; then echo \"\\\\providecommand{\\\\"
					+ curControlVariable + "}{1} % "
					+ controlVariablesDescriptions.elementAt(curPos)
					+ "\" >> \"Files/settings.tex\"; fi\n";
		}

		// Put date time into settings.tex
		String dateTime = "";
		dateTime = dateTime + "dateResult=`date`\n";
		dateTime = dateTime
				+ "echo \"\\\\providecommand{\\\\dateTime}{\"$dateResult}\"\" >> \"Files/settings.tex\"\n";

		// Create and write final script
		Vector<String> requirements = new Vector<String>();
		requirements.add("pdflatex");
		scriptFile = this.createScriptInitialization(
				this.locationOfWorkflowScipt, requirements, true, true, false)
				+ "\n\n"
				+ bashVariables
				+ "\n\n"
				+ latexVariables
				+ "\n"
				+ dateTime + "\n\n" + scriptFile + "\n\n";

		// PDF latex
		scriptFile = scriptFile + "# Create PDF file from Latex.\n";
		scriptFile = scriptFile + "if [ `command -v pdflatex` ]; then\n";
				
		scriptFile = scriptFile + "cd ${RESULTDIR}" + "/" + "Report" + "\n";
		scriptFile = scriptFile + "pdflatex " + "${RESULTDIR}"
				+ relativeTargetFileName + ".tex\n";
		scriptFile = scriptFile + "makeindex " + "${RESULTDIR}"
				+ relativeTargetFileName + ".tex\n";
		scriptFile = scriptFile + "pdflatex " + "${RESULTDIR}"
				+ relativeTargetFileName + ".tex\n";
		scriptFile = scriptFile + "pdflatex " + "${RESULTDIR}"
				+ relativeTargetFileName + ".tex\n";
		scriptFile = scriptFile + "grep -i warnin " + "${RESULTDIR}"
				+ relativeTargetFileName + ".log\n";
		scriptFile = scriptFile + "rm -rf report.aux\n";
		scriptFile = scriptFile + "rm -rf report.lof\n";
		scriptFile = scriptFile + "rm -rf report.toc\n";

		scriptFile = scriptFile + "else\n";
		scriptFile = scriptFile + "\techo \"Error! pdflatex not found!\"\n";		
		scriptFile = scriptFile + "fi\n";
		
		
		
		this.writeFile(exportFolder + File.separator + "Results"
				+ relativeTargetFileName + ".sh", scriptFile, true);
		
		statusDialog.setVisibleAll(false);
		statusDialog.disposeAll();
		statusDialog = null;
	}

	/**
	 * This method creates the script to create a report about of the workflow.
	 */
	public void createReport(final WorkflowPanel workflowPanel) throws IOException {
		final String exportFolder = this.locationOfWorkflowScipt;
		if (exportFolder.equals("")) {
			JOptionPane
					.showMessageDialog(
							workflowPanel,
							"The current workflow has not been exported yet.\nPlease export and execute the workflow fist.",
							"Error", JOptionPane.ERROR_MESSAGE);
		} else {
			// TODO Ensure display of all types?

			Thread createReportThread = new Thread() {
				public void run() {
					createReport(exportFolder, workflowPanel);
				}
			};
			createReportThread.start();

		}

	}

	/**
	 * This method creates the script which carry out the workflow. This script
	 * executes the tools of the workflow.
	 * 
	 * @param requirements
	 */
	public void createWorkflowScript(String targetFileName,
			Vector<String> requirements, boolean callReport) throws IOException {
		String resultHead = this.createScriptInitialization(new TypedFile(
				targetFileName).getParent(), requirements, true, true, true);

		String resultTail = "";
		String inputVisited = "";
		String outputVisited = "";

		// Create Vectors
		Vector<GraphNode> visitedAbstractNodes = new Vector<GraphNode>();
		Vector<GraphNode> unvisitedAbstractNodes = new Vector<GraphNode>();

		// First, find FileAbstractNodes and create their calls and
		// the beginning (and the end) of the batch
		resultHead = resultHead + "# ----- Convert non-XML files ----- \n";
		this.sortNodes();
		for (GraphNode curAbstractNode : this.nodes) {
			if (curAbstractNode instanceof InputFileNode) {

				visitedAbstractNodes.add(curAbstractNode);

				resultHead = resultHead
						+ "\n"
						+ curAbstractNode.getCommandLineCall("${SOFTWAREDIR}/",
								"${RESULTDIR}/", true);
				inputVisited = inputVisited + curAbstractNode.getName();
			} else if (curAbstractNode instanceof OutputFileNode) {

				visitedAbstractNodes.add(curAbstractNode);

				resultTail = resultTail
						+ "\n"
						+ curAbstractNode.getCommandLineCall("${SOFTWAREDIR}/",
								"${RESULTDIR}/", true);
				outputVisited = outputVisited + curAbstractNode.getName();
			} else {
				unvisitedAbstractNodes.add(curAbstractNode);
			}
		}

		resultHead = resultHead + "\n";
		resultHead = resultHead + "\n";
		resultHead = resultHead + "# ----- Execute Jobs ----- \n";

		// Then create calls for all normal jobs
		while (unvisitedAbstractNodes.size() > 0) {
			GraphNode curAbstractNode = this
					.findAbstractNodeWithAllPredecessorsVisited(
							visitedAbstractNodes, unvisitedAbstractNodes);
			visitedAbstractNodes.add(curAbstractNode);
			unvisitedAbstractNodes.remove(curAbstractNode);
			resultHead = resultHead
					+ curAbstractNode.getCommandLineCall("${SOFTWAREDIR}/",
							"${RESULTDIR}/", true) + "\n";
		}

		resultHead = resultHead + "\n";
		resultHead = resultHead + "\n";
		resultHead = resultHead
				+ "# ----- Convert XML-Files to external outputs ----- \n";

		// Create, display and output final batch
		String result = resultHead + resultTail + "\n";

		// Call report
		if (callReport) {
			result = result + "\n";
			result = result + "# ----- Create report ----- \n";
			result = result + "echo \"Creating report...\"\n";
			result = result + "cd ${RESULTDIR}/Report/\n";
			result = result + "${RESULTDIR}/Report/report.sh\n";
		}

		result = result + "echo \"Done...\"\n";
		this.writeFile(targetFileName, result, true);
	}

	/**
	 * This Method is needed to save Files to the FileSystem.
	 */
	public void writeFile(String targetFileName, String content,
			boolean setExecutable) {
		try {
			File targetFile = new File(targetFileName);
			FileWriter writer = new FileWriter(targetFile);
			BufferedWriter buffer = new BufferedWriter(writer);
			buffer.write(content);
			buffer.close();
			if (setExecutable) {
				targetFile.setExecutable(true);
			}
		} catch (Exception e) {
			JOptionPane.showMessageDialog(null, "The file " + targetFileName
					+ " could not be saved.", "Error",
					JOptionPane.ERROR_MESSAGE);
		}
	}

	/**
	 * Sorts this graphs node according to their position value.
	 */
	private void sortNodes() {
		Vector<GraphNode> newNodes = new Vector<GraphNode>();
		while (this.nodes.size() > 0) {
			GraphNode minNode = null;
			for (GraphNode curNode : this.nodes) {
				if (minNode == null
						|| curNode.getOrderPosition() < minNode
								.getOrderPosition()) {
					minNode = curNode;
				}
			}

			this.nodes.remove(minNode);
			newNodes.add(minNode);
		}

		this.nodes = newNodes;
	}

	/**
	 * This method generates the batch header used when creating scripts.
	 */
	private String createScriptHeader(boolean exitOnError) {
		String result = "#!/bin/sh\n";
		result = result + "# -------------------------------\n";
		result = result + "#   Script for workflow: " + this.name + "\n";
		result = result + "# -------------------------------\n";
		result = result + "\n";
		result = result + "# This workflow was created with " + AboutFrame.NAME
				+ " " + AboutFrame.PROFIDO_VERSION + "\n";

		result = result + "\n#Helper function to check if a file exists.\n";
		result = result + "function checkIfFileExists {\n";
		result = result + "\tif [ ! -f $1 ]; then\n";
		result = result + "\techo \"Error! The file ${1} does not exist!\"\n";
		if (exitOnError)
			result = result + "\texit\n";
		result = result + "\tfi\n";
		result = result + "}\n\n";

		return result;
	}

	/**
	 * This method determines the unvisited nodes of which all predecessor were
	 * visited. This method is called by createWorkflowScript to create the
	 * calls for the job nodes.
	 */
	private GraphNode findAbstractNodeWithAllPredecessorsVisited(
			Vector<GraphNode> visitedAbstractNodes,
			Vector<GraphNode> unvisitedAbstractNodes) {
		if (visitedAbstractNodes.size() == 0)
			return unvisitedAbstractNodes.get(0);
		else {
			for (GraphNode curAbstractNode : unvisitedAbstractNodes) {
				if (curAbstractNode
						.allPredecessorsVisited(visitedAbstractNodes))
					return curAbstractNode;
			}

		}
		return null;
	}

	/**
	 * Add a new action to the history.
	 */
	public void addNewAction(HistoryObject newAction) {
		this.history.addNewAction(newAction);
	}

	/**
	 * Perform the last action added to the histroy.
	 */
	public void performLastAction() {
		this.saved = false;
		this.history.performLastAction();
	}

	public String getHistoryText() {
		return this.history.toString();
	}

	/**
	 * Undo the last change applied to the graph.
	 */
	public void undo() {
		this.history.undo(1);
	}

	/**
	 * Redo the last change applied to the graph.
	 */
	public void redo() {
		this.history.redo(1);
	}

	public boolean isSaved() {
		return this.saved;
	}

	public void setSaved(boolean saved) {
		this.saved = saved;
	}

	public void clearHistory() {
		this.history.clear();
	}

	/**
	 * This method creates a string used to store the graph in an XML file and
	 * returns it. The elements of the workflow are saved in the string step by
	 * step in the following order input files, output files, job nodes and
	 * finally edges.
	 */
	public String getXMLRepresentationOfGraph() {
		String savefile = "";
		savefile += "<?xml version= \"1.0\" ?>" + "\n";
		savefile += "<workflow>" + "\n";
		savefile += "\t<general>" + "\n";
		savefile += "\t\t<name> " + this.getName() + " </name>" + "\n";
		savefile += "\t\t<defaultdir> " + this.getWorkingDirectory()
				+ " </defaultdir>" + "\n";
		savefile += "\t\t<exportdir> " + this.getExpFolder() + " </exportdir>"
				+ "\n";
		savefile += "\t\t<settings> " + Graph.DISPLAY_NODE_INFORMATION + " "
				+ Graph.DISPLAY_EDGE_INFORMATION + " "
				+ Graph.ALWAYS_DISPLAY_ORDER_KEYS + " "
				+ Graph.DISPLAY_ALL_NODE_INFORMATION + " "
				+ Graph.DISPLAY_ALL_EDGE_INFORMATION + " </settings>" + "\n";
		savefile += "\t\t<version> " + AboutFrame.PROFIDO_VERSION
				+ " </version>" + "\n";
		savefile += "\t</general>" + "\n";
		savefile += "\t<nodes>" + "\n";

		for (GraphNode aN : this.nodes) {
			if (aN instanceof InputFileNode) {
				savefile += aN.getXMLRepresentationOfNode(this.nodes
						.indexOf(aN)) + "\n";
			}
		}
		for (GraphNode aN : this.nodes) {
			if (aN instanceof OutputFileNode) {
				savefile += aN.getXMLRepresentationOfNode(this.nodes
						.indexOf(aN)) + "\n";
			}
		}
		for (GraphNode aN : this.nodes) {
			if (aN instanceof JobNode) {
				savefile += aN.getXMLRepresentationOfNode(this.nodes
						.indexOf(aN)) + "\n";
			}
		}
		savefile += "\t</nodes>" + "\n";

		if (this.edges.size() > 0) {

			savefile += "\t<edges>" + "\n";
			for (Edge e : this.edges) {
				GraphNode ef = e.getFrom();
				GraphNode et = e.getTo();
				int a = this.nodes.indexOf(ef);
				int b = this.nodes.indexOf(et);

				savefile += e.getXMLRepresentationOfEdge(a, b) + "\n";
			}
			savefile += "\t</edges>" + "\n";

		}
		savefile += "</workflow>" + "\n";

		return savefile;
	}

	public History getHistory() {
		return this.history;
	}

	/**
	 * This method checks if the workflow is build correctly. For a correct
	 * workflow every node has to be connected and at least one input and output
	 * node have to be present. If any error is detected, a detailed message is
	 * returned.
	 * 
	 * An input node has to have at least one outgoing edge. An output node has
	 * to have at least one incoming edge. A job node has to have at least one
	 * incoming and one outgoing edge.
	 */
	public String checkBuild() {
		String result = "";
		boolean outputNodePresent = false;
		boolean inputNodePresent = false;

		for (GraphNode v : this.nodes) {
			if (v instanceof InputFileNode) {
				inputNodePresent = true;
				if (v.getOutgoingEdges().isEmpty()) {
					result = result + "Input node \"" + v.getName()
							+ "\" has to have at least one outgoing edge.\n";
				}
			}

			else if (v instanceof OutputFileNode) {
				outputNodePresent = true;
				if (v.getIncomingEdges().isEmpty()) {
					result = result + "Output node \"" + v.getName()
							+ "\" has to have at least one incoming edge.\n";
				}
			}

			else if (v instanceof JobNode) {
				if (v.getIncomingEdges().isEmpty()) {
					result = result + "Job node \"" + v.getName()
							+ "\" has to have at least one incoming edge.\n";
				}
				if (v.getOutgoingEdges().isEmpty()) {
					result = result + "Job node \"" + v.getName()
							+ "\" has to have at least one outgoing edge.\n";
				}
			}

		}

		if (!inputNodePresent) {
			result = result + "The workflow needs at least one input node.\n";
		}

		if (!outputNodePresent) {
			result = result + "The workflow needs at least one output node.\n";
		}

		return result;
	}

	/**
	 * This method checks if the graph contains a cycle and returns a detailed
	 * message containing further information if any is found.
	 * 
	 * @return
	 */
	public String checkForCycle() {
		String result = "";

		for (GraphNode curNodeToCheck : this.nodes) {
			Vector<GraphNode> markedNodes = new Vector<GraphNode>();
			Vector<GraphNode> nodesToVisit = new Vector<GraphNode>();
			nodesToVisit.add(curNodeToCheck);
			boolean cycle = false;

			while (nodesToVisit.size() > 0 && !cycle) {
				GraphNode curNode = nodesToVisit.remove(0);
				markedNodes.add(curNode);
				for (Edge curEdge : curNode.getOutgoingEdges()) {
					GraphNode nextNode = curEdge.getTo();

					if (nextNode == curNodeToCheck) {
						cycle = true;
						result = result + "\"" + curNodeToCheck.getName()
								+ "\", ";
					} else if (markedNodes.indexOf(nextNode) == -1) {
						nodesToVisit.add(nextNode);
					}
				}
			}
		}

		if (!result.equals("")) {
			result = "The nodes " + result.substring(0, result.length() - 2)
					+ " are part of a cycle.\n";
		}

		return result;
	}

	/**
	 * Returns a bounding box completely enclosing this graph.
	 * 
	 * @param grid
	 */
	public Rectangle getBoundingBox(int cellWidth, int cellHeight) {
		int ulX = -1;
		int ulY = -1;
		int brX = -1;
		int brY = -1;

		for (GraphNode curNode : this.nodes) {
			int curX = curNode.getX();
			int curY = curNode.getY();
			if (ulX == -1 || curX < ulX) {
				ulX = curX;
			}
			if (ulY == -1 || curY < ulY) {
				ulY = curY;
			}
			if (brX == -1 || curX > brX) {
				brX = curX;
			}
			if (brY == -1 || curY > brY) {
				brY = curY;
			}
		}

		return new Rectangle(ulX - cellWidth / 2, ulY - cellHeight / 2, brX
				- ulX + cellWidth, brY - ulY + cellHeight);

		// return new Rectangle(ulX - space.width / 2, ulY - space.height / 2,
		// brX - ulX + space.width, brY - ulY + space.height);
	}

	public Vector<Point> getJobNodePositions() {
		Vector<Point> result = new Vector<Point>();
		for (BasicNode curNode : this.nodes) {
			result.add(curNode.getPosition());
		}
		return result;
	}

	public Vector<Point> getEdgePositions() {
		Vector<Point> result = new Vector<Point>();
		for (Edge curEdge : this.edges) {
			result.add(curEdge.getLastPoint());
		}
		return result;
	}

	public boolean hadNodes() {
		return this.nodes.size() > 0;
	}

	public boolean hasEdges() {
		return this.edges.size() > 0;
	}

	/**
	 * This method checks if the passed file name is already used as the output
	 * of any node within the graph.
	 */
	public GraphNode getNodeWithSameFileName(String fileName) {
		String nameToCheck = new File(fileName).getName();

		for (GraphNode curNode : this.nodes) {
			TypedFile curNodeOutputFile = curNode.getXMLOutputFile();
			if (curNodeOutputFile != null) {
				String curFile = curNodeOutputFile.getName();
				if (nameToCheck.equals(curFile))
					return curNode;
			}
		}

		return null;
	}

	/**
	 * This Method creates a Screenshot of the current Graph and saves it as an
	 * png file.
	 */
	public void saveScreenshot(String fileName, Grid grid) {
		BufferedImage image = new BufferedImage(grid.getWidth(),
				grid.getHeight(), BufferedImage.TYPE_INT_RGB);
		Graphics2D g = image.createGraphics();

		g.setFont(WorkflowPanel.FONT);

		grid.draw(g);
		this.draw(g, grid);

		File outputfile = new File(fileName);
		try {
			ImageIO.write(image, "png", outputfile);
		} catch (IOException e) {
			e.printStackTrace();
		}

	}

	public String getLocationOfWorkflowScript() {
		return this.locationOfWorkflowScipt;
	}

	/**
	 * This method returns the current graph as tikz definition.
	 * 
	 * @return The string that contains the tikz code.
	 * @throws IOException
	 */
	public String createTiKzGraphic() {
		String result = "";

		result = result + this.getEdgeNodeTiKzDefinition();
		result = result + this.getInOutNodeTiKzDefinition();
		result = result + this.getJobNodeTiKzDefinition();
		result = result + this.getEdgeConnectionTiKzDefinition();

		result = result + "\\begin{figure}[htp]\n";

		result = result + "\\begin{center}\n";
		result = result + "\\resizebox{0.8\\textwidth}{!}{\n";
		result = result + "\\begin{tikzpicture}[]\n";

		// Get nodes
		for (GraphNode curNode : this.nodes) {
			result = result + curNode.getTikzRepresentationOfNode();
		}

		// Get point nodes
		for (Edge curEdge : this.edges) {
			result = result + curEdge.getTikzRepresentationOfPointNodes();
		}

		result = result + "\\begin{pgfonlayer}{bg}\n";

		// Get edges
		for (Edge curEdge : this.edges) {
			result = result + curEdge.getTikzRepresentation();
		}

		result = result + "\\end{pgfonlayer}\n";

		result = result + "\\end{tikzpicture}\n";
		result = result + "}\n";
		result = result + "\\caption{Workflow " + this.getName() + "}\n";
		result = result + "\\end{center}\n";
		result = result + "\\end{figure}\n";

		return result;
	}

	/**
	 * This method returns the needed preamble for the tikz graphic
	 * 
	 * @return The string that contains the preamble
	 */
	/*
	 * public String createTikZPreamble() { String result = ""; result = result
	 * + "\\documentclass[a4paper]{scrreprt} \n"; result = result +
	 * "\\usepackage[latin1]{inputenc}\n"; result = result +
	 * "\\usepackage[T1]{fontenc}\n"; result = result +
	 * "\\usepackage{amsmath}\n"; result = result + "\\usepackage{amssymb}\n";
	 * result = result + "\\usepackage{amsthm}\n"; result = result +
	 * "\\usepackage{textcomp} \n"; result = result + "\\usepackage{tikz}\n";
	 * result = result + "\\usetikzlibrary{shapes}\n"; result = result +
	 * "\\usetikzlibrary{arrows,backgrounds,automata}\n"; result = result +
	 * "\\usetikzlibrary{fit}\n"; result = result + "\\pgfdeclarelayer{bg}\n";
	 * result = result + "\\pgfsetlayers{bg,main}\n";
	 * 
	 * 
	 * return result; }
	 */

	/**
	 * This method gives you the definition for a EdgeNode in tikz.
	 * 
	 * @return The string that represents the definition.
	 */
	public String getEdgeNodeTiKzDefinition() {

		return "\\tikzstyle{edgenode}=[draw,thin,fill=green!100,circle,scale=0.46]\n";

	}

	/**
	 * This method gives you the definition for In/OutputNodes in TikZ
	 * 
	 * @return The string that represents the definition.
	 */
	public String getInOutNodeTiKzDefinition() {

		return "\\tikzstyle{inoutnode}=[draw,semithick,fill=white!100,circle,text centered,inner sep=3pt,minimum size=8pt]\n";
	}

	/**
	 * This method gives you the definition for JobNodes in Tikz
	 * 
	 * @return The string that represents the definition.
	 */
	public String getJobNodeTiKzDefinition() {

		return "\\tikzstyle{jobnode}=[draw,semithick,fill=white!100,rounded corners,text centered,inner sep=3pt, minimum width=90pt,minimum height=40pt,text width=60pt, auto]\n";

	}

	/**
	 * This method gives you the definition for the edge style in TiKz
	 * 
	 * @return The string that represents the definition.
	 */
	public String getEdgeConnectionTiKzDefinition() {

		String result = "";
		result = result
				+ "\\tikzstyle{to1}=[-triangle 60,fill=red,shorten >=0.1pt,thin]\n";
		result = result + "\\tikzstyle{to2}=[thin]\n";
		result = result + "\\newlength\\mysize\n";
		result = result + "\\pgfarrowsdeclare{myarrow}{myarrow}{%\n";
		result = result + "\\mysize=11pt\n";
		result = result + "\\advance\\mysize by 0.5\\pgflinewidth\n";
		result = result + "\\pgfarrowsleftextend{\\mysize}\n";
		result = result + "\\pgfarrowsrightextend{+0.5\\pgflinewidth}\n";
		result = result + "}{%\n";
		result = result + "\\mysize=11pt\n";
		result = result + "\\advance\\mysize by 0.5\\pgflinewidth\n";
		result = result + "\\pgfsetdash{}{0pt}\n";
		result = result + "\\pgfsetroundjoin\n";
		result = result + "\\pgfsetroundcap\n";
		result = result + "\\pgfpathmoveto{\\pgfpointorigin}\n";
		result = result
				+ "\\pgfpathlineto{\\pgfqpoint{+-\\mysize}{+0.60\\mysize}}\n";
		result = result
				+ "\\pgfpathlineto{\\pgfqpoint{+-\\mysize}{+-0.60\\mysize}}\n";
		result = result + "\\pgfpathclose\n";
		result = result + "\\pgfusepathqfillstroke\n";
		result = result + "}\n";
		return result;
	}

}
