/*
 * 
 * ProFiDo - The Processes Fitting Toolkit Dortmund
 * Copyright (C) 2010 - 2012 LS Informatik IV, TU Dortmund
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

package core.graph;

import gui.helper.Grid;
import gui.helper.ReportObject;
import gui.helper.TextManager;
import helper.SmallHelpers;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Point;
import java.awt.Rectangle;
import java.io.Serializable;
import java.util.HashMap;
import java.util.Map;
import java.util.Vector;

import core.TypedFile;
import core.storage.ConfigurationXMLReader;

/**
 * This abstract class defines the properties of nodes as used in the workflow
 * graph. The subclasses of this class represent the specific node types such as
 * file and job node.
 * 
 * It stores and controls most of the information relevant for managing the
 * node within the graph and for drawing it on the canvas.
 * 
 * Each node has a position within the canvas and a set of incoming and outgoing
 * edges.
 * 
 */

public abstract class GraphNode extends BasicNode implements Serializable {
	private static final long serialVersionUID = 1L;
	protected static final Color DEFAULT_COLOR = Color.BLACK;
	protected static final int BORDER_SIZE = 1;
	private static final int DEFAULT_ORDER_KEY = 10;

	public static final int OUTPUT_FORMAT_HTML = 0;
	public static final int OUTPUT_FORMAT_TEX = 1;

	private static int nextFreeOrderPosition = 1;

	/**
	 * Name of this node.
	 */
	protected String name;

	/**
	 * ID explicitly identifying this nodes assigned tool.
	 */
	protected int referenceID;

	/**
	 * Color of this node used when drawing it on the canvas.
	 */
	protected Color color;

	/**
	 * Storage for the edges connected to this node
	 */
	protected Vector<Edge> incomingEdges, outgoingEdges;

	/**
	 * Indicates if this node is currently edited in order to draw it in a
	 * different color.
	 */
	protected boolean currentlyEdited = false;

	/**
	 * Reference to the XML reader used for reading the configurations.
	 */
	protected ConfigurationXMLReader reader;

	/**
	 * File representing the xml output of this node.
	 */
	protected TypedFile xmlOutputFile;

	/**
	 * Position of this node within the graphs node list.
	 */
	private int positionInGraphNodes;

	/**
	 * Storage for the comment.
	 */
	protected String comment;
	
	/**
	 * Indicates if the comment for the GraphNode is visible.
	 */
	protected boolean commentVisibilty;

	/**
	 * Generate and return an information text of this node to display it in the
	 * workflow.
	 */
	protected abstract String getCaptionText(boolean ignoreVisibilty,
			boolean markEnabled, boolean sortEnabled, boolean showTypes,
			boolean hideComment);

	@Override
	public String getTooltipText() {
		String result = "<html>";
		result = result + this.getCaptionText(true, true, true, true, false);
		return result + "</html>";
	}

	/**
	 * Returns a vector of string containing all types this node in its current
	 * state can accept as input.
	 */
	public abstract Vector<String> getInputFileTypes();

	/**
	 * Returns a vector of string containing all types this node in its current
	 * state will return as output.
	 */
	public abstract Vector<String> getOutputFileTypes();

	/**
	 * This method generates and returns the command line call represented by
	 * this node.
	 * 
	 * @param softwarePath
	 *            Path where the software binaries are located.
	 * @param resultPath
	 *            Path where the output of this node should be stored.
	 */
	public abstract String getCommandLineCall(String softwarePath,
			String resultPath, boolean comments);

	/**
	 * Calculates and returns all intersections of the passed edge with the
	 * shape of this node.
	 */
	public abstract Point calculateIntersection(Edge pEdge);

	/**
	 * Creates the node according to the passed values.
	 */
	public GraphNode(int pX, int pY, ConfigurationXMLReader reader, Grid pGrid,
			int id, Graph graph) {
		super(pX, pY, pGrid, graph);
		this.referenceID = id;
		this.reader = reader;
		this.incomingEdges = new Vector<Edge>();
		this.outgoingEdges = new Vector<Edge>();
		this.xmlOutputFile = null;
		this.color = GraphNode.DEFAULT_COLOR;
		this.positionInGraphNodes = GraphNode.nextFreeOrderPosition;
		this.comment = "";
		GraphNode.nextFreeOrderPosition++;
	}

	public int getOrderPosition() {
		return this.positionInGraphNodes;
	}

	/**
	 * Creates the node according to the passed values.
	 */
	public GraphNode(int pX, int pY) {
		super(pX, pY, null, null);
	}

	public void setColor(Color pColor) {
		this.color = pColor;
	}

	public Color getColor() {
		return this.color;
	}

	/**
	 * Adds the passed edges to the vector storing the incoming edges and
	 * triggers the resorting this nodes incoming edges.
	 */
	public void addIncomingEdge(Edge k) {
		this.incomingEdges.add(k);
		this.sortIncomingEdgesAccordingToOrderKey();
	}

	/**
	 * Adds the passed edges to the vector storing the outgoing edges.
	 */
	public void addOutgoingEdge(Edge k) {
		this.outgoingEdges.add(k);
	}

	/**
	 * Removes the passed edge from either edge vectors of this node.
	 */
	public void removeEdge(Edge k) {
		this.incomingEdges.remove(k);
		this.outgoingEdges.remove(k);
	}

	/**
	 * Removes all edges from the edge vectors of this node.
	 */
	public void removeAllEdges() {
		this.incomingEdges = new Vector<Edge>();
		this.outgoingEdges = new Vector<Edge>();
	}

	public Vector<Edge> getIncomingEdges() {
		return this.incomingEdges;
	}

	public Vector<Edge> getOutgoingEdges() {
		return this.outgoingEdges;
	}

	/**
	 * This method determines the point within the passed vector of points which
	 * has the smallest distance to the additional passed location and returns
	 * it.
	 */
	protected Point getClosestIntersection(Vector<Point> points, double px,
			double py) {
		// TODO Improve calculations
		Point closestIntersection = null;
		double closestDist = 99999;
		double curDist;
		for (Point curIntersection : points) {
			if (curIntersection != null) {
				curDist = curIntersection.distance(px, py);
				if (curDist < closestDist) {
					closestDist = curDist;
					closestIntersection = curIntersection;
				}
			}
		}
		return closestIntersection;
	}

	public void setCurrentlyEdited(boolean b) {
		this.currentlyEdited = b;
	}

	public int getReferenceID() {
		return this.referenceID;
	}

	public void setName(String newName) {
		this.name = newName;
	}

	public void setComment(String comment) {
		this.comment = comment;
	}

	public void setCommentVisibilty(boolean newValue) {
		this.commentVisibilty = newValue;
	}

	public TypedFile getXMLOutputFile() {
		return this.xmlOutputFile;
	}

	/**
	 * This method checks if any of this nodes predecessors are not part of the
	 * passed vector.
	 */
	public boolean allPredecessorsVisited(Vector<GraphNode> visitedAbstractNodes) {
		boolean allVisited = true;
		for (Edge curEdge : this.incomingEdges) {
			if (visitedAbstractNodes.indexOf(curEdge.getFrom()) == -1) {
				allVisited = false;
				break;
			}
		}

		return allVisited;
	}

	@Override
	public void moveToPixelCoordinates(int pX, int pY, boolean pDoRealign) {
		if (pDoRealign) {
			Point gridPosition = this.grid.getNearestCell(pX, pY);
			this.gridX = gridPosition.x;
			this.gridY = gridPosition.y;
			this.realignToGrid();
		} else {
			this.x = pX;
			this.y = pY;
		}

		// Notify edges that this node has moved in order to realign PointNodes
		// on the edges
		if (this.outgoingEdges != null) {
			for (Edge curEdge : this.outgoingEdges) {
				curEdge.notifiyThatNodeHasMoved();
			}
		}

		if (this.incomingEdges != null) {
			for (Edge curEdge : this.incomingEdges) {
				curEdge.notifiyThatNodeHasMoved();
			}
		}
	}

	@Override
	public void moveToGridCoordinates(Point newPosition) {
		this.gridX = newPosition.x;
		this.gridY = newPosition.y;
		this.realignToGrid();

		if (this.outgoingEdges != null) {
			for (Edge curEdge : this.outgoingEdges) {
				curEdge.notifiyThatNodeHasMoved();
			}
		}

		if (this.incomingEdges != null) {
			for (Edge curEdge : this.incomingEdges) {
				curEdge.notifiyThatNodeHasMoved();
			}
		}
	}

	/**
	 * This method draws the outline of this node.
	 * 
	 * @param pGraphics
	 *            Graphics object to draw the outline on.
	 * @param pShape
	 *            The bounding box of this node. Does not determine the exact
	 *            outline of this node, but its outer boundaries. The exact
	 *            outline of the node determined within the particular
	 *            implementation within the subclasses.
	 */
	protected abstract void drawOutline(Graphics pGraphics, Rectangle pShape);

	@Override
	public void draw(Graphics pGraphics) {
		// Set color according to the state of this node
		if (this.currentlyEdited) {
			pGraphics.setColor(Color.red);
		} else {
			pGraphics.setColor(this.color);
		}

		// Fill outer shape of node in nodes color
		Rectangle outterShape = this.getBoundingBox();
		this.drawOutline(pGraphics, outterShape);

		// Clear inner shape of node
		int borderSize = GraphNode.BORDER_SIZE;
		if (this.marked) {
			// If marked, draw border in bold
			borderSize = 2 * borderSize;
		}

		Rectangle innerShape = new Rectangle(outterShape.x + borderSize,
				outterShape.y + borderSize, outterShape.width - 2 * borderSize,
				outterShape.height - 2 * borderSize);
		pGraphics.setColor(Color.white);
		this.drawOutline(pGraphics, innerShape);

		// Write name inside the node
		pGraphics.setColor(this.getColor());

		String toolTip = this.getName();
		String displayName = toolTip;

		// Cut name if to long
		if (displayName.length() > 8) {
			displayName = displayName.substring(0, 7);
			displayName += "...";
		}

		TextManager.writeLargeText(displayName, (int) outterShape.getCenterX(),
				(int) outterShape.getCenterY(), pGraphics, true);

		// Write further information (parameters, etc.) above the node
		if (Graph.DISPLAY_ALL_NODE_INFORMATION
				|| Graph.DISPLAY_NODE_INFORMATION) {
			String tooltipLines = this.getCaptionText(
					Graph.DISPLAY_ALL_NODE_INFORMATION,
					Graph.DISPLAY_ALL_NODE_INFORMATION,
					Graph.DISPLAY_ALL_NODE_INFORMATION, false, false);
			TextManager
					.writeSmallText(tooltipLines, (int) (outterShape
							.getCenterX() - outterShape.getWidth() / 2) + 7,
							(int) (this.y - outterShape.getHeight() / 2) - 7,
							pGraphics, false);
		}
	}

	/**
	 * This method intersects this nodes output types with the input types of
	 * the passed node and returns a vector containing all common types.
	 */
	public Vector<String> intersectOutputTypesWithNodesInputTypes(GraphNode to) {
		return SmallHelpers.intersect(this.getOutputFileTypes(),
				to.getInputFileTypes());
	}

	public void setXMLFileName(String newName) {
		Vector<String> types = this.xmlOutputFile.getTypes();
		this.xmlOutputFile = new TypedFile(newName, types);
	}

	/**
	 * Returns the next free order key.
	 */
	public String getNextFreeOrderKey() {
		String maxKey = "";
		String result = Integer.toString(GraphNode.DEFAULT_ORDER_KEY);

		// Find maximum key on edge
		for (Edge curEdge : this.incomingEdges)
			if (maxKey == "" || curEdge.getOrderKey().compareTo(maxKey) > 0) {
				maxKey = curEdge.getOrderKey();
			}

		if (maxKey.length() > 0) {
			// Determine next key according to the nature of the maxkey
			try {
				// If number increment by default value
				int maxKeyValue = Integer.valueOf(maxKey);
				result = Integer.toString(maxKeyValue
						+ GraphNode.DEFAULT_ORDER_KEY);
			} catch (NumberFormatException e) {
				// If no number, try to increment last char
				char lastChar = maxKey.charAt(maxKey.length() - 1);
				char newLastChar = (char) (lastChar + 1);
				if (Character.isLetter(newLastChar)) {
					String oldPrefix = "";
					if (maxKey.length() > 1) {
						oldPrefix = maxKey.substring(0, maxKey.length() - 1);
					}
					result = oldPrefix + newLastChar;
				} else {
					result = maxKey + "a";
				}
			}
		}
		return result;
	}

	/**
	 * Sorts this nodes incoming edges according to their oder key.
	 */
	public void sortIncomingEdgesAccordingToOrderKey() {
		Vector<Edge> newIncomingEdges = new Vector<Edge>();
		while (this.incomingEdges.size() > 0) {
			// Find minimum
			Edge minKeyEdge = null;
			for (Edge curEdge : this.incomingEdges)
				if (minKeyEdge == null
						|| curEdge.getOrderKey().compareTo(
								minKeyEdge.getOrderKey()) < 0) {
					minKeyEdge = curEdge;
				}

			// Remove minimum
			this.incomingEdges.remove(minKeyEdge);
			newIncomingEdges.add(minKeyEdge);
		}

		this.incomingEdges = newIncomingEdges;
	}

	public String getComment() {
		return this.comment;
	}

	/**
	 * This method generates and returns a string representing all information
	 * of this node needed to store this node in an xml-file.
	 */
	public String getXMLRepresentationOfNode(int id) {
		String xmlDescription = "";
		xmlDescription += "<node>\n";
		xmlDescription += "\t<id> " + id + " </id>\n";
		xmlDescription += "\t<name> " + this.getName() + " </name>\n";
		if (!this.comment.trim().equals("")) {
			if (this.commentVisibilty) {
				xmlDescription += "\t<comment visible=\"true\"> "
						+ this.getComment() + " </comment>\n";
			} else {
				xmlDescription += "\t<comment visible=\"false\"> "
						+ this.getComment() + " </comment>\n";
			}
		}
		xmlDescription += "\t<coords> " + this.getX() + " " + this.getY()
				+ " </coords>\n";
		xmlDescription += "\t<gridcoords> " + this.gridX + " " + this.gridY
				+ " </gridcoords>\n";
		return xmlDescription;
	}

	public boolean isCommentVisible() {
		return this.commentVisibilty;
	}

	/**
	 * This Method returns a LatexTable representation of a GraphNode.
	 */
	private String createLatexTable(String[] header, String content,
			String caption) {
		String result = "";

		result = result + "\\begin{table}[H]" + "\n";
		result = result + "\\begin{center}" + "\n";
		result = result + "\t\\begin{tabular}{ |";
		for (String element : header) {
			result = result + " l |";
		}
		result = result + "}" + "\n";
		result = result + "\t\t\\hline" + "\n";

		result = result + "\t\t";
		for (String element : header) {
			result = result + "\\textbf{" + element + "} & ";
		}
		result = result.substring(0, result.length() - 2) + " \\\\ \\hline \n";
		result = result + content;
		result = result + "\t\\end{tabular}" + "\n";
		result = result + "\\caption{" + caption + "}" + "\n";
		result = result + "\\end{center}\n";
		result = result + "\\end{table}\n";

		return result;
	}

	/**
	 * Returns a string describing this nodes parameters in text or latex
	 * form. The overall format is determined by the passed integer whilst
	 * further details can be specified by the boolean variables.
	 */
	protected String getParameterText(boolean ignoreVisibilty,
			boolean markEnabled, boolean sortEnabled, boolean inculdeDisabled,
			boolean showTypes, int outputFormat) {
		return "";
	}

	/*
	 * Deprecated, but maybe still usefull???
	 * 
	 * public String createSubFigureForReportObjects( Vector<ReportObject>
	 * reportObjects, String title) { int coloumCount = 2;
	 * 
	 * String result = "";
	 * 
	 * if (reportObjects.size() == 1) { // If only one elemnt, create figure
	 * directly
	 * 
	 * 
	 * 
	 * result = result + reportObjects.get(0).getLatexInclude("") + "\n"; } else
	 * { // Otherwise split into coloumns String coloumns[] = new
	 * String[coloumCount]; int curCol = 0;
	 * 
	 * String colWidth = String.valueOf((0.85 / coloumCount) * 100); colWidth =
	 * colWidth.substring(0, colWidth.indexOf("."));
	 * 
	 * while (reportObjects.size() > 0) { String curLine = coloumns[curCol %
	 * coloumCount]; if (curLine == null) curLine = "";
	 * 
	 * coloumns[curCol % coloumCount] = curLine +
	 * reportObjects.remove(0).getLatexInclude("\t\t\t"); curCol++; }
	 * 
	 * // Create figure environment result = result + "% Subfigure for " + title
	 * + "\n"; result = result + "\\begin{figure}[H]" + "\n"; result = result +
	 * "\t\\centering" + "\n";
	 * 
	 * for (int curColPos = 0; curColPos < coloumCount; curColPos++) { result =
	 * result + "\t\\begin{subfigure}[t]{." + colWidth + "\\textwidth}" + "\n";
	 * result = result + "\t\t\\centering" + "\n"; result = result +
	 * coloumns[curColPos]; result = result + "\t\\end{subfigure}" + "\n";
	 * result = result + "\t\\qquad" + "\n"; }
	 * 
	 * result = result + "\t\\caption{" + title + "}" + "\n"; result = result +
	 * "\\end{figure}" + "\n";
	 * 
	 * }
	 * 
	 * return result; }
	 */

	/**
	 * This method creates a latex minipage arrangement including
	 * all report objects passed in the reportObjects vector.
	 * The title for the entire minipage is determined by the
	 * title parameter.
	 */
	public String createMiniPagesForReportObjects(
			Vector<ReportObject> reportObjects, String title) {
		int miniPageCount = 2;

		String result = "% Output for Group " + title + "\n";

		if (reportObjects.size() == 1) {
			// If only one element, create figure directly
			// result = result + "\\resizebox{0.95\\textwidth}{!}{" + "\n";
			result = result + reportObjects.get(0).getLatexInclude("") + "\n";
			// result = result + "}" + "\n";
		} else {
			// Otherwise create minipages
			String pageWidth = String.valueOf(0.95 / miniPageCount * 100);
			pageWidth = pageWidth.substring(0, pageWidth.indexOf("."));

			String curMiniPages = "";
			while (reportObjects.size() > 0) {
				String curRow = "";
				// Get next mini pages
				for (int curMiniPage = 0; curMiniPage < miniPageCount
						&& reportObjects.size() > 0; curMiniPage++) {
					String curPage = "";
					// curPage = curPage + "\\resizebox{0." + pageWidth +
					// "\\textwidth}{!}{" + "\n";
					curPage = curPage + "\\begin{minipage}[t]{." + pageWidth
							+ "\\textwidth}" + "\n";
					curPage = curPage
							+ reportObjects.remove(0).getLatexInclude("\t\t\t");
					curPage = curPage + "\\end{minipage}" + "\n";
					// curPage = curPage + "}" + "\n";
					curPage = curPage + "\\hfill" + "\n";
					curRow = curRow + curPage;
				}
				curMiniPages = curMiniPages + "% Next row\n" + curRow;
			}

			result = result + curMiniPages + "\n";
		}

		return result;
	}

	/**
	 * Returns a latex formated textual description of this node containing
	 * all relevant information. All passed reportObjects objects are included
	 * within the description.
	 */
	public String getLatexRepresentation(Vector<ReportObject> reportObjects) {
		String result = "";

		result = result + "% -----------------------------\n";
		result = result + "% Report for Node " + this.getName() + "\n";
		result = result + "% -----------------------------\n";

		
		result = result + "\\section{Node " + SmallHelpers.escapeLatex(this.getName()) + "}\n";
		result = result + "% Add custom text describing this node here.\n";
		result = result + "\\subsection{Overview}\n\n";

		// Output comment
		if (!this.comment.trim().equals("")) {
			result = result + "% Comment for this node\n";
			result = result + this.comment.trim() + "\n\n";
		}

		// Output call
		result = result + "% Command line call\n";
		result = result + "\\begin{lstlisting}[caption=Command line call of "
				+ SmallHelpers.escapeLatex(this.getName()) + "]\n";
		result = result
				+ this.getCommandLineCall("${SOFTWAREDIR}/", "${RESULTDIR}/",
						false) + "\n";
		result = result + "\\end{lstlisting}\n\n";

		// Output Information
		result = result + "\\subsubsection{Input- and Output-Files}\n";
		result = result + "% Global information\n";

		String header[] = { "Name", "Value" };
		String content = "";
		for (Edge curInputEdge : this.incomingEdges) {
			content = content
					+ "\t\t\\textbf{Input File} & "
					+ SmallHelpers.escapeLatex(curInputEdge.getFileName()
							.getName())
					+ " (From: "
					+ SmallHelpers
							.escapeLatex(curInputEdge.getFrom().getName())
					+ ")\\\\ \\hline\n";
		}
		for (Edge curOutputEdge : this.outgoingEdges) {
			content = content
					+ "\t\t\\textbf{Output File} & "
					+ SmallHelpers.escapeLatex(curOutputEdge.getFileName()
							.getName()) + " (To: "
					+ SmallHelpers.escapeLatex(curOutputEdge.getTo().getName())
					+ ")\\\\ \\hline\n";
		}
		result = result
				+ this.createLatexTable(header, content,
						"Input- and Output-Files of " + SmallHelpers.escapeLatex(this.getName()))
				+ "\n\n";

		// Output parameters
		content = SmallHelpers.escapeLatex(this.getParameterText(true, true,
				true, false, true, GraphNode.OUTPUT_FORMAT_TEX));
		if (!content.equals("")) {
			result = result + "\\subsubsection{Parameter}\n";
			result = result + "% Parameter Settings\n";
			String header2[] = { "Name", "Value", "Description" };// ,
																	// "Enabled"};
			result = result
					+ this.createLatexTable(header2, content, "Parameter of "
							+ SmallHelpers.escapeLatex(this.getName())) + "\n\n";
		}

		if (reportObjects.size() > 0) {
			result = result + "\\subsection{Results}\n";
			// Set variables
			result = result + "\\renewcommand{\\nodename}{"
					+ SmallHelpers.escapeLatex(this.getName()) + "}\n";

			// Output report objects
			for (Map.Entry<String, Vector<ReportObject>> entry : this
					.groupReportObjects(reportObjects).entrySet()) {
				result = result
						+ this.createMiniPagesForReportObjects(
								entry.getValue(), entry.getKey() + " for "
										+ this.getName()) + "\n";

			}
		}

		return result;
	}

	/**
	 * Groups the passed reportObjects into a hash map of vectors. 
	 * The keys under which each report object is stored is determined
	 * by its viewer name. 
	 */
	private HashMap<String, Vector<ReportObject>> groupReportObjects(
			Vector<ReportObject> reportObjects) {
		HashMap<String, Vector<ReportObject>> result = new HashMap<String, Vector<ReportObject>>();

		for (ReportObject curReportObject : reportObjects) {
			String key = curReportObject.getViewerName();
			Vector<ReportObject> targetVector = result.get(key);

			// Insert if not found
			if (targetVector == null) {
				targetVector = new Vector<ReportObject>();
				result.put(key, targetVector);
			}

			targetVector.add(curReportObject);
		}

		return result;
	}

	/**
	 * Creates latex code to initialize all latex variables that
	 * are used within the report export.
	 */
	public static String getInitializationForLatexVariables() {
		String result = "";

		result = result
				+ "% The following variables can be used in any sub latex file.\n";
		result = result
				+ "% They refer to the corresponding information and are updated\n";
		result = result
				+ "% for each report object.\n";;
		result = result + "\\newcommand{\\nodename}{-}\n";
		result = result + "\\newcommand{\\filename}{-}\n";
		result = result + "\\newcommand{\\filecontent}{-}\n";
		result = result + "\\newcommand{\\viewername}{-}\n";
		result = result + "\\newcommand{\\labelname}{-}\n";
		result = result + "\\newcommand{\\localID}{-}\n";

		return result;
	}

	/**
	 * This returns all command line calls that are needed to
	 * created all reportObjects in the passed vector.
	 */
	public String getScriptForLatexRepresentation(
			Vector<ReportObject> reportObjects) {
		String result = "";

		result = result + "# -----------------------------\n";
		result = result + "# Report for Node " + this.getName() + "\n";
		result = result + "# -----------------------------\n";

		for (ReportObject curObject : reportObjects) {
			result = result + curObject.getWorkflowScript() + "\n";
		}
		result = result.substring(0, result.length() - 1);

		return result;
	}

	/**
	 * Method that all GraphNodes have to implement, in order to get a TiKz
	 * graphic of the hole graph (current workflow).
	 * 
	 * @return The string that represents the definition for TiKz .
	 */
	public abstract String getTikzRepresentationOfNode();

}