/*
 * 
 * ProFiDo - The Processes Fitting Toolkit Dortmund
 * Copyright (C) 2010 - 2012 LS Informatik IV, TU Dortmund
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

package core.graph;

import gui.helper.Grid;
import helper.SmallHelpers;

import java.io.File;
import java.util.Vector;

import core.TypedFile;
import core.storage.ConfigurationXMLReader;

/**
 * This class defines the properties of an input file node.
 * 
 * InpuFileNodes are the first nodes of a workflow and have no predecessors.
 * They are used to specify files that are used as input for the workflow.
 * InpuFileNodes can have one of the following types: Trace, XML-MAP, XML-Dist,
 * XML-ATRA, XML-PH, XML-ARIMA and are mostly used to load a trace file.
 */

public class InputFileNode extends FileNode {
	private static final long serialVersionUID = 1L;

	/**
	 * Creates the node according to the passed values and sets its name to "I".
	 */
	public InputFileNode(int px, int py, ConfigurationXMLReader reader,
			Grid grid, int id, Graph graph, TypedFile inputFile) {
		super(px, py, reader, grid, id, graph);
		this.name = "I";

		this.externalFileName = inputFile;
		this.externalFileName
				.setType(reader.getInputConverterInputTypeByID(id));

		String outputName = "";
		do {
			outputName = this.externalFileName.getName()
					+ Graph.getNextFreeID()
					+ reader.getInputConverterOutputSuffixByID(id);
		} while (graph.getNodeWithSameFileName(outputName) != null);

		this.xmlOutputFile = new TypedFile(outputName, this
				.getOutputFileTypes().get(0));
	}

	@Override
	public String getXMLRepresentationOfNode(int id) {
		String xmlDescription = super.getXMLRepresentationOfNode(id);
		xmlDescription += "\t<input>\n";
		xmlDescription += "\t\t<file>\n";
		xmlDescription += "\t\t\t<name> " + this.externalFileName.getName()
				+ " </name>\n";
		if (this.externalFileName.getParent() != null
				&& !this.externalFileName.getParent().equals("")) {
			xmlDescription += "\t\t\t<path> "
					+ this.externalFileName.getParent() + File.separator
					+ " </path>\n";
		}
		xmlDescription += "\t\t\t<type> "
				+ this.externalFileName.getTypes().get(0) + " </type>\n";
		xmlDescription += "\t\t</file>\n";
		xmlDescription += "\t</input>\n";
		xmlDescription += "</node>";

		return xmlDescription;
	}

	@Override
	public String getCommandLineCall(String softwarePath, String resultPath,
			boolean comments) {
		String result = "\"" + softwarePath
				+ this.reader.getInputConverterBinaryByID(this.referenceID)
				+ "\"";

		result = result + " \"" + this.externalFileName.getPath() + "\" \""
				+ this.xmlOutputFile.getNameWithPath(resultPath, false) + "\"";
		
		// Check if output was created
		result = result + "\n" + "checkIfFileExists " + this.getXMLOutputFile().getNameWithPath(resultPath, false)  + " true";

		return result;
	}

	@Override
	public Vector<String> getInputFileTypes() {
		return this.externalFileName.getTypes();
	}

	@Override
	public Vector<String> getOutputFileTypes() {
		Vector<String> result = new Vector<String>();
		result.add(this.reader
				.getInputConverterOutputTypeByID(this.referenceID));
		return result;
	}

	@Override
	public void setConverterID(int newConverterID) {
		this.referenceID = newConverterID;
		this.externalFileName.setType(this.reader
				.getInputConverterInputTypeByID(this.referenceID));
		for (Edge curEdge : this.outgoingEdges) {
			curEdge.refreshType();
		}
	}

	public Vector<String> getCommonOutgoingTypes() {
		Vector<String> result = null;
		for (Edge curEdge : this.outgoingEdges) {
			GraphNode curFromNode = curEdge.getTo();
			Vector<String> curNodeInputFileTypes = curFromNode
					.getInputFileTypes();
			result = SmallHelpers.intersect(result, curNodeInputFileTypes);
		}
		return result;
	}

	@Override
	public String getTikzRepresentationOfNode() {
		String result = "";
		String label = this.getFileNodeCaptionText();
		result = result
				+ "\\node[inoutnode,label={[xshift=14pt]above:"
				+ SmallHelpers.escapeLatex(label.replace("<br>", " \\\\ ")
						.replace("\\", "")) + "}] (" + SmallHelpers.removeForLatexId(this.getName()
				+ this.getOrderPosition()) + ") at (" + this.getgridX() * 4
				+ ", -" + this.getgridY() * 2 + ") {"
				+ SmallHelpers.escapeLatex(this.getName()) + "};\n";
		return result;
	}

}
