/*
 * 
 * ProFiDo - The Processes Fitting Toolkit Dortmund
 * Copyright (C) 2010 - 2012 LS Informatik IV, TU Dortmund
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

package core.graph;

import gui.EditJobFrame;
import gui.helper.Grid;
import gui.helper.Line;
import helper.SmallHelpers;

import java.awt.Graphics;
import java.awt.Point;
import java.awt.Rectangle;
import java.io.Serializable;
import java.util.HashMap;
import java.util.Vector;

import core.FileNameParameterGroup;
import core.ParameterGroup;
import core.TypedFile;
import core.storage.ConfigurationXMLReader;

/**
 * This class is a specific node that represents a job node. It contains all
 * relevant properties of the job and provides interfaces for the GUIs which are
 * used within the workflow management and batch creation.
 * 
 * Job nodes are the basic building block of a workflow that perform the actual
 * tasks. Each job node corresponds to one tool for fitting and analyzing
 * distributions and stochastic processes. Within a workflow, each job is
 * defined by the used tool and a set of parameters where the number of
 * parameters might vary dependent on the job type. Each job node
 * expects/accepts a set of inputs and generates an output.
 * 
 * Currently, the following tools are supported: G-FIT, Momfit, MAP_EM,
 * MAP_MOEA, JMomfit, ACfit, MEPfit, ARIMA Fitting, ARTAfit, Modgen, Dist_Tests,
 * Plot, Print.
 * 
 */

public class JobNode extends GraphNode implements Serializable {
	private static final long serialVersionUID = 1L;

	/**
	 * Stores information about each parameter needed to execute this specific
	 * job.
	 */
	private Vector<ParameterGroup> parameterGroups;

	/**
	 * Reference to the editFrame.
	 */
	private EditJobFrame editFrame;

	/**
	 * This constructor passes all parameters used for managing this node to the
	 * superclass. Furthermore the specific information (Name, Paraemters) of
	 * the selected tool are read from the xml-file using the passed xmlReader.
	 * 
	 * @param id
	 *            specifies this jobs tool
	 * @param presetParameters
	 */
	public JobNode(String preSetName, int x, int y,
			ConfigurationXMLReader xmlReader, Grid grid, int id, Graph graph) {
		super(x, y, xmlReader, grid, id, graph);

		// Read information about corresponding application from XML-file
		this.parameterGroups = this.reader.generateProgramParameterGroups(id);
		this.name = this.reader.getProgrammNameByID(id);

		// System.out.println("Job " + this.getName() + " : " +
		// this.getOutputFileTypes().toString());

		String outputName = "";
		do {
			outputName = preSetName + Graph.getNextFreeID()
					+ xmlReader.getDefaultExtension();
		} while (graph.getNodeWithSameFileName(outputName) != null);

		this.xmlOutputFile = new TypedFile(outputName,
				this.getOutputFileTypes());
	}

	public JobNode(int x, int y, ConfigurationXMLReader xmlReader, Grid grid,
			int id, Graph graph) {
		this(graph.getName() + "_" + xmlReader.getProgrammNameByID(id), x, y,
				xmlReader, grid, id, graph);
	}

	@Override
	public boolean isInside(int x, int y) {
		Rectangle space = this.getBoundingBox();
		return space.contains(x, y);
	}

	@Override
	public Point calculateIntersection(Edge edge) {
		Line edgeLine = edge.getLine();
		Rectangle outerShape = this.getBoundingBox();

		// Get line for each border edge
		Line leftBorder = new Line(outerShape.x, outerShape.y, outerShape.x,
				outerShape.y + outerShape.height);
		Line rightBorder = new Line(outerShape.x + outerShape.width,
				outerShape.y, outerShape.x + outerShape.width, outerShape.y
						+ outerShape.height);
		Line topBorder = new Line(outerShape.x, outerShape.y, outerShape.x
				+ outerShape.width, outerShape.y);
		Line bottomBorder = new Line(outerShape.x, outerShape.y
				+ outerShape.height, outerShape.x + outerShape.width,
				outerShape.y + outerShape.height);

		// Calculate intersection of the passed edge with each of the border
		// edges
		Vector<Point> intersections = new Vector<Point>();
		intersections.add(leftBorder.getIntersection(edgeLine));
		intersections.add(rightBorder.getIntersection(edgeLine));
		intersections.add(topBorder.getIntersection(edgeLine));
		intersections.add(bottomBorder.getIntersection(edgeLine));

		// Find closest intersection and return it
		double px = edge.getLastPoint().getX();
		double py = edge.getLastPoint().getY();
		Point closestIntersection = this.getClosestIntersection(intersections,
				px, py);
		return closestIntersection;
	}

	@Override
	protected Rectangle getBoundingBox() {
		return this.grid.getDrawingSpace(this.x, this.y);
	}

	@Override
	protected void drawOutline(Graphics graphics, Rectangle shape) {
		graphics.fillRoundRect(shape.x, shape.y, shape.width, shape.height,
				shape.width / 10, shape.height / 10);
	}

	@Override
	public String getName() {
		return this.name;
	}

	@Override
	protected String getParameterText(boolean ignoreVisibilty,
			boolean markEnabled, boolean sortEnabled, boolean inculdeDisabled,
			boolean showTypes, int outputFormat) {
		String allGroups = "";
		String enabledGroups = "";
		String notEnabledGroups = "";

		// Iterate through all parameter groups
		for (ParameterGroup curParameterGroup : this.parameterGroups) {

			if (!(curParameterGroup instanceof FileNameParameterGroup)
					&& (curParameterGroup.isVisible() || ignoreVisibilty)) {
				if (outputFormat == GraphNode.OUTPUT_FORMAT_HTML) {
					String curGroupText = curParameterGroup.getName() + " "
							+ curParameterGroup.getDisplayFriendlyValueString();

					// Add information text to result vectors
					if (curParameterGroup.isEnabled()) {
						if (markEnabled) {
							curGroupText = "<b>" + curGroupText + "</b>";
						}
						enabledGroups = enabledGroups + curGroupText + "<br>\n";
					} else {
						notEnabledGroups = notEnabledGroups + curGroupText
								+ "<br>\n";
					}
					allGroups = allGroups + curGroupText + "<br>";
				} else if (outputFormat == GraphNode.OUTPUT_FORMAT_TEX) {
					// "Name", "Value", "Enabled", "Description"
					String curGroupText = "\t\t";
					if (markEnabled && inculdeDisabled) {
						if (curParameterGroup.isEnabled()) {
							curGroupText = curGroupText
									+ curParameterGroup.getName()
									+ " & "
									+ curParameterGroup
											.getDisplayFriendlyValueString()
									+ " & "
									+ curParameterGroup.getDescription().trim()
									+ " & Enabled";
						} else {
							curGroupText = curGroupText
									+ curParameterGroup.getName()
									+ " & "
									+ curParameterGroup
											.getDisplayFriendlyValueString()
									+ " & "
									+ curParameterGroup.getDescription().trim()
									+ " & Disabled";
						}
					} else {
						curGroupText = curGroupText
								+ curParameterGroup.getName()
								+ " & "
								+ curParameterGroup
										.getDisplayFriendlyValueString()
								+ " & "
								+ curParameterGroup.getDescription().trim();
					}
					curGroupText = curGroupText + "\\\\ \\hline\n";

					// Add information text to result vectors
					if (curParameterGroup.isEnabled()) {
						enabledGroups = enabledGroups + curGroupText;
					} else if (inculdeDisabled) {
						notEnabledGroups = notEnabledGroups + curGroupText;
					}
					allGroups = allGroups + curGroupText;
				}

			}
		}

		// If sorting is enabled, compose sorted result
		if (sortEnabled) {
			allGroups = "";
			allGroups = allGroups + notEnabledGroups;
			allGroups = allGroups + enabledGroups;
		}

		return allGroups;

	}

	@Override
	protected String getCaptionText(boolean ignoreVisibilty,
			boolean markEnabled, boolean sortEnabled, boolean showTypes,
			boolean hideComment) {
		String result = this.getParameterText(ignoreVisibilty, markEnabled,
				sortEnabled, true, showTypes, GraphNode.OUTPUT_FORMAT_HTML);

		// List types
		if (showTypes) {
			String inputLine = "<b>Input</b>: ";
			for (String curType : this.getInputFileTypes()) {
				inputLine = inputLine + curType + ", ";
			}
			result = result + inputLine.substring(0, inputLine.length() - 2)
					+ "<br>";

			String outputLine = "<b>Output</b>: ";
			for (String curType : this.getOutputFileTypes()) {
				outputLine = outputLine + curType + ", ";
			}
			result = result + outputLine.substring(0, outputLine.length() - 2)
					+ "<br>";
		}

		// Print comment
		if (!hideComment && (this.commentVisibilty || ignoreVisibilty)) {
			result = result + "<br>";

			String commentLines[] = this.comment.split("\n");
			for (String curLine : commentLines) {
				result = result + curLine + "<br>";
			}
		}

		return result;
	}

	public Vector<ParameterGroup> getParameterGroups() {
		return this.parameterGroups;
	}

	public ConfigurationXMLReader getReader() {
		return this.reader;
	}

	@Override
	/**
	 * This method returns the command line call to execute the job as 
	 * specified by this JobNote. For the generation of the command line call 
	 * all specified parameters are converted into strings. Furthermore the 
	 * filenames of the tools input and output, as represented by the edges 
	 * are taken into account and translated into the corresponding parameters.
	 * 
	 * @param softwarePath Path where the binaries are stored.
	 * @param resultPath Path where the result files should be stored.
	 * @return command line call
	 */
	public String getCommandLineCall(String softwarePath, String resultPath,
			boolean comments) {
		String result = "";
		if (comments) {
			result = result + "# Execute " + this.name + "\n";
		}
		result = result + "cd \"" + softwarePath
				+ this.reader.getProgrammBinaryPathByID(this.referenceID)
				+ "\"\n";

		// Transfer input filenames into corresponding parameters
		Vector<FileNameParameterGroup> usedInputParameters = new Vector<FileNameParameterGroup>();
		for (Edge curInputEdge : this.incomingEdges) {
			String curType = curInputEdge.getEdgeFileType();
			String corresponingParameterName = this.reader
					.getProgrammInputParameterByFileType(this.referenceID,
							curType);

			FileNameParameterGroup currentInputFileParameterGroup = (FileNameParameterGroup) SmallHelpers
					.findParameterGroupByName(corresponingParameterName,
							this.parameterGroups);

			currentInputFileParameterGroup.addFileName(curInputEdge
					.getFileName());

			String converterBinary = this.reader
					.getProgrammInputConverterBinaryByFileType(
							this.referenceID, curType);

			currentInputFileParameterGroup
					.setInputConverterBinary(converterBinary);

			if (usedInputParameters.indexOf(currentInputFileParameterGroup) == -1) {
				usedInputParameters.add(currentInputFileParameterGroup);
				// System.out.println("Parameter for Type: " + curType + " is "
				// +
				// currentInputFileParameterGroup.getName() +
				// " and will use converter binary: " + converterBinary);
			}
		}

		// Transfer output filenames into corresponding parameters
		FileNameParameterGroup outputFileName = (FileNameParameterGroup) SmallHelpers
				.findParameterGroupByName(this.reader
						.getProgrammOutputParameterByID(this.referenceID),
						this.parameterGroups);
		outputFileName.addFileName(this.xmlOutputFile);

		String outputConverterBinary = this.reader
				.getProgrammOutputConverterBinaryByProgrammID(this.referenceID);
		outputFileName.setOutputConverterBinary(outputConverterBinary);

		// Create input converter calls
		for (FileNameParameterGroup curGroup : usedInputParameters) {
			String curCall = curGroup.createConverterCalls(softwarePath,
					resultPath).trim();
			if (!curCall.equals("")) {
				result = result + curCall + "\n";
			}
		}

		result = result
				+ "./"
				+ this.reader.getProgrammBinaryNameByID(this.referenceID)
						.trim();
		ParameterGroup curParameterGroup = this.parameterGroups.get(0);
		while (curParameterGroup != null) {
			if (curParameterGroup.isEnabled()) {
				// System.out.println(curParameterGroup.getName() +
				// " liefert: <" + curParameterGroup.getCommandlineCallString()
				// + ">");
				String curParameterCommandlineCall = "";
				if (curParameterGroup instanceof FileNameParameterGroup) {
					curParameterCommandlineCall = ((FileNameParameterGroup) curParameterGroup)
							.getCommandlineCallString(resultPath).trim();
				} else {
					curParameterCommandlineCall = curParameterGroup
							.getCommandlineCallString().trim();
				}

				if (!curParameterCommandlineCall.equals("")) {
					result = result + " " + curParameterCommandlineCall;
				}
			}
			curParameterGroup = curParameterGroup.getSuccessor();
		}

		// Create output converter calls
		if (!outputConverterBinary.equals("")) {
			result = result
					+ "\n"
					+ outputFileName.createConverterCalls(softwarePath,
							resultPath);
		}
		
		// Check if output was created
		result = result + "\n" + "checkIfFileExists " + this.getXMLOutputFile().getNameWithPath(resultPath, false)  + " true";

		// Remove previsouly set Edges in Groups
		for (ParameterGroup curGroup : this.parameterGroups) {
			if (curGroup instanceof FileNameParameterGroup) {
				((FileNameParameterGroup) curGroup).removeAllFileNames();
			}
		}

		return result;
	}

	@Override
	public String getXMLRepresentationOfNode(int id) {
		String xmlDescription = super.getXMLRepresentationOfNode(id);
		xmlDescription += "\t<job>\n";
		xmlDescription += "\t\t<name> "
				+ this.reader.getProgrammNameByID(this.referenceID)
				+ " </name>\n";
		xmlDescription += "\t\t<parameterlist>\n";

		for (ParameterGroup p : this.parameterGroups) {
			xmlDescription += "\t\t\t<parameter>\n";
			xmlDescription += "\t\t\t\t<name> " + p.getName() + " </name>"
					+ "\n";
			xmlDescription += "\t\t\t\t<value> "
					+ p.getDisplayFriendlyValueString() + " </value>" + "\n";
			xmlDescription += "\t\t\t\t<enable> " + p.isEnabled()
					+ " </enable>" + "\n";
			xmlDescription += "\t\t\t\t<visible> " + p.isVisible()
					+ " </visible>" + "\n";
			xmlDescription += "\t\t\t</parameter>\n";
		}

		xmlDescription += "\t\t</parameterlist>\n";
		xmlDescription += "\t</job>\n";
		xmlDescription += "</node>";

		return xmlDescription;
	}

	@Override
	public Vector<String> getInputFileTypes() {
		return this.reader.getProgramInputTypesByID(this.referenceID);
	}

	@Override
	public Vector<String> getOutputFileTypes() {
		return this.reader.getProgramOutputTypesByID(this.referenceID);
	}

	public void setEditDialog(EditJobFrame editFrame) {
		this.editFrame = editFrame;
	}

	/**
	 * This method return weather or not this Node has a currently visible edit
	 * dialog.
	 */
	public boolean hasEditDialog() {
		return this.editFrame != null;
	}

	public void hideEditDialog() {
		this.editFrame.setVisible(false);
	}

	public void setParameterValues(HashMap<String, String> presetParameters) {
		ParameterGroup.setParameterValues(this.parameterGroups,
				presetParameters);
	}

	public void setFixedParameterValues(Vector<String> fixedParameters) {
		ParameterGroup.setFixedAttribute(this.parameterGroups,
				fixedParameters);
	}

	@Override
	public String getTikzRepresentationOfNode() {
		String result = "";
		String label = this.getCaptionText(Graph.DISPLAY_ALL_NODE_INFORMATION,
				Graph.DISPLAY_ALL_NODE_INFORMATION,
				Graph.DISPLAY_ALL_NODE_INFORMATION, false, true);
		result = result
				+ "\\node[jobnode,label={[text width=8cm, xshift=70pt]above:"
				+ SmallHelpers.escapeLatex(label.replace("<br>", " \\\\ ")
						) + "}] (" + SmallHelpers.removeForLatexId(this.getName()
				+ this.getOrderPosition()) + ") at (" + this.getgridX() * 4
				+ ", -" + this.getgridY() * 2 + ") {"
				+ SmallHelpers.escapeLatex(this.getName()) + "};\n";
		return result;
	}

}
