/*
 * 
 * ProFiDo - The Processes Fitting Toolkit Dortmund
 * Copyright (C) 2010 - 2012 LS Informatik IV, TU Dortmund
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

package core.graph;

import gui.helper.Grid;
import helper.SmallHelpers;

import java.io.File;
import java.util.Vector;

import core.TypedFile;
import core.storage.ConfigurationXMLReader;

/**
 * This abstract class defines the properties of an output file node.
 * 
 * OutputFileNodes mark the end of a workflow and have no successors, they
 * represent files that are used to specify the results of a workflow that the
 * user is interested in, such as fitted models or plots with characteristics of
 * the models. They have a name and a location where they should be saved.
 * OutputFileNodes can have the following output formats: XML-Trace, PS, Latex,
 * Trace, Map.
 */

public class OutputFileNode extends FileNode {
	private static final long serialVersionUID = 1L;

	/**
	 * Creates the node according to the passed values and sets its name to "O".
	 */
	public OutputFileNode(int px, int py, ConfigurationXMLReader xmlReader,
			Grid grid, int id, Graph graph, TypedFile fileName) {
		super(px, py, xmlReader, grid, id, graph);
		this.name = "O";

		this.externalFileName = fileName;
		this.externalFileName.setType(this.reader
				.getOutputConverterOutputTypeByID(id));
	}

	@Override
	public String getCommandLineCall(String softwarePath, String resultPath,
			boolean comments) {
		GraphNode inputNode = this.incomingEdges.get(0).getFrom();

		TypedFile inputNodeXMLOutputFile = inputNode.getXMLOutputFile();
		String result = "\"" + softwarePath
				+ this.reader.getOutputConverterBinaryByID(this.referenceID)
				+ "\"";
		result = result + " \""
				+ inputNodeXMLOutputFile.getNameWithPath(resultPath, false)
				+ "\" \""
				+ this.externalFileName.getNameWithPath(resultPath, false)
				+ "\"";

		return result;
	}

	@Override
	public String getXMLRepresentationOfNode(int id) {
		String xmlDescription = super.getXMLRepresentationOfNode(id);
		xmlDescription += "\t<output>\n";
		xmlDescription += "\t\t<file>\n";
		xmlDescription += "\t\t\t<name> "
				+ this.getExternalFileName().getName() + " </name>\n";
		if (this.externalFileName.getParent() != null
				&& !this.externalFileName.getParent().equals("")) {
			xmlDescription += "\t\t\t<path> "
					+ this.externalFileName.getParent() + File.separator
					+ " </path>\n";
		}
		xmlDescription += "\t\t\t<type> "
				+ this.reader
						.getOutputConverterOutputTypeByID(this.referenceID)
				+ " </type>\n";
		xmlDescription += "\t\t</file>\n";
		xmlDescription += "\t</output>\n";
		xmlDescription += "</node>";

		return xmlDescription;
	}

	@Override
	public Vector<String> getInputFileTypes() {
		Vector<String> result = new Vector<String>();
		result.add(this.reader
				.getOutputConverterInputTypeByID(this.referenceID));
		return result;
	}

	@Override
	public Vector<String> getOutputFileTypes() {
		Vector<String> result = new Vector<String>();
		result.add(this.reader
				.getOutputConverterOutputTypeByID(this.referenceID));
		return result;
	}

	@Override
	public void setConverterID(int newConverterID) {
		this.referenceID = newConverterID;
		this.externalFileName.setType(this.reader
				.getOutputConverterOutputTypeByID(this.referenceID));
		for (Edge curEdge : this.incomingEdges) {
			curEdge.refreshType();
		}
	}

	public Vector<String> getCommonIncomingTypes() {
		Vector<String> result = null;
		for (Edge curEdge : this.incomingEdges) {
			GraphNode curFromNode = curEdge.getFrom();
			Vector<String> curNodeOutputFileTypes = curFromNode
					.getOutputFileTypes();
			result = SmallHelpers.intersect(result, curNodeOutputFileTypes);
		}
		return result;
	}

	@Override
	public String getTikzRepresentationOfNode() {

		String result = "";
		String label = this.getFileNodeCaptionText();
		result = result
				+ "\\node[inoutnode,label={[xshift=14pt]above:"
				+ SmallHelpers.escapeLatex(label.replace("<br>", " \\\\ ")
						.replace("\\", "")) + "}] (" + SmallHelpers.removeForLatexId(this.getName()
				+ this.getOrderPosition()) + ") at (" + this.getgridX() * 4
				+ ", -" + this.getgridY() * 2 + ") {"
				+ SmallHelpers.escapeLatex(this.getName()) + "};\n";
		return result;
	}

}
