/*
 * 
 * ProFiDo - The Processes Fitting Toolkit Dortmund
 * Copyright (C) 2010 - 2012 LS Informatik IV, TU Dortmund
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

package core.graph;

import gui.helper.Grid;
import helper.SmallHelpers;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Point;
import java.awt.Rectangle;
import java.io.Serializable;

/**
 * This class is a specific node that creates the bases (green points) of the
 * edges used to allow more flexible alignment in order to avoid crossings and
 * thus to enable a more readable graph layout.
 */

public class PointNode extends BasicNode implements Serializable {
	private static final long serialVersionUID = 1L;
	private static final int RADIUS = 4;

	/**
	 * Indicates if this node is used positioned or if its position is
	 * calculated according to its neighboring nodes.
	 */
	private boolean isUserPositioned = false;

	/**
	 * Reference to the left neighbouring node of this node.
	 */
	private BasicNode leftNeighbour;

	/**
	 * Reference to the right neighbouring node of this node.
	 */
	private BasicNode rightNeighbour;

	/**
	 * Reference to the edge on which the PointNode is located.
	 */
	private Edge edge;

	/**
	 * Control variable to give every PointNode a unique id.
	 */
	private static int pid = 0;

	/**
	 * Reference to the unique id.
	 */
	private int id;

	/**
	 * Creates the node according to the passed values.
	 */
	public PointNode(Grid grid, BasicNode leftNeighbour,
			BasicNode rightNeighbour, boolean userPositioned, Edge edge) {
		super(-1, -1, grid, null);
		this.leftNeighbour = leftNeighbour;
		this.rightNeighbour = rightNeighbour;
		this.isUserPositioned = userPositioned;
		this.edge = edge;
		this.id = PointNode.getNextPid();
		this.determinePositionAccordingToNeighbours();

		if (userPositioned) {
			this.moveToPixelCoordinates(this.x, this.y, true);
		}
	}

	/**
	 * This method returns the next free id for a PointNode. This id is used to
	 * uniquely determine a PointNode.
	 */
	private static int getNextPid() {

		return PointNode.pid++;
	}

	@Override
	public void moveToGridCoordinates(Point newPosition) {
		if (this.isUserPositioned) {
			this.gridX = newPosition.x;
			this.gridY = newPosition.y;
			this.realignToGrid();

			this.edge.notifiyThatNodeHasMoved();
		}
	}

	@Override
	public void moveToPixelCoordinates(int pX, int pY, boolean pDoRealign) {
		if (this.isUserPositioned) {
			if (pDoRealign) {
				Point gridPosition = this.grid.getNearestCell(pX, pY);
				this.gridX = gridPosition.x;
				this.gridY = gridPosition.y;

				this.realignToGrid();
			} else {
				this.x = pX;
				this.y = pY;
			}

			this.edge.notifiyThatNodeHasMoved();
		}
	}

	/**
	 * This method determines this nodes position by calculating the center
	 * point between its two neighbours.
	 */
	private void determinePositionAccordingToNeighbours() {
		BasicNode left = this.leftNeighbour;
		BasicNode right = this.rightNeighbour;

		// Search until suitable end and start points are found
		while (!(left instanceof GraphNode || left instanceof PointNode
				&& ((PointNode) left).isUserPositioned())) {
			left = ((PointNode) left).getLeftNeighbour();
		}

		while (!(right instanceof GraphNode || right instanceof PointNode
				&& ((PointNode) right).isUserPositioned())) {
			right = ((PointNode) right).getRightNeighbour();
		}

		int distX = Math.round((left.getX() - right.getX()) / 2);
		int distY = Math.round((left.getY() - right.getY()) / 2);

		this.x = right.getX() + distX;
		this.y = right.getY() + distY;
	}

	public boolean isUserPositioned() {
		return this.isUserPositioned;
	}

	/**
	 * This method sets whether a PointNode is auto positioned or manually
	 * positioned by the user. If it sets it to user positioned the method also
	 * notify the edge to reposition this PointNode.
	 */
	public void setUserPositioned(boolean userPositioned) {
		this.isUserPositioned = userPositioned;
		if (userPositioned) {
			this.moveToPixelCoordinates(this.x, this.y, true);
		} else {
			this.determinePositionAccordingToNeighbours();
			this.edge.notifiyThatNodeHasMoved();
		}
	}

	public BasicNode getLeftNeighbour() {
		return this.leftNeighbour;
	}

	public BasicNode getRightNeighbour() {
		return this.rightNeighbour;
	}

	/**
	 * Draws the node on the passed graphics object.
	 */
	@Override
	public void draw(Graphics pGraphics) {
		Rectangle shape = this.getBoundingBox();
		pGraphics.setColor(Color.BLACK);
		pGraphics.fillOval(shape.x, shape.y, shape.width, shape.height);
		pGraphics.setColor(Color.GREEN);
		pGraphics.fillOval(shape.x + 1, shape.y + 1, shape.width - 2,
				shape.height - 2);
	}

	@Override
	protected Rectangle getBoundingBox() {
		Rectangle space = new Rectangle(this.x - PointNode.RADIUS, this.y
				- PointNode.RADIUS, 2 * PointNode.RADIUS, 2 * PointNode.RADIUS);
		return space;
	}

	@Override
	public boolean isInside(int pX, int pY) {
		Rectangle shape = this.getBoundingBox();

		double xdist = pX - this.x;
		double ydist = pY - this.y;

		double xdist2 = Math.pow(xdist, 2);
		double ydist2 = Math.pow(ydist, 2);

		// Note: The correct mathematical calculation if
		// the passed coordinates are within this node require
		// a division by 2 to determine this nodes radius.
		// As point nodes are very very small, it would be very
		// hard to hit them with the mouse, so the radius
		// is multiplied by 1.8 in order to enlarge the
		// click area.
		return Math.sqrt(xdist2 + ydist2) <= 1.8 * (shape.width / 2);
	}

	public void notifiyThatNodeOrEdgeHasMoved() {
		if (!this.isUserPositioned) {
			this.determinePositionAccordingToNeighbours();
		}
	}

	public void setRightNeighbour(BasicNode newNode) {
		this.rightNeighbour = newNode;
	}

	public void setLeftNeighbour(BasicNode newNode) {
		this.leftNeighbour = newNode;

	}

	@Override
	public void realignToGrid() {
		if (this.gridX != -1 && this.gridY != -1 && this.isUserPositioned) {
			Point newLocation = this.grid.getCellCenter(this.gridX, this.gridY);
			this.x = newLocation.x;
			this.y = newLocation.y;
		}
	}

	@Override
	public Point getGridPosition() {
		if (this.isUserPositioned)
			return new Point(this.gridX, this.gridY);
		else
			return new Point(-1, -1);
	}

	@Override
	public String getTooltipText() {
		return this.edge.getTooltipText();
	}

	public Edge getEdge() {
		return this.edge;
	}

	/**
	 * This Method returns true if there is only one PointNode on the edge.
	 * Otherwise it returns false, which means there a more PointNodes on the
	 * edge.
	 */
	public boolean isOnlyPointNodeOfEdge() {
		return this.edge.getLastPointNode() == this
				&& this.edge.getFirstPointNode() == this;
	}

	@Override
	public String getName() {
		return "EdgePoint on " + this.edge.getDisplayFriendlyName();
	}

	public int getID() {

		return this.id;
	}

	/*
	 * @Override public String getName() { return "Edge from " +
	 * edge.getFrom().getName() + " to " + edge.getTo().getName(); }
	 */

	/**
	 * This method gives you the TikZ representation for the current PointNode.
	 * 
	 * @return The string that represents the PointNode in TiKz code.
	 */
	public String getTikzRepresentation() {
		String result = "";
		String label = this.edge.getTikZPointNodeCaptionText();
		if (!(this.getgridX() == -1) && !(this.getgridY() == -1)) {
			if (this.equals(this.edge.getFirstPointNode())) {
				result = result
						+ "\\node[edgenode, label={above:"
						+ SmallHelpers.escapeLatex(label.replace("<br>",
								" \\\\ ")) + "}] (" + this.getID() + ") at ("
						+ 4 * this.gridX + ", -" + 2 * this.gridY + ") {};\n";
			} else {

				result = result + "\\node[edgenode] (" + this.getID()
						+ ") at (" + 4 * this.gridX + ", -" + 2 * this.gridY
						+ ") {};\n";
			}
		}
		return result;

	}

	/**
	 * This method gives you the right neighbor of your current PointNode
	 * 
	 * @return A PointNode object that is the right neighbor
	 */
	public PointNode getRightNeighbourAsPointNode() {

		return (PointNode) this.rightNeighbour;
	}

}
