/*
 * ProFiDo - The Processes Fitting Toolkit Dortmund
 * Copyright (C) 2010 - 2012 LS Informatik IV, TU Dortmund
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */
package core.storage;

import java.awt.Dimension;
import java.io.File;
import java.io.IOException;
import java.net.URL;
import java.util.Vector;

import javax.swing.JOptionPane;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.xml.XMLConstants;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.Source;
import javax.xml.transform.stream.StreamSource;
import javax.xml.validation.Schema;
import javax.xml.validation.SchemaFactory;
import javax.xml.validation.Validator;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpression;
import javax.xml.xpath.XPathExpressionException;
import javax.xml.xpath.XPathFactory;

import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.ErrorHandler;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;

/**
 * This class provides basic interface to access a xml file via dom and xpath.
 */
public abstract class BasicXMLReader {
	/**
	 * Memory representation of the xml file of this xml reader.
	 */
	protected Document document;

	/**
	 * Reference to the xpath object used to adress nodes within this readers
	 * document.
	 */
	private XPath xpath;

	/**
	 * default constructor. 
	 */
	public BasicXMLReader() throws XPathExpressionException,
			ParserConfigurationException, SAXException, IOException {
	}

	/**
	 * This method opens the passed xml file and reads its content into the
	 * document field.
	 * 
	 * @param schemaURL
	 */
	protected void openFile(String filename, URL schemaURL)
			throws ParserConfigurationException, SAXException, IOException,
			XPathExpressionException {
		File file = new File(filename);

		if (file.exists()) {
			// Perform checks
			this.validateXML(file, schemaURL);

			// Load document
			DocumentBuilderFactory domFactory = DocumentBuilderFactory
					.newInstance();
			domFactory.setNamespaceAware(true);
			DocumentBuilder builder = domFactory.newDocumentBuilder();
			this.document = builder.parse(filename);

			// Prepare XPath
			XPathFactory xpathFactory = XPathFactory.newInstance();
			this.xpath = xpathFactory.newXPath();
		} else
			throw new IOException("File " + file + " not found.");
	}

	/**
	 * This method validates an XML file against the passed schema.
	 */
	public void validateXML(File inputFile, URL schemaURL) throws SAXException,
			IOException {
		ValidationErrorHandler validationErrorHandler = new ValidationErrorHandler();
		// Perform schema checks
		if (schemaURL != null) {
			// 1. Lookup a factory for the W3C XML Schema language
			SchemaFactory factory = SchemaFactory
					.newInstance(XMLConstants.W3C_XML_SCHEMA_NS_URI);

			// 2. Compile the schema.
			Schema schema = factory.newSchema(schemaURL);

			// 3. Get a validator from the schema.
			Validator validator = schema.newValidator();
			validator.setErrorHandler(validationErrorHandler);

			// 4. Parse the document we want to check.
			Source source = new StreamSource(inputFile);
			try {
				validator.validate(source);
			} catch (Exception e) {
			}

			// Handle validation errors
			if (validationErrorHandler.notEmpty()) {
				JTextArea textArea = new JTextArea(
						validationErrorHandler.createErrorText());
				JScrollPane scrollPane = new JScrollPane(textArea);
				textArea.setLineWrap(true);
				textArea.setWrapStyleWord(true);
				textArea.setEditable(false);
				scrollPane.setPreferredSize(new Dimension(400, 200));

				JOptionPane.showMessageDialog(null, scrollPane, "Error",
						JOptionPane.ERROR_MESSAGE);

				if (validationErrorHandler.hasErrors())
					throw new SAXException("Verification Error.");
			}
		}
	}

	/**
	 * This method performs all semeantic checks of the xml file.
	 */
	protected abstract void performSemanticChecks();

	/**
	 * This method returns a node list containing the result of the execution of
	 * the passed XPath expression string.
	 */
	public NodeList getNodeListOfXPathExpression(Node root, String xpathString) {
		try {
			XPathExpression xpathExpression = this.xpath.compile(xpathString);
			NodeList result = (NodeList) xpathExpression.evaluate(root,
					XPathConstants.NODESET);
			return result;
		} catch (XPathExpressionException e) {
			System.out
					.println("Error while reading xml file. Please check if input file is valid.\nUsed XPath: "
							+ xpathString);
			return null;
		}
	}

	/**
	 * This method returns a string containing the result of the execution of
	 * the passed XPath expression string.
	 */
	public String getStringOfXPathExpression(Node root, String xpathString) {
		try {
			XPathExpression xpathExpression = this.xpath.compile(xpathString);
			String result = (String) xpathExpression.evaluate(root,
					XPathConstants.STRING);
			return result;
		} catch (XPathExpressionException e) {
			System.out
					.println("Error while reading xml file. Please check if input file is valid.\nUsed XPath: "
							+ xpathString);
			return null;
		}
	}

	/**
	 * This method collects all child nodes (of type text) of the nodes
	 * specified by the passed xpathString. Afterwards it reads their values,
	 * stores them in vector and returns it. Please note: The term child node
	 * can be misleading. In the following example xml: <foo> <bar> Test </bar>
	 * </foo> bar is obviously the child node of foo. But according to the
	 * specification the atomic value "Test" is also a node and child of bar.
	 * Therefore, the child node of the node specified by the xpath "/foo/bar"
	 * is the text node "Test".
	 */
	public Vector<String> getValuesOfAllTextNodesBelowXPathNodes(Node root,
			String xpathString) {
		Vector<String> result = new Vector<String>();
		NodeList nodes = this.getNodeListOfXPathExpression(root, xpathString
				+ "/child::text()");
		for (int i = 0; i < nodes.getLength(); i++) {
			result.add(nodes.item(i).getNodeValue());
		}
		return result;
	}

	/**
	 * This method collects the first child node (of type text) of the nodes
	 * specified by the passed xpathString and returns them as a string. Please
	 * note: The term child node can be misleading. In the following example
	 * xml: <foo> <bar> Test </bar> </foo> bar is obviously the child node of
	 * foo. But according to the specification the atomic value "Test" is also a
	 * node and child of bar. Therefore, the child node of the node specified by
	 * the xpath "/foo/bar" is the text node "Test".
	 */
	public String getValuesOfFirstTextNodeBelowXPathNodes(Node root,
			String xpathString) {
		return this.getStringOfXPathExpression(root, xpathString
				+ "/child::text()[1]");
	}

	/**
	 * ErrorHandler for the Validation of an XML file.
	 */
	private class ValidationErrorHandler implements ErrorHandler {
		private Vector<SAXParseException> warnings = new Vector<SAXParseException>();
		private Vector<SAXParseException> errors = new Vector<SAXParseException>();
		private Vector<SAXParseException> fatalErrors = new Vector<SAXParseException>();

		@Override
		public void warning(SAXParseException e) {
			this.warnings.add(e);
		}

		public boolean notEmpty() {
			return this.warnings.size() > 0 || this.errors.size() > 0
					|| this.fatalErrors.size() > 0;
		}

		@Override
		public void error(SAXParseException e) {
			this.errors.add(e);
		}

		@Override
		public void fatalError(SAXParseException e) {
			this.fatalErrors.add(e);
		}

		public boolean hasErrors() {
			return this.errors.size() > 0 || this.fatalErrors.size() > 0;
		}

		/**
		 * This Method creates a report for all SAXParseExceptions included 
		 * in the passed Vector.
		 */
		private String createExceptionsInfoString(
				Vector<SAXParseException> exceptions) {
			String result = "";
			for (SAXParseException curException : exceptions) {
				String message = curException.getMessage();
				result = result
						+ "\n\n- L: "
						+ curException.getLineNumber()
						+ ", C: "
						+ curException.getColumnNumber()
						+ ": "
						+ message.substring(message.indexOf(": ") + 2,
								message.length());
			}
			return result.trim();
		}

		/**
		 * This Method returns an error report text for all
		 * fatal errors / errors / warnings that are found while validating 
		 * an XML file against an XML schema.
		 */
		public String createErrorText() {
			String result = "";

			if (this.fatalErrors.size() > 0) {
				result = result + "Fatal Errors: \n"
						+ this.createExceptionsInfoString(this.fatalErrors)
						+ "\n\n\n";
			}
			if (this.errors.size() > 0) {
				result = result + "Errors: \n"
						+ this.createExceptionsInfoString(this.errors)
						+ "\n\n\n";
			}
			if (this.warnings.size() > 0) {
				result = result + "Warnings: \n"
						+ this.createExceptionsInfoString(this.warnings)
						+ "\n\n\n";
			}

			return result;
		}

	}

}