/*
 * ProFiDo - The Processes Fitting Toolkit Dortmund
 * Copyright (C) 2010 - 2012 LS Informatik IV, TU Dortmund
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */
package core.storage;

import helper.ConverterSignature;
import helper.SmallHelpers;

import java.io.File;
import java.io.IOException;
import java.net.URL;
import java.util.HashMap;
import java.util.Vector;

import javax.swing.JOptionPane;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.xpath.XPathExpressionException;

import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

import core.BooleanParameterGroup;
import core.DynamicTextParameter;
import core.FileNameParameterGroup;
import core.ParameterGroup;
import core.SelectParameterGroup;
import core.StaticTextParameter;
import core.TextParameter;
import core.TextParameterGroup;

/**
 * This xml reader is used to read the applications configuration file which
 * specifies all possible jobs of a workflow and furhter settings of the gui.
 */
public class ConfigurationXMLReader extends BasicXMLReader {
	public static final int REPORT = 0;
	public static final int EDGE_VIEWER = 1;

	/**
	 * Reference to a node list storing all <program> nodes of the config file.
	 * The position within this list corresponds to the program id.
	 */
	NodeList programXMLNodes;

	/**
	 * Reference to a node list storing all <converter> nodes of type "input"
	 * within the config file. The position within this list corresponds to the
	 * program id.
	 */
	NodeList inputConverterXMLNodes;

	/**
	 * Reference to a node list storing all <converter> nodes of type "output"
	 * within the config file. The position within this list corresponds to the
	 * program id.
	 */
	NodeList outputConverterXMLNodes;

	/**
	 * Reference to a node list storing all <viewer> nodes within the <report>
	 * section of the config file. The position within this list corresponds to
	 * the program id.
	 */
	NodeList edgeViewerXMLNodes;

	/**
	 * Reference to a node list storing all <viewer> nodes within the <report>
	 * section of the config file. The position within this list corresponds to
	 * the program id.
	 */
	NodeList reportXMLNodes;

	private boolean checkXML;

	@Override
	protected void openFile(String filename, URL schemaURL)
			throws XPathExpressionException, ParserConfigurationException,
			SAXException, IOException {
		super.openFile(filename, schemaURL);
		this.programXMLNodes = this.getNodeListOfXPathExpression(this.document,
				"/settings/software/program");
		this.inputConverterXMLNodes = this.getNodeListOfXPathExpression(
				this.document, "/settings/tools/converter[@type='input']");
		this.outputConverterXMLNodes = this.getNodeListOfXPathExpression(
				this.document, "/settings/tools/converter[@type='output']");
		this.edgeViewerXMLNodes = this.getNodeListOfXPathExpression(
				this.document, "/settings/edgeviewer/viewer");
		this.reportXMLNodes = this.getNodeListOfXPathExpression(this.document,
				"/settings/report/viewer");
		if (this.checkXML) {
			this.performSemanticChecks();
		}
	}

	/**
	 * This method loads all static and global settings from the configuration
	 * file.
	 */
	public void loadStaticSettings() {
		// Load valid character expression
		String validInputCharacters = this
				.getValuesOfFirstTextNodeBelowXPathNodes(this.document,
						"/settings/config/validinputcharacters");
		if (!validInputCharacters.equals("")) {
			SmallHelpers.characterRegExp = validInputCharacters;
		}
	}

	public ConfigurationXMLReader(String filename, boolean checkSchema)
			throws XPathExpressionException, ParserConfigurationException,
			SAXException, IOException {
		super();

		this.checkXML = checkSchema;
		// Load schema url
		URL schemaURL = null;
		if (checkSchema) {
			schemaURL = this.getClass().getResource("/software.xsd");
		}

		this.openFile(filename, schemaURL);

		this.loadStaticSettings();
	}

	/**
	 * This method returns the names of all the programs in the xml file..
	 */
	public Vector<String> getProgrammNames() {
		return this.getValuesOfAllTextNodesBelowXPathNodes(this.document,
				"/settings/software/program/general[1]/name[1]");
	}

	/**
	 * This method returns the id of the first job matching the passed name.
	 */
	public int getProgrammIDByName(String name) {
		Vector<String> names = this.getProgrammNames();
		for (int curProgramIndex = 0; curProgramIndex < names.size(); curProgramIndex++)
			if (names.get(curProgramIndex).equalsIgnoreCase(name))
				return curProgramIndex;
		return -1;
	}

	/**
	 * This method returns a dictionary containing the signatures (Input Type,
	 * Input Extension, Output Type, Output Extension) of any matching input
	 * converter using the converter ids as keys. An input converter is included
	 * in the dictionary if it outputs a type listed in the additionally passed
	 * vector. If null or an empty vector is passed, all input converters are
	 * included in the dictionary.
	 */
	public HashMap<Integer, ConverterSignature> getInputConverterSignatures(
			Vector<String> outputTypeFilter) {
		HashMap<Integer, ConverterSignature> result = new HashMap<Integer, ConverterSignature>();

		for (int id = 0; id < this.inputConverterXMLNodes.getLength(); id++) {
			String curOutputType = this.getInputConverterOutputTypeByID(id);
			if (this.isInVector(curOutputType, outputTypeFilter)) {
				ConverterSignature curSignatrue = new ConverterSignature(id,
						this.getInputConverterInputTypeByID(id),
						this.getInputConverterInputSuffixByID(id),
						curOutputType,
						this.getInputConverterOutputSuffixByID(id));
				result.put(id, curSignatrue);
			}
		}

		return result;
	}

	/**
	 * This method return the input type of the viewer specified by the passed
	 * id.
	 * 
	 * @param section
	 */
	public Vector<String> getViewerInputTypesByID(int id, int section) {
		if (section == ConfigurationXMLReader.EDGE_VIEWER)
			return this.getValuesOfAllTextNodesBelowXPathNodes(
					this.edgeViewerXMLNodes.item(id), "type");
		else if (section == ConfigurationXMLReader.REPORT)
			return this.getValuesOfAllTextNodesBelowXPathNodes(
					this.reportXMLNodes.item(id), "type");
		else
			return null;
	}

	public boolean getIsViewerInternByID(Integer id) {
		return !this.getViewerInternTypeByID(id).equalsIgnoreCase("");
	}

	public String getViewerInternTypeByID(Integer id) {
		return this.getValuesOfFirstTextNodeBelowXPathNodes(
				this.edgeViewerXMLNodes.item(id), "intern[1]");
	}

	public boolean getIsViewerExternByID(Integer id) {
		return !this.getViewerExternBinaryByID(id).equalsIgnoreCase("");
	}

	public String getViewerExternBinaryByID(Integer id) {
		return this.getValuesOfFirstTextNodeBelowXPathNodes(
				this.edgeViewerXMLNodes.item(id), "extern[1]");
	}

	/**
	 * This method returns the preset parameters for the viewer specified by the
	 * passed id.
	 * 
	 * @param id
	 *            Id of the viewer.
	 * @param section
	 * @return
	 */
	public HashMap<String, String> getViewerPresetParameters(Integer id,
			int section) {
		HashMap<String, String> result = new HashMap<String, String>();

		NodeList parameterGroups = null;
		if (section == ConfigurationXMLReader.EDGE_VIEWER) {
			parameterGroups = this.getNodeListOfXPathExpression(
					this.edgeViewerXMLNodes.item(id),
					"program/preset/parametergroup");
		} else if (section == ConfigurationXMLReader.REPORT) {
			parameterGroups = this.getNodeListOfXPathExpression(
					this.reportXMLNodes.item(id),
					"program/preset/parametergroup");
		}

		for (int curParameterGroupID = 0; curParameterGroupID < parameterGroups
				.getLength(); curParameterGroupID++) {
			String curName = this.getValuesOfFirstTextNodeBelowXPathNodes(
					parameterGroups.item(curParameterGroupID), "name[1]");
			String curValue = this.getValuesOfFirstTextNodeBelowXPathNodes(
					parameterGroups.item(curParameterGroupID), "value[1]");
			result.put(curName, curValue);
		}
		return result;
	}

	/**
	 * This method returns all parameters that are marked "fixed"
	 * for the specified viewer.
	 * 
	 * @param id ID of the viewer.
	 * @param section Section of the viewer.
	 * @return
	 */
	public Vector<String> getViewerFixedParameters(Integer id, int section) {
		Vector<String> result = new Vector<String>();

		NodeList parameterGroups = null;
		if (section == ConfigurationXMLReader.EDGE_VIEWER) {
			parameterGroups = this.getNodeListOfXPathExpression(
					this.edgeViewerXMLNodes.item(id),
					"program/preset/parametergroup");
		} else if (section == ConfigurationXMLReader.REPORT) {
			parameterGroups = this.getNodeListOfXPathExpression(
					this.reportXMLNodes.item(id),
					"program/preset/parametergroup");
		}

		for (int curParameterGroupID = 0; curParameterGroupID < parameterGroups
				.getLength(); curParameterGroupID++) {
			boolean isFixed = this.getStringOfXPathExpression(
					parameterGroups.item(curParameterGroupID), "@fixed[1]")
					.equalsIgnoreCase("true");
			String curName = this.getValuesOfFirstTextNodeBelowXPathNodes(
					parameterGroups.item(curParameterGroupID), "name[1]");
			if (isFixed) {
				result.add(curName);
			}
		}
		return result;
	}

	/**
	 * This method return the name of the viewer specified by the passed id.
	 * 
	 * @param section
	 * @param section
	 */
	public String getViewerNameByID(int id, int section) {
		if (section == ConfigurationXMLReader.EDGE_VIEWER)
			return this.getValuesOfFirstTextNodeBelowXPathNodes(
					this.edgeViewerXMLNodes.item(id), "name[1]");
		else if (section == ConfigurationXMLReader.REPORT)
			return this.getValuesOfFirstTextNodeBelowXPathNodes(
					this.reportXMLNodes.item(id), "name[1]");
		else
			return null;
	}


	public String getViewerDescriptionByID(int id, int section) {
		if (section == ConfigurationXMLReader.EDGE_VIEWER)
			return this.getValuesOfFirstTextNodeBelowXPathNodes(
					this.edgeViewerXMLNodes.item(id), "description[1]");
		else if (section == ConfigurationXMLReader.REPORT)
			return this.getValuesOfFirstTextNodeBelowXPathNodes(
					this.reportXMLNodes.item(id), "description[1]");
		else
			return null;
	}

	/**
	 * This method return all viewers that are able to display the passed type.
	 */
	public Vector<Integer> getAllMatchingViewerIDsForType(String selectedType,
			int section) {
		int nodeCount = 0;
		if (section == ConfigurationXMLReader.EDGE_VIEWER) {
			nodeCount = this.edgeViewerXMLNodes.getLength();
		} else if (section == ConfigurationXMLReader.REPORT) {
			nodeCount = this.reportXMLNodes.getLength();
		}

		Vector<Integer> result = new Vector<Integer>();
		for (int id = 0; id < nodeCount; id++) {
			Vector<String> curViewerTypes = this.getViewerInputTypesByID(id,
					section);

			// Look for matching type
			for (String curType : curViewerTypes) {
				if (selectedType.equalsIgnoreCase(curType)) {
					result.add(id);
					break;
				}
			}
		}

		return result;
	}

	/**
	 * This method return the ID of the programm corresponding to the passed
	 * viwer.
	 * 
	 * @param section
	 */
	public int getProgramIDOfViewer(Integer viewerID, int section) {
		String programName = this.getNameOfViewer(viewerID, section);
		// Find programm with name
		for (int curProgrammIndex = 0; curProgrammIndex < this.programXMLNodes
				.getLength(); curProgrammIndex++) {
			String curProgramName = this
					.getValuesOfFirstTextNodeBelowXPathNodes(
							this.programXMLNodes.item(curProgrammIndex),
							"general[1]/name[1]");

			if (curProgramName.equalsIgnoreCase(programName))
				return curProgrammIndex;
		}

		return -1;
	}

	/**
	 * This method returns the name of the specified viewer.	 
	 * 
	 * @param id ID of the viewer.
	 * @param section Section of the viewer.
	 */
	public String getNameOfViewer(int viewerID, int section) {
		String programName = "";
		if (section == ConfigurationXMLReader.EDGE_VIEWER) {
			programName = this.getValuesOfFirstTextNodeBelowXPathNodes(
					this.edgeViewerXMLNodes.item(viewerID),
					"program[1]/name[1]");
		} else if (section == ConfigurationXMLReader.REPORT) {
			programName = this.getValuesOfFirstTextNodeBelowXPathNodes(
					this.reportXMLNodes.item(viewerID), "program[1]/name[1]");
		}

		return programName;
	}
/*
	public String getProgramNameOfViewer(Integer viewerID) {
		return this.getValuesOfFirstTextNodeBelowXPathNodes(
				this.edgeViewerXMLNodes.item(viewerID), "program/name[1]");
	}
*/
	/**
	 * Returns the converter assigned to the passed viewer.
	 * 
	 * @param id ID of the viewer.
	 * @param section Section of the viewer.
	 */
	public int getConverterIDOFViewer(Integer viewerID, int section) {
		String inputSignature = "";
		String outputSignature = "";

		if (section == ConfigurationXMLReader.EDGE_VIEWER) {
			inputSignature = this.getValuesOfFirstTextNodeBelowXPathNodes(
					this.edgeViewerXMLNodes.item(viewerID),
					"converter[1]/input[1]");
			outputSignature = this.getValuesOfFirstTextNodeBelowXPathNodes(
					this.edgeViewerXMLNodes.item(viewerID),
					"converter[1]/output[1]");
		} else if (section == ConfigurationXMLReader.REPORT) {
			inputSignature = this
					.getValuesOfFirstTextNodeBelowXPathNodes(
							this.reportXMLNodes.item(viewerID),
							"converter[1]/input[1]");
			outputSignature = this.getValuesOfFirstTextNodeBelowXPathNodes(
					this.reportXMLNodes.item(viewerID),
					"converter[1]/output[1]");
		}

		for (int id = 0; id < this.outputConverterXMLNodes.getLength(); id++) {
			String curInputType = this.getOutputConverterInputTypeByID(id);
			String curOutputType = this.getOutputConverterOutputTypeByID(id);

			if (inputSignature.equalsIgnoreCase(curInputType)
					&& outputSignature.equalsIgnoreCase(curOutputType))
				return id;
		}

		return -1;
	}

	/**
	 * Determines if the passed string is an element of the passed vector.
	 */
	private boolean isInVector(String string, Vector<String> vector) {
		if (vector == null || vector.size() == 0)
			return true;
		else {
			boolean found = false;
			for (String curElement : vector) {
				if (curElement.equalsIgnoreCase(string)) {
					found = true;
					break;
				}
			}
			return found;
		}
	}

	/**
	 * This method returns a dictionary containing the signatures (Input Type,
	 * Input Extension, Output Type, Output Extension) of any matching output
	 * converter using the converter ids as keys. An output converter is
	 * included in the dictionary if it accepts a type listed in the
	 * additionally passed vector. If null or an empty vector is passed, all
	 * output converters are included in the dictionary.
	 */
	public HashMap<Integer, ConverterSignature> getOutputConverterSignatures(
			Vector<String> inputTypeFilter) {
		HashMap<Integer, ConverterSignature> result = new HashMap<Integer, ConverterSignature>();

		for (int id = 0; id < this.outputConverterXMLNodes.getLength(); id++) {
			String curInputType = this.getOutputConverterInputTypeByID(id);
			if (this.isInVector(curInputType, inputTypeFilter)) {
				ConverterSignature curSignatrue = new ConverterSignature(id,
						curInputType,
						this.getOutputConverterInputSuffixByID(id),
						this.getOutputConverterOutputTypeByID(id),
						this.getOutputConverterOutputSuffixByID(id));
				result.put(id, curSignatrue);

			}
		}

		return result;
	}

	/**
	 * This method determines the id of the first output converter excepting the
	 * passed type as its input.
	 */
	public int getOutputConverterIDByOutputType(String type) {
		Vector<String> types = this.getValuesOfAllTextNodesBelowXPathNodes(
				this.document,
				"/settings/tools/converter[@type='output']/output[1]/type[1]");
		for (int curConverterIndex = 0; curConverterIndex < types.size(); curConverterIndex++)
			if (types.get(curConverterIndex).equalsIgnoreCase(type))
				return curConverterIndex;
		return -1;
	}

	/**
	 * This method determines the id of the first input converter excepting the
	 * passed type as its input.
	 */
	public int getInputConverterIDByInputType(String type) {
		Vector<String> types = this.getValuesOfAllTextNodesBelowXPathNodes(
				this.document,
				"/settings/tools/converter[@type='input']/input[1]/type[1]");
		for (int curConverterIndex = 0; curConverterIndex < types.size(); curConverterIndex++)
			if (types.get(curConverterIndex).equalsIgnoreCase(type))
				return curConverterIndex;
		return -1;
	}

	/**
	 * This method returns the requirements of the specified program.
	 */
	public Vector<String> getProgrammRequirementsByID(int id) {
		Vector<String> allRequirements = this
				.getValuesOfAllTextNodesBelowXPathNodes(
						this.programXMLNodes.item(id), "general/requires");
		allRequirements.addAll(this.getValuesOfAllTextNodesBelowXPathNodes(
				this.document, "/settings/software/requires"));
		return this.flattenRequirements(allRequirements);
	}

	/**
	 * This method returns the requirements of the specified input converter.
	 */
	public Vector<String> getInputConverterRequirementsByID(int id) {
		Vector<String> allRequirements = this
				.getValuesOfAllTextNodesBelowXPathNodes(
						this.inputConverterXMLNodes.item(id), "requires");
		allRequirements.addAll(this.getValuesOfAllTextNodesBelowXPathNodes(
				this.document, "/settings/tools/requires"));
		return this.flattenRequirements(allRequirements);
	}

	/**
	 * This method returns the requirements of the specified output converter.
	 */
	public Vector<String> getOutputConverterRequirementsByID(int id) {
		Vector<String> allRequirements = this
				.getValuesOfAllTextNodesBelowXPathNodes(
						this.outputConverterXMLNodes.item(id), "requires");
		allRequirements.addAll(this.getValuesOfAllTextNodesBelowXPathNodes(
				this.document, "/settings/tools/requires"));

		return this.flattenRequirements(allRequirements);
	}

	/**
	 * This methods flattens the passed vector of string into one vector of
	 * string containing each element only once.
	 */
	public Vector<String> flattenRequirements(Vector<String> requirements) {
		Vector<String> result = new Vector<String>();
		for (String curLine : requirements) {
			String[] curLineRequirements = curLine.split(",");
			for (String curRequirement : curLineRequirements) {
				String curRequirementTrimmed = curRequirement.trim();
				// Check if already existing
				boolean found = false;
				for (String curResultRequirement : result) {
					if (curRequirementTrimmed
							.equalsIgnoreCase(curResultRequirement)) {
						found = true;
						break;
					}
				}
				if (!found) {
					result.add(curRequirementTrimmed);
				}
			}
		}

		return result;
	}

	/**
	 * This method returns the name of the program identified by the passed id.
	 */
	public String getProgrammNameByID(int id) {
		return this.getValuesOfFirstTextNodeBelowXPathNodes(
				this.programXMLNodes.item(id), "general[1]/name[1]");
	}

	/**
	 * This method returns the output types of the specified program.
	 */
	public Vector<String> getProgramOutputTypesByID(int id) {
		return this.getValuesOfAllTextNodesBelowXPathNodes(
				this.programXMLNodes.item(id), "output/type");
	}

	/**
	 * This method returns a vector of string containing all input types the
	 * program specified by the passed id accepts.
	 */
	public Vector<String> getProgramInputTypesByID(int id) {
		return this.getValuesOfAllTextNodesBelowXPathNodes(
				this.programXMLNodes.item(id), "input/type[1]");
	}

	/**
	 * This method returns the input parameter for the specified file type of
	 * the program specified by the passed id.
	 */
	public String getProgrammInputParameterByFileType(int id, String type) {
		return this.getValuesOfFirstTextNodeBelowXPathNodes(
				this.programXMLNodes.item(id), "input[type='" + type
						+ "'][1]/parameter[1]");
	}

	/**
	 * This method returns the converter binary that should be run on inputs of
	 * the specified file type before the program specified by the passed id can
	 * be executed.
	 */
	public String getProgrammInputConverterBinaryByFileType(int id, String type) {
		return this.getValuesOfFirstTextNodeBelowXPathNodes(
				this.programXMLNodes.item(id), "input[type='" + type
						+ "'][1]/converter[1]");
	}

	/**
	 * This method returns the converter binary that will be run on inputs of
	 * the program specified by the passed id.
	 */
	public String getProgrammOutputConverterBinaryByProgrammID(int id) {
		return this.getValuesOfFirstTextNodeBelowXPathNodes(
				this.programXMLNodes.item(id), "output[1]/converter[1]");
	}

	/**
	 * This method determines the name of the binary (only the name, not the
	 * path) of the program specified by the passed id.
	 */
	public String getProgrammBinaryNameByID(int id) {
		File binary = new File(this.getProgrammBinaryByID(id));
		return binary.getName();
	}

	/**
	 * This method determines the path of the binary (only the path, not the
	 * name) of the program specified by the passed id.
	 */
	public String getProgrammBinaryPathByID(int id) {
		String name = this.getProgrammBinaryByID(id);
		return name.substring(0, name.lastIndexOf("/") + 1);
	}

	/**
	 * This method determines the path of the binary of the program specified by
	 * the passed id.
	 */
	private String getProgrammBinaryByID(int id) {
		return this.getValuesOfFirstTextNodeBelowXPathNodes(
				this.programXMLNodes.item(id), "general[1]/binary[1]");
	}

	/**
	 * This method returns the name of the binary of the input parameter by the
	 * passed id.
	 */
	public String getInputConverterBinaryByID(int id) {
		return this.getValuesOfFirstTextNodeBelowXPathNodes(
				this.inputConverterXMLNodes.item(id), "binary[1]");
	}

	/**
	 * This method returns the name of the binary of the input parameter by the
	 * passed id.
	 */
	public String getOutputConverterBinaryByID(int id) {
		return this.getValuesOfFirstTextNodeBelowXPathNodes(
				this.outputConverterXMLNodes.item(id), "binary[1]");
	}

	/**
	 * This method return the input type of the output converter specified by
	 * the passed id.
	 */
	public String getOutputConverterInputTypeByID(int id) {
		return this.getValuesOfFirstTextNodeBelowXPathNodes(
				this.outputConverterXMLNodes.item(id), "input[1]/type[1]");
	}

	/**
	 * This method return the input suffix of the output converter specified by
	 * the passed id.
	 */
	private String getOutputConverterInputSuffixByID(int id) {
		return this.getValuesOfFirstTextNodeBelowXPathNodes(
				this.outputConverterXMLNodes.item(id), "input[1]/suffix[1]");
	}

	/**
	 * This method return the output type of the output converter specified by
	 * the passed id.
	 */
	public String getOutputConverterOutputTypeByID(int id) {
		return this.getValuesOfFirstTextNodeBelowXPathNodes(
				this.outputConverterXMLNodes.item(id), "output[1]/type[1]");
	}

	public String getOutputConverterOutputSuffixByID(int id) {
		return this.getValuesOfFirstTextNodeBelowXPathNodes(
				this.outputConverterXMLNodes.item(id), "output[1]/suffix[1]");
	}

	/**
	 * This method return the output type of the input converter specified by
	 * the passed id.
	 */
	public String getInputConverterOutputTypeByID(int id) {
		return this.getValuesOfFirstTextNodeBelowXPathNodes(
				this.inputConverterXMLNodes.item(id), "output[1]/type[1]");
	}

	public String getInputConverterOutputSuffixByID(int id) {
		return this.getValuesOfFirstTextNodeBelowXPathNodes(
				this.inputConverterXMLNodes.item(id), "output[1]/suffix[1]");
	}

	/**
	 * This method return the input type of the input converter specified by the
	 * passed id.
	 */
	public String getInputConverterInputTypeByID(int id) {
		return this.getValuesOfFirstTextNodeBelowXPathNodes(
				this.inputConverterXMLNodes.item(id), "input[1]/type[1]");
	}

	public String getInputConverterInputSuffixByID(int id) {
		return this.getValuesOfFirstTextNodeBelowXPathNodes(
				this.inputConverterXMLNodes.item(id), "input[1]/suffix[1]");
	}

	/**
	 * This method reads out the configuration value specified by the passed
	 * tag.
	 */
	public Vector<String> getConfigValue(String tag) {
		return this.getValuesOfAllTextNodesBelowXPathNodes(this.document,
				"/settings[1]/config[1]/" + tag);
	}

	/**
	 * Returns the default extension for files within the workflow.
	 */
	public String getDefaultExtension() {
		Vector<String> extension = this.getConfigValue("defaultsuffix");
		if (extension.size() > 0)
			return extension.get(0);
		else
			return ".xml";
	}

	/**
	 * This method returns the name of the parameter the program specified by
	 * the passed id expects its output file name in.
	 */
	public String getProgrammOutputParameterByID(int id) {
		return this.getValuesOfFirstTextNodeBelowXPathNodes(
				this.programXMLNodes.item(id), "output[1]/parameter[1]");
	}

	/**
	 * This method generates and returns all parameter group objects for the job
	 * specified by the passed id.
	 */
	public Vector<ParameterGroup> generateProgramParameterGroups(int id) {
		Vector<ParameterGroup> result = new Vector<ParameterGroup>();

		NodeList programParameterGroups = this.getNodeListOfXPathExpression(
				this.programXMLNodes.item(id), "parameterlist/parametergroup");
		for (int curParameterGroupPos = 0; curParameterGroupPos < programParameterGroups
				.getLength(); curParameterGroupPos++) {
			result.add(this.generateParameterGroup(programParameterGroups
					.item(curParameterGroupPos)));
		}
		this.createParameterGroupReferences(result);

		return result;
	}

	/**
	 * This method generates and returns the text parameter represented by the
	 * passed node.
	 */
	private TextParameter generateTextParameter(Node curNode) {
		String type = this.getStringOfXPathExpression(curNode, "@type[1]");
		String name = this.getValuesOfFirstTextNodeBelowXPathNodes(curNode,
				"name[1]");
		String description = this.getValuesOfFirstTextNodeBelowXPathNodes(
				curNode, "description[1]");
		String successor = this.getValuesOfFirstTextNodeBelowXPathNodes(
				curNode, "successor[1]");

		if (type.equalsIgnoreCase("dynamic")) {
			String dynamicpart = this.getValuesOfFirstTextNodeBelowXPathNodes(
					curNode, "dynamicpart[1]");
			String dynamicref = this.getValuesOfFirstTextNodeBelowXPathNodes(
					curNode, "dynamicref[1]");
			return new DynamicTextParameter(name, description, successor,
					dynamicpart, dynamicref);
		} else
			// if (type.equalsIgnoreCase("static")) {
			return new StaticTextParameter(name, description, successor);
	}

	/**
	 * This method generates and returns the parameter group represented by the
	 * passed node.
	 */
	private ParameterGroup generateParameterGroup(Node curNode) {
		ParameterGroup result = null;

		// Read values from xml
		String successor = this.getValuesOfFirstTextNodeBelowXPathNodes(
				curNode, "successor[1]");
		String name = this.getValuesOfFirstTextNodeBelowXPathNodes(curNode,
				"name[1]");
		String description = this.getValuesOfFirstTextNodeBelowXPathNodes(
				curNode, "description[1]");
		String key = this.getValuesOfFirstTextNodeBelowXPathNodes(curNode,
				"key[1]");
		boolean isVisible = this.getStringOfXPathExpression(curNode,
				"@visible[1]").equalsIgnoreCase("true");
		boolean isFixed = this.getStringOfXPathExpression(curNode, "@fixed[1]")
				.equalsIgnoreCase("true");
		String type = this.getStringOfXPathExpression(curNode, "@type[1]");

		// bool
		if (type.equalsIgnoreCase("bool")) {
			boolean defaultValue = this
					.getValuesOfFirstTextNodeBelowXPathNodes(curNode, "default")
					.equalsIgnoreCase("1");
			result = new BooleanParameterGroup(name, key, defaultValue,
					description, successor, isVisible);
		} else {
			boolean isDisabled = this.getStringOfXPathExpression(curNode,
					"@disabled[1]").equalsIgnoreCase("true");
			// filename
			if (type.equalsIgnoreCase("filename")) {
				boolean isMulti = this.getStringOfXPathExpression(curNode,
						"@multi[1]").equalsIgnoreCase("true");
				result = new FileNameParameterGroup(name, key, description,
						successor, isVisible, isMulti);
			}
			// select
			else if (type.equalsIgnoreCase("select")) {
				Vector<String> options = this
						.getValuesOfAllTextNodesBelowXPathNodes(curNode,
								"values/value");
				String defaultValue = this
						.getValuesOfFirstTextNodeBelowXPathNodes(curNode,
								"default").trim();
				result = new SelectParameterGroup(name, key, description,
						successor, isVisible, !isDisabled, options,
						defaultValue);
			}
			// text
			else {
				Vector<TextParameter> parameters = new Vector<TextParameter>();
				NodeList parameterNodes = this.getNodeListOfXPathExpression(
						curNode, "parameter");
				for (int curParameterPos = 0; curParameterPos < parameterNodes
						.getLength(); curParameterPos++) {
					Node curParameterNode = parameterNodes
							.item(curParameterPos);
					parameters
							.add(this.generateTextParameter(curParameterNode));
				}
				String defaultValue = this
						.getValuesOfFirstTextNodeBelowXPathNodes(curNode,
								"default");
				result = new TextParameterGroup(name, key, description,
						successor, isVisible, !isDisabled, parameters);

				((TextParameterGroup) result).createReferences();
				((TextParameterGroup) result).setDefaultValue(defaultValue);

			}

			result.setEnable(!isDisabled);
			result.setFixed(isFixed);
		}
		return result;
	}

	/**
	 * This method sets the successor reference of the passed parameter groups
	 * according to the textual value of the corresponding field.
	 */
	private void createParameterGroupReferences(Vector<ParameterGroup> groups) {
		for (ParameterGroup curParameterGroup : groups) {
			ParameterGroup curSuccessor = SmallHelpers
					.findParameterGroupByName(
							curParameterGroup.getSuccessorName(), groups);
			curParameterGroup.setSuccessor(curSuccessor);
		}

	}

	public Vector<String> getRequirements(Vector<Integer> usedProgrammIDs,
			Vector<Integer> usedInputConverterIDs,
			Vector<Integer> usedOutputConverterIDs) {
		Vector<String> result = new Vector<String>();
		for (int curID : usedProgrammIDs) {
			result.addAll(this.getProgrammRequirementsByID(curID));
		}
		for (int curID : usedInputConverterIDs) {
			result.addAll(this.getInputConverterRequirementsByID(curID));
		}
		for (int curID : usedOutputConverterIDs) {
			result.addAll(this.getOutputConverterRequirementsByID(curID));
		}

		return this.flattenRequirements(result);
	}

	@Override
	protected void performSemanticChecks() {
		// Check software source
		String softwareSource = this.getConfigValue("softwaresource").get(0);
		softwareSource = SmallHelpers.replaceVariables(softwareSource);
		// Only verify folders without any variables
		if (softwareSource.indexOf("$") == -1) {
			File softwareSourceFile = new File(softwareSource);
			if (!softwareSourceFile.exists() || !softwareSourceFile.canRead()) {
				JOptionPane.showMessageDialog(null,
						"You do not have the rights to access the Softwaresource directory ("
								+ softwareSourceFile.getPath()
								+ ") or the directory does not exist.",
						"Warning", JOptionPane.WARNING_MESSAGE);
			}

		}
	}


}
