/*
 * 
 * ProFiDo - The Processes Fitting Toolkit Dortmund
 * Copyright (C) 2010 - 2012 LS Informatik IV, TU Dortmund
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

package gui;

import gui.helper.ProFiDoTextField;

import java.awt.Choice;
import java.awt.Rectangle;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;

import core.graph.Edge;
import core.graph.GraphNode;
import core.history.ChangeEdgeInformationVisibiltyHistoryObject;
import core.history.SetEdgeOrderKeyHistoryObject;
import core.history.SetEdgeTypeHistoryObject;
import core.history.SetFileNameHistoryObject;

/**
 * This class defines the properties of the "Edit File" panel.
 * 
 * It is used to change file name and type of an edge. The file name can be
 * entered in a text field while the type can be selected from a choice box. The
 * listed file types are the permitted types for a connection between two nodes.
 * A click on the "OK" button confirms the changes, stores the new values,
 * closes the window and updates the information over the edge. A click on the
 * "Last Values" button reverts the changes to the last saved version. A click
 * on the "Cancel" button cancels the window without performing any kind of
 * change.
 */

public class EditFilePanel extends JPanel {

	private static final long serialVersionUID = 1L;

	/**
	 * Height which this panel needs in order to be displayed correctly. The
	 * value is depended on the amount and type of the subpanels.
	 */
	protected int desiredHeight;

	/**
	 * Reference to the workflow panel.
	 */
	private WorkflowPanel workflowPanel;

	/**
	 * Reference to the frame this panel is on.
	 */
	private EditFileFrame editFileFrame;

	/**
	 * Checkbox used to choose if this edges file name is visible.
	 */
	protected JCheckBox fileNameVisibleCheckBox = null;

	/**
	 * Checkbox used to choose if this edges file type is visible.
	 */
	protected JCheckBox fileTypeVisibleCheckBox = null;

	/**
	 * Checkbox used to choose if this edges order key is visible.
	 */
	protected JCheckBox orderKeyVisibleCheckBox = null;

	/**
	 * Button used to close the EditFilePanel. All changes will be saved.
	 */
	private JButton okButton = null;

	/**
	 * Button used to close the EditFilePanel. All changes get lost.
	 */
	private JButton cancelButton = null;

	/**
	 * Button used to set the default value.
	 */
	private JButton defaultValueButton = null;

	/**
	 * Button used to set the last value.
	 */
	private JButton lastValueButton = null;

	/**
	 * JLabel that represents the name of the file.
	 */
	private JLabel nameLabel = null;

	/**
	 * JLabel that represents the type of the file.
	 */
	private JLabel typeLabel = null;

	/**
	 * JLabel that represents the order.
	 */
	private JLabel orderTextFieldLabel = null;

	/**
	 * JTextField used to be change the filename.
	 */
	private JTextField nameTextField = null;

	/**
	 * JTextField used to change the orderKey.
	 */
	private JTextField orderTextField = null;

	/**
	 * Choice box displaying the possible file types of this panels edge.
	 */
	private Choice typeChoice;

	/**
	 * Edge this panel is assigned to.
	 */
	private Edge edge;

	/**
	 * Creates and initializes the panel.
	 */
	public EditFilePanel(Edge edge, WorkflowPanel workflowPanel,
			EditFileFrame editFileFrame) {
		super();
		this.edge = edge;
		this.workflowPanel = workflowPanel;
		this.editFileFrame = editFileFrame;

		this.initialize();
		this.writeInformation();
	}

	/**
	 * Initializes all components used by this panel and adds them to its
	 * content pane.
	 */
	private void initialize() {
		this.setSize(300, 500);
		this.setLayout(null);
		this.add(this.getOkButton(), null);
		this.add(this.getCancelButton(), null);
		this.add(this.getDefaultValueButton(), null);
		this.add(this.getLastValueButton(), null);
		this.add(this.getPrefixTextField(), null);
		this.add(this.getOrderTextField(), null);
		this.add(this.getNameTextField(), null);
		this.add(this.getPrefixChoiceField(), null);
		this.add(this.getOrderTextFieldLabel(), null);
		this.add(this.getChoiceField(), null);
		this.add(this.getFileNameVisibleCheckBox(), null);
		this.add(this.getFileTypeVisibleCheckBox(), null);
		this.add(this.getOrderKeyVisibleCheckBox(), null);

		this.addComponentListener(new ComponentAdapter() {
			@Override
			public void componentResized(ComponentEvent evt) {
				EditFilePanel.this.updatePositions();
			}
		});

		this.updatePositions();
	}

	private Choice getChoiceField() {
		if (this.typeChoice == null) {
			this.typeChoice = new Choice();
			for (String curType : this.edge.getPossibleTypes()) {
				this.typeChoice.add(curType);
			}
		}
		return this.typeChoice;

	}

	/**
	 * Updates the position of all components on this panel.
	 */
	public void updatePositions() {
		// int curTopPosition = 2;

		// Define Margins
		int topMargin = 4;
		int leftMargin = 5;
		int rightMargin = 5;
		int bottomMargin = 5;

		// Define component sizes
		int buttonLeftSpacing = 5;
		int labelWidth = 66;
		int checkBoxWidth = 80;

		// int panelTopSpacing = 2;
		int buttonTopSpacing = 6;
		int buttonHeigth = 20;
		int textFieldHeight = 28;

		int checkBoxHeight = 23;
		int componentLeftSpacing = 2;

		int contentWidth = this.getWidth() - (leftMargin + rightMargin);

		int textFieldWidth = contentWidth - labelWidth - checkBoxWidth - 2
				* componentLeftSpacing;

		if (this.nameLabel != null) {
			this.nameLabel.setBounds(new Rectangle(leftMargin, topMargin,
					labelWidth, textFieldHeight));
		}

		if (this.nameTextField != null) {
			this.nameTextField.setBounds(new Rectangle(leftMargin + labelWidth
					+ componentLeftSpacing, topMargin, textFieldWidth,
					textFieldHeight));
		}

		if (this.fileNameVisibleCheckBox != null) {
			this.fileNameVisibleCheckBox.setBounds(new Rectangle(leftMargin
					+ labelWidth + textFieldWidth + 2 * componentLeftSpacing,
					topMargin, checkBoxWidth, checkBoxHeight));
		}

		int secondRowTopPosition = this.nameTextField.getY()
				+ this.nameTextField.getHeight() + buttonTopSpacing;

		if (this.typeLabel != null) {
			this.typeLabel.setBounds(new Rectangle(leftMargin,
					secondRowTopPosition, labelWidth, textFieldHeight));
		}

		if (this.typeChoice != null) {
			this.typeChoice.setBounds(new Rectangle(leftMargin + labelWidth
					+ componentLeftSpacing, secondRowTopPosition,
					textFieldWidth, textFieldHeight));
		}

		if (this.fileTypeVisibleCheckBox != null) {
			this.fileTypeVisibleCheckBox.setBounds(new Rectangle(leftMargin
					+ labelWidth + textFieldWidth + 2 * componentLeftSpacing,
					secondRowTopPosition, checkBoxWidth, checkBoxHeight));
		}

		int thirdRowTopPosition = this.typeChoice.getY()
				+ this.typeChoice.getHeight() + buttonTopSpacing;

		if (this.orderTextFieldLabel != null) {
			this.orderTextFieldLabel.setBounds(new Rectangle(leftMargin,
					thirdRowTopPosition, textFieldWidth, textFieldHeight));
		}

		if (this.orderTextField != null) {
			this.orderTextField.setBounds(new Rectangle(leftMargin + labelWidth
					+ componentLeftSpacing, thirdRowTopPosition,
					textFieldWidth, textFieldHeight));
		}

		if (this.orderKeyVisibleCheckBox != null) {
			this.orderKeyVisibleCheckBox.setBounds(new Rectangle(leftMargin
					+ labelWidth + textFieldWidth + 2 * componentLeftSpacing,
					thirdRowTopPosition, checkBoxWidth, checkBoxHeight));
		}

		int fourthRowTopPosition = this.orderTextField.getY()
				+ this.orderTextField.getHeight() + buttonTopSpacing;
		int buttonWidth = (contentWidth - 3 * buttonLeftSpacing) / 4;

		if (this.okButton != null) {
			this.okButton.setBounds(new Rectangle(leftMargin,
					fourthRowTopPosition, buttonWidth, buttonHeigth));
		}

		if (this.lastValueButton != null) {
			this.lastValueButton.setBounds(new Rectangle(leftMargin
					+ buttonWidth + buttonLeftSpacing, fourthRowTopPosition,
					buttonWidth, buttonHeigth));
		}

		if (this.defaultValueButton != null) {
			this.defaultValueButton.setBounds(new Rectangle(leftMargin + 2
					* buttonWidth + 2 * buttonLeftSpacing,
					fourthRowTopPosition, buttonWidth, buttonHeigth));
		}

		if (this.cancelButton != null) {
			this.cancelButton.setBounds(new Rectangle(leftMargin + 3
					* buttonWidth + 3 * buttonLeftSpacing,
					fourthRowTopPosition, buttonWidth, buttonHeigth));
		}

		this.desiredHeight = this.okButton.getY() + this.okButton.getHeight()
				+ bottomMargin;
	}

	public int getDesiredHeight() {
		return this.desiredHeight;
	}

	/**
	 * This method creates the ok Button.
	 */
	private JButton getOkButton() {
		if (this.okButton == null) {
			this.okButton = new JButton();
			this.okButton.setBounds(new Rectangle(9, 134, 75, 29));
			this.okButton.setText("OK");
			this.okButton
					.addActionListener(new java.awt.event.ActionListener() {
						@Override
						public void actionPerformed(java.awt.event.ActionEvent e) {
							String oldTyp = EditFilePanel.this.edge
									.getEdgeFileType();
							String oldName = EditFilePanel.this.edge
									.getFileName().getPath();
							String oldOrderKey = EditFilePanel.this.edge
									.getOrderKey();

							// Check if new name is valid
							GraphNode nodeWithSameName = EditFilePanel.this.workflowPanel
									.getGraph().getNodeWithSameFileName(
											EditFilePanel.this.nameTextField
													.getText());
							if (!oldName
									.equals(EditFilePanel.this.nameTextField
											.getText())
									&& nodeWithSameName != null) {
								JOptionPane.showMessageDialog(
										EditFilePanel.this.workflowPanel,
										"The new file name is already used by node "
												+ nodeWithSameName.getName()
												+ ".", "Error",
										JOptionPane.ERROR_MESSAGE);
							} else {
								if (!oldName
										.equals(EditFilePanel.this.nameTextField
												.getText())) {
									EditFilePanel.this.workflowPanel
											.getGraph()
											.addNewAction(
													new SetFileNameHistoryObject(
															EditFilePanel.this.edge,
															EditFilePanel.this.nameTextField
																	.getText()));
									EditFilePanel.this.workflowPanel.getGraph()
											.performLastAction();
								}

								if (!oldTyp
										.equals(EditFilePanel.this.typeChoice
												.getSelectedItem())) {
									EditFilePanel.this.workflowPanel
											.getGraph()
											.addNewAction(
													new SetEdgeTypeHistoryObject(
															EditFilePanel.this.edge,
															EditFilePanel.this.typeChoice
																	.getSelectedItem()));
									EditFilePanel.this.workflowPanel.getGraph()
											.performLastAction();
								}

								if (!oldOrderKey
										.equals(EditFilePanel.this.orderTextField
												.getText())) {
									EditFilePanel.this.workflowPanel
											.getGraph()
											.addNewAction(
													new SetEdgeOrderKeyHistoryObject(
															EditFilePanel.this.edge,
															EditFilePanel.this.orderTextField
																	.getText()));
									EditFilePanel.this.workflowPanel.getGraph()
											.performLastAction();
								}

								if (EditFilePanel.this.fileNameVisibleCheckBox
										.isSelected() != EditFilePanel.this.edge
										.isFileNameVisible()
										|| EditFilePanel.this.fileTypeVisibleCheckBox
												.isSelected() != EditFilePanel.this.edge
												.isFileTypeVisible()
										|| EditFilePanel.this.orderKeyVisibleCheckBox
												.isSelected() != EditFilePanel.this.edge
												.isOrderKeyVisible()) {
									EditFilePanel.this.workflowPanel
											.getGraph()
											.addNewAction(
													new ChangeEdgeInformationVisibiltyHistoryObject(
															EditFilePanel.this.edge,
															EditFilePanel.this.fileNameVisibleCheckBox
																	.isSelected(),
															EditFilePanel.this.fileTypeVisibleCheckBox
																	.isSelected(),
															EditFilePanel.this.orderKeyVisibleCheckBox
																	.isSelected()));
									EditFilePanel.this.workflowPanel.getGraph()
											.performLastAction();
								}

								EditFilePanel.this.editFileFrame.finished();
							}
						}
					});
		}
		return this.okButton;
	}

	/**
	 * This method creates the cancel Button.
	 */
	private JButton getCancelButton() {
		if (this.cancelButton == null) {
			this.cancelButton = new JButton();
			this.cancelButton.setBounds(new Rectangle(81, 131, 75, 29));
			this.cancelButton.setText("Cancel");
			this.cancelButton
					.addActionListener(new java.awt.event.ActionListener() {
						@Override
						public void actionPerformed(java.awt.event.ActionEvent e) {
							EditFilePanel.this.editFileFrame.finished();
						}
					});
		}
		return this.cancelButton;
	}

	/**
	 * This method creates the defaultValueButton.
	 */
	private JButton getDefaultValueButton() {
		if (this.defaultValueButton == null) {
			this.defaultValueButton = new JButton();
			this.defaultValueButton.setBounds(new Rectangle(152, 131, 75, 29));
			this.defaultValueButton.setText("Generate");
			this.defaultValueButton
					.addActionListener(new java.awt.event.ActionListener() {
						@Override
						public void actionPerformed(java.awt.event.ActionEvent e) {
							// TODO Implement function
						}
					});
		}
		return this.defaultValueButton;
	}

	/**
	 * This method creates the lastValueButton.
	 */
	private JButton getLastValueButton() {
		if (this.lastValueButton == null) {
			this.lastValueButton = new JButton();
			this.lastValueButton.setText("Last Values");
			this.lastValueButton.setBounds(new Rectangle(187, 170, 75, 29));
			this.lastValueButton
					.addActionListener(new java.awt.event.ActionListener() {
						@Override
						public void actionPerformed(java.awt.event.ActionEvent e) {
							EditFilePanel.this.writeInformation();
						}
					});
		}
		return this.lastValueButton;
	}

	/**
	 * This method writes the file name and file type on this panel.
	 */
	private void writeInformation() {
		this.orderTextField.setText(this.edge.getOrderKey());
		this.nameTextField.setText(this.edge.getFileName().getPath());
		this.typeChoice.select(this.edge.getEdgeFileType());
		this.orderKeyVisibleCheckBox.setSelected(this.edge.isOrderKeyVisible());
		this.fileTypeVisibleCheckBox.setSelected(this.edge.isFileTypeVisible());
		this.fileNameVisibleCheckBox.setSelected(this.edge.isFileNameVisible());
	}

	/**
	 * This method creates the nameLabel.
	 */
	private JLabel getPrefixTextField() {
		if (this.nameLabel == null) {
			this.nameLabel = new JLabel();
			this.nameLabel.setText("File Name");
		}
		return this.nameLabel;
	}

	/**
	 * This method creates the nameTextField.
	 */
	private JTextField getNameTextField() {
		if (this.nameTextField == null) {
			this.nameTextField = new ProFiDoTextField();
			this.nameTextField.setBounds(new Rectangle(98, 63, 14, 28));
		}
		return this.nameTextField;
	}

	/**
	 * This method creates the orderTextField which is necessary to change the
	 * orderKey.
	 */
	private JTextField getOrderTextField() {
		if (this.orderTextField == null) {
			this.orderTextField = new ProFiDoTextField();
			this.orderTextField.setBounds(new Rectangle(98, 63, 14, 28));
		}
		return this.orderTextField;
	}

	/**
	 * This method creates the orderTextFieldLabel.
	 */
	private JLabel getOrderTextFieldLabel() {
		if (this.orderTextFieldLabel == null) {
			this.orderTextFieldLabel = new JLabel();
			this.orderTextFieldLabel.setText("Order Key");
		}
		return this.orderTextFieldLabel;
	}

	/**
	 * This method creates the typeLabel.
	 */
	private JLabel getPrefixChoiceField() {
		if (this.typeLabel == null) {
			this.typeLabel = new JLabel();
			this.typeLabel.setText("File Type");
		}
		return this.typeLabel;
	}

	/**
	 * This method creates the fileNameVisibleCheckBox.
	 */
	private JCheckBox getFileNameVisibleCheckBox() {
		if (this.fileNameVisibleCheckBox == null) {
			this.fileNameVisibleCheckBox = new JCheckBox();
			this.fileNameVisibleCheckBox.setText("Visible");
			this.fileNameVisibleCheckBox.setSelected(this.edge
					.isFileNameVisible());
			this.fileNameVisibleCheckBox.setBounds(new Rectangle(307, 35, 76,
					23));
		}
		return this.fileNameVisibleCheckBox;
	}

	/**
	 * This method creates the fileTypeVisibleCheckBox.
	 */
	private JCheckBox getFileTypeVisibleCheckBox() {
		if (this.fileTypeVisibleCheckBox == null) {
			this.fileTypeVisibleCheckBox = new JCheckBox();
			this.fileTypeVisibleCheckBox.setText("Visible");
			this.fileTypeVisibleCheckBox.setSelected(this.edge
					.isFileTypeVisible());
			this.fileTypeVisibleCheckBox.setBounds(new Rectangle(307, 35, 76,
					23));
		}
		return this.fileTypeVisibleCheckBox;
	}

	/**
	 * This method creates the orderKeyVisibleCheckBox which is needed to change
	 * the visibility of the orderKey.
	 */
	private JCheckBox getOrderKeyVisibleCheckBox() {
		if (this.orderKeyVisibleCheckBox == null) {
			this.orderKeyVisibleCheckBox = new JCheckBox();
			this.orderKeyVisibleCheckBox.setText("Visible");
			this.orderKeyVisibleCheckBox.setSelected(this.edge
					.isOrderKeyVisible());
			this.orderKeyVisibleCheckBox.setBounds(new Rectangle(307, 35, 76,
					23));
		}
		return this.orderKeyVisibleCheckBox;
	}

}
