/*
 * 
 * ProFiDo - The Processes Fitting Toolkit Dortmund
 * Copyright (C) 2010 - 2012 LS Informatik IV, TU Dortmund
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

package gui;

import gui.helper.InstantViewManager;

import java.awt.Rectangle;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.util.Vector;

import javax.swing.JButton;
import javax.swing.JPanel;

import core.BooleanParameterGroup;
import core.FileNameParameterGroup;
import core.ParameterGroup;
import core.SelectParameterGroup;
import core.TextParameterGroup;
import core.graph.JobNode;
import core.history.HistoryObject;

/**
 * This class defines the properties of the "Edit Job" panel.
 * 
 * This panel displays all parameter groups of the corresponding job. It
 * consists of a panel for each of the jobs parameter groups. The panels are of
 * type EditOneParameterPanel, EditOneTextParameterPanel or
 * EditOneBooleanParameterPanel, depending on the type of the corresponding
 * ParameterGroup.
 * 
 * A click on the "OK" button confirms the changes, stores the new values,
 * closes the window and updates the information on the canvas. A click on the
 * "Last Values" button reverts the changes to the last saved version. A click
 * on the "Cancel" button cancels the current operation without performing any
 * kind of change.
 */

public class EditJobPanel extends JPanel {

	private static final long serialVersionUID = 1L;

	/**
	 * Vector of all subpanels representing the nodes parameter groups.
	 */
	private Vector<EditOneParameterPanel> parameterPanels;

	/**
	 * Job node which this frame is assigned to.
	 */
	private JobNode correspondingJob;

	/**
	 * Height which this panel needs in order to be displayed correctly. The
	 * value is depended on the amount and type of the subpanels.
	 */
	protected int desiredHeight;

	/**
	 * Reference to the workflow panel.
	 */
	private WorkflowPanel workflowPanel;

	/**
	 * Reference to the ok Button which will close the EditJobPanel and saves
	 * all changes.
	 */
	private JButton okButton = null;

	/**
	 * Reference to the cancel Button which will close the EditJobPanel and
	 * discharges all changes.
	 */
	private JButton cancelButton = null;

	/**
	 * Reference to the defaultValueButton which will insert the default values
	 * for the job.
	 */
	private JButton defaultValueButton = null;

	/**
	 * Reference to the lastValueButton.
	 */
	private JButton lastValueButton = null;

	/**
	 * Reference to the frame this panel is on.
	 */
	private EditJobFrame editJobFrame;

	/**
	 * Reference to the instantViewManager which will tell you whether you can
	 * display the job with the instantView feature.
	 */
	private InstantViewManager instantViewManager;

	/**
	 * Creates and initializes the panel.
	 * 
	 * @param instantViewManager
	 */
	public EditJobPanel(JobNode myJob, WorkflowPanel workflowPanel,
			EditJobFrame editJobFrame, InstantViewManager instantViewManager) {
		super();
		this.correspondingJob = myJob;
		this.workflowPanel = workflowPanel;
		this.editJobFrame = editJobFrame;
		this.instantViewManager = instantViewManager;

		this.initialize();
	}

	/**
	 * This method creates the panels for the parameter groups of the JobNode.
	 **/
	private Vector<EditOneParameterPanel> createParameterPanels() {
		if (this.parameterPanels == null) {
			boolean showVisibleCheckBox = this.instantViewManager != null;
			this.parameterPanels = new Vector<EditOneParameterPanel>();
			Vector<ParameterGroup> parameterGroups = this.correspondingJob
					.getParameterGroups();
			for (ParameterGroup curGroup : parameterGroups) {
				if (!(curGroup instanceof FileNameParameterGroup)
						&& !curGroup.isFixed()) {
					EditOneParameterPanel curPanel = null;
					if (curGroup instanceof TextParameterGroup) {
						curPanel = new EditOneTextParameterPanel(
								this.correspondingJob,
								(TextParameterGroup) curGroup, this,
								this.workflowPanel, showVisibleCheckBox);
					} else if (curGroup instanceof BooleanParameterGroup) {
						curPanel = new EditOneBooleanParameterPanel(
								this.correspondingJob,
								(BooleanParameterGroup) curGroup, this,
								this.workflowPanel, showVisibleCheckBox);
					} else if (curGroup instanceof SelectParameterGroup) {
						curPanel = new EditOneSelectParameterPanel(
								this.correspondingJob,
								(SelectParameterGroup) curGroup, this,
								this.workflowPanel, showVisibleCheckBox);
					}

					curPanel.setBounds(new Rectangle(42, 22, 198, 22));
					this.add(curPanel, null);
					this.parameterPanels.add(curPanel);
				}

			}
		}
		this.updatePositions();
		return this.parameterPanels;
	}

	/**
	 * Initializes all components used by this panel and adds them to its
	 * content pane.
	 */
	private void initialize() {
		this.setSize(300, 200);
		this.setLayout(null);
		this.add(this.getOkButton(), null);
		this.add(this.getCancelButton(), null);
		this.add(this.getDefaultValueButton(), null);
		this.add(this.getLastValueButton(), null);
		this.addComponentListener(new ComponentAdapter() {
			@Override
			public void componentResized(ComponentEvent evt) {
				EditJobPanel.this.updatePositions();
			}
		});

		this.createParameterPanels();
	}

	/**
	 * Updates the position of all components on this panel.
	 */
	public void updatePositions() {
		int curTopPosition = 2;

		int panelTopSpacing = 2;
		int buttonTopSpacing = 6;
		int leftMargin = 5;
		int rightMargin = 5;
		int bottomMargin = 5;

		int buttonLeftSpacing = 5;
		int buttonHeigth = 20;

		int contentWidth = this.getWidth() - (leftMargin + rightMargin);

		int curLeftPosition = leftMargin;

		for (EditOneParameterPanel curPanel : this.parameterPanels) {
			curPanel.setBounds(new Rectangle(curLeftPosition, curTopPosition,
					contentWidth, curPanel.getDesiredHeight()));
			curTopPosition = curTopPosition + panelTopSpacing
					+ curPanel.getDesiredHeight();
		}

		int secondRowTopPosition;
		if (this.parameterPanels.size() > 0) {
			EditOneParameterPanel lastPanel = this.parameterPanels
					.get(this.parameterPanels.size() - 1);

			secondRowTopPosition = lastPanel.getY()
					+ lastPanel.getDesiredHeight() + buttonTopSpacing;
		} else {
			secondRowTopPosition = buttonTopSpacing;
		}

		int buttonWidth = (contentWidth - 3 * buttonLeftSpacing) / 4;

		if (this.okButton != null) {
			this.okButton.setBounds(new Rectangle(leftMargin,
					secondRowTopPosition, buttonWidth, buttonHeigth));
		}

		if (this.lastValueButton != null) {
			this.lastValueButton.setBounds(new Rectangle(leftMargin
					+ buttonWidth + buttonLeftSpacing, secondRowTopPosition,
					buttonWidth, buttonHeigth));
		}

		if (this.defaultValueButton != null) {
			this.defaultValueButton.setBounds(new Rectangle(leftMargin + 2
					* buttonWidth + 2 * buttonLeftSpacing,
					secondRowTopPosition, buttonWidth, buttonHeigth));
		}

		if (this.cancelButton != null) {
			this.cancelButton.setBounds(new Rectangle(leftMargin + 3
					* buttonWidth + 3 * buttonLeftSpacing,
					secondRowTopPosition, buttonWidth, buttonHeigth));
		}

		this.desiredHeight = this.okButton.getY() + this.okButton.getHeight()
				+ bottomMargin;
	}

	public int getDesiredHeight() {
		return this.desiredHeight;
	}

	/**
	 * This method creates the ok Button.
	 */
	private JButton getOkButton() {
		if (this.okButton == null) {
			this.okButton = new JButton();
			this.okButton.setBounds(new Rectangle(9, 134, 75, 29));
			this.okButton.setText("OK");
			this.okButton
					.addActionListener(new java.awt.event.ActionListener() {
						@Override
						public void actionPerformed(java.awt.event.ActionEvent e) {
							Vector<HistoryObject> changes = new Vector<HistoryObject>();

							// Iterate all parameters
							for (EditOneParameterPanel curPanel : EditJobPanel.this.parameterPanels) {

								if (curPanel instanceof EditOneBooleanParameterPanel) {
									// check if change occoured
									Vector<HistoryObject> curPanelCheckBoxHistoryObjects = ((EditOneBooleanParameterPanel) curPanel)
											.createHistoryObjectToInheritTemporaryBooleanParameterCheckBoxValues();
									changes.addAll(curPanelCheckBoxHistoryObjects);

								} else if (curPanel instanceof EditOneSelectParameterPanel) {
									// check if selection changed
									HistoryObject curPanelHistoryObject = curPanel
											.createHistoryObjectToInheritTemporaryValues();
									if (curPanelHistoryObject != null) {
										changes.add(curPanelHistoryObject);
									}

									// check if check boxes changed
									Vector<HistoryObject> curPanelCheckBoxHistoryObjects = ((EditOneSelectParameterPanel) curPanel)
											.createHistoryObjectToInheritTemporarySelectParameterCheckBoxValues();
									changes.addAll(curPanelCheckBoxHistoryObjects);
								} else if (curPanel instanceof EditOneTextParameterPanel) {

									// check if text fields changed
									HistoryObject curPanelHistoryObject = curPanel
											.createHistoryObjectToInheritTemporaryValues();
									if (curPanelHistoryObject != null) {
										changes.add(curPanelHistoryObject);
									}

									// check if check boxes changed
									Vector<HistoryObject> curPanelCheckBoxHistoryObjects = ((EditOneTextParameterPanel) curPanel)
											.createHistoryObjectToInheritTemporaryTextParameterCheckBoxValues();
									changes.addAll(curPanelCheckBoxHistoryObjects);
								}

							}

							// Iterate through all changes and apply them
							if (EditJobPanel.this.instantViewManager == null) {
								for (HistoryObject curChange : changes) {
									EditJobPanel.this.workflowPanel.getGraph()
											.addNewAction(curChange);
									EditJobPanel.this.workflowPanel.getGraph()
											.performLastAction();
								}
							} else {
								for (HistoryObject curChange : changes) {
									curChange.performAction();
								}
								EditJobPanel.this.instantViewManager
										.createAndExecuteEdgeViewWorkflow(EditJobPanel.this.correspondingJob);
							}
							EditJobPanel.this.editJobFrame.finished();
						}
					});
		}
		return this.okButton;
	}

	/**
	 * This method creates the cancel Button.
	 */
	private JButton getCancelButton() {
		if (this.cancelButton == null) {
			this.cancelButton = new JButton();
			this.cancelButton.setBounds(new Rectangle(81, 131, 75, 29));
			this.cancelButton.setText("Cancel");
			this.cancelButton
					.addActionListener(new java.awt.event.ActionListener() {
						@Override
						public void actionPerformed(java.awt.event.ActionEvent e) {
							EditJobPanel.this.editJobFrame.finished();
						}
					});
		}
		return this.cancelButton;
	}

	/**
	 * This method creates default value Buttons for every parameterPanel.
	 */
	private JButton getDefaultValueButton() {
		if (this.defaultValueButton == null) {
			this.defaultValueButton = new JButton();
			this.defaultValueButton.setBounds(new Rectangle(152, 131, 75, 29));
			this.defaultValueButton.setText("Defaults");
			this.defaultValueButton
					.addActionListener(new java.awt.event.ActionListener() {
						@Override
						public void actionPerformed(java.awt.event.ActionEvent e) {
							for (EditOneParameterPanel curPanel : EditJobPanel.this.parameterPanels) {
								curPanel.showDefaultValues();
							}
						}
					});
		}
		return this.defaultValueButton;
	}

	/**
	 * This method creates the lastValueButton for every parameterPanel.
	 */
	private JButton getLastValueButton() {
		if (this.lastValueButton == null) {
			this.lastValueButton = new JButton();
			this.lastValueButton.setText("Last Values");
			this.lastValueButton.setBounds(new Rectangle(187, 170, 75, 29));
			this.lastValueButton
					.addActionListener(new java.awt.event.ActionListener() {
						@Override
						public void actionPerformed(java.awt.event.ActionEvent e) {
							for (EditOneParameterPanel curPanel : EditJobPanel.this.parameterPanels) {
								curPanel.showLastValues();

							}

						}
					});
		}
		return this.lastValueButton;
	}

}