/*
 * 
 * ProFiDo - The Processes Fitting Toolkit Dortmund
 * Copyright (C) 2010 - 2012 LS Informatik IV, TU Dortmund
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

package gui;

import java.awt.Rectangle;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.util.Vector;

import core.BooleanParameterGroup;
import core.graph.JobNode;
import core.history.ChangeBooleanParameterGroupValuesHistoryObject;
import core.history.ChangeParameterGroupVisibility;
import core.history.HistoryObject;

/**
 * This class defines the properties of the panel for editing one
 * BooleanParameterGroup. It contains no additional elements as the value of the
 * parameter group is defined by the state of the enable attribute.
 */

public class EditOneBooleanParameterPanel extends EditOneParameterPanel {
	private static final long serialVersionUID = 1L;

	/**
	 * Creates and initializes the panel.
	 * 
	 * @param showVisibleCheckBox
	 */
	public EditOneBooleanParameterPanel(JobNode node,
			BooleanParameterGroup group, EditJobPanel editJobPanel,
			WorkflowPanel workflowPanel, boolean showVisibleCheckBox) {
		super(node, editJobPanel, workflowPanel, showVisibleCheckBox);
		this.myParameterGroup = group;
		this.initialize();
	}

	/**
	 * Initializes all components used by this panel and adds them to its
	 * content pane.
	 */
	@Override
	protected void initialize() {
		super.initialize();
		this.enableCheckBox.setText("Set");
		this.addComponentListener(new ComponentAdapter() {
			@Override
			public void componentResized(ComponentEvent evt) {
				EditOneBooleanParameterPanel.this.updatePositions();
			}
		});
		this.updatePositions();

	}

	@Override
	public void updatePositions() {
		int topMargin = 13;
		int bottomMargin = 4;

		int leftMargin = 6;
		int rightMargin = 6;
		int componentLeftSpacing = 2;
		int componentTopSpacing = 2;

		int contentWidth = this.getWidth() - leftMargin - rightMargin;
		int labelPositionCorrection = 4;

		// Define desired component sizes
		int helpLabelWidth = 10;
		// int enableCheckBoxWidth = 74;
		int visibleCheckBoxWidth = 76;

		int labelHeight = 16;
		// int textFieldHeight = 28;
		int checkBoxHeight = 23;

		if (this.helpLabel != null) {
			this.helpLabel.setBounds(new Rectangle(leftMargin
					+ labelPositionCorrection + contentWidth - helpLabelWidth,
					topMargin, helpLabelWidth, labelHeight));
		}

		int secondRowTopPosition = topMargin + labelHeight
				+ componentTopSpacing;

		if (this.visibleCheckBox != null) {
			this.visibleCheckBox.setBounds(new Rectangle(this.getWidth()
					- rightMargin - visibleCheckBoxWidth, secondRowTopPosition,
					visibleCheckBoxWidth, checkBoxHeight));
		}

		if (this.enableCheckBox != null) {
			this.enableCheckBox.setBounds(new Rectangle(leftMargin,
					secondRowTopPosition, contentWidth - componentLeftSpacing
							- visibleCheckBoxWidth, checkBoxHeight));
		}

		this.desiredHeight = this.enableCheckBox.getY()
				+ this.enableCheckBox.getHeight() + bottomMargin;

	}

	/**
	 * This method sets the values of the text field and the check boxes to the
	 * default values.
	 */
	@Override
	public void showDefaultValues() {
		this.enableCheckBox
				.setSelected(((BooleanParameterGroup) this.myParameterGroup)
						.getDefaultValueEnabled());
		this.visibleCheckBox
				.setSelected(((BooleanParameterGroup) this.myParameterGroup)
						.getDefaultVisibility());
	}

	/**
	 * This method determines the check boxes whose values has been changed and
	 * creates and return a history object to inherit these changes.
	 */
	public Vector<HistoryObject> createHistoryObjectToInheritTemporaryBooleanParameterCheckBoxValues() {

		Vector<HistoryObject> curPanelHistoryObject = new Vector<HistoryObject>();

		if (this.enableCheckBox.isSelected() != ((BooleanParameterGroup) this.myParameterGroup)
				.isEnabled()
				&& this.visibleCheckBox.isSelected() != ((BooleanParameterGroup) this.myParameterGroup)
						.isVisible()) {

			curPanelHistoryObject
					.add(new ChangeBooleanParameterGroupValuesHistoryObject(
							this.node.getName(),
							(BooleanParameterGroup) this.myParameterGroup,
							this.enableCheckBox.isSelected()));
			curPanelHistoryObject.add(new ChangeParameterGroupVisibility(
					this.node.getName(), this.myParameterGroup,
					this.visibleCheckBox.isSelected()));
			// System.out.println("E & V");

		}

		else if (this.enableCheckBox.isSelected() != ((BooleanParameterGroup) this.myParameterGroup)
				.isEnabled()
				&& !(this.visibleCheckBox.isSelected() != ((BooleanParameterGroup) this.myParameterGroup)
						.isVisible())) {

			curPanelHistoryObject
					.add(new ChangeBooleanParameterGroupValuesHistoryObject(
							this.node.getName(),
							(BooleanParameterGroup) this.myParameterGroup,
							this.enableCheckBox.isSelected()));
			// System.out.println("E & !V");
		}

		else if (!(this.enableCheckBox.isSelected() != ((BooleanParameterGroup) this.myParameterGroup)
				.isEnabled())
				&& this.visibleCheckBox.isSelected() != ((BooleanParameterGroup) this.myParameterGroup)
						.isVisible()) {

			curPanelHistoryObject.add(new ChangeParameterGroupVisibility(
					this.node.getName(), this.myParameterGroup,
					this.visibleCheckBox.isSelected()));
			// System.out.println("!E & V");
		}

		return curPanelHistoryObject;
	}

	@Override
	public HistoryObject createHistoryObjectToInheritTemporaryValues() {
		if (this.enableCheckBox.isSelected() != ((BooleanParameterGroup) this.myParameterGroup)
				.isEnabled())
			return new ChangeBooleanParameterGroupValuesHistoryObject(
					this.node.getName(),
					(BooleanParameterGroup) this.myParameterGroup,
					this.enableCheckBox.isSelected());
		else
			return null;
	}

	/**
	 * This method sets the values of the text field and the check boxes to the
	 * last stored values.
	 */
	@Override
	public void showLastValues() {
		this.enableCheckBox
				.setSelected(((BooleanParameterGroup) this.myParameterGroup)
						.isEnabled());
		this.visibleCheckBox
				.setSelected(((BooleanParameterGroup) this.myParameterGroup)
						.isVisible());
	}

}