/*
 * 
 * ProFiDo - The Processes Fitting Toolkit Dortmund
 * Copyright (C) 2010 - 2012 LS Informatik IV, TU Dortmund
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

package gui;

import java.awt.Color;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.event.MouseEvent;

import javax.swing.BorderFactory;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.border.TitledBorder;

import core.ParameterGroup;
import core.graph.JobNode;
import core.history.HistoryObject;

/**
 * This abstract class defines the properties of the panel for editing one
 * parameter group. It displays all values and settings of the parameter group
 * and performs any changes to it by using the history.
 */

public abstract class EditOneParameterPanel extends JPanel {
	private static final long serialVersionUID = 1L;

	/**
	 * Height which this panel needs in order to be displayed correctly. The
	 * value is depended on the type of the panel as determined by its class.
	 */
	protected int desiredHeight;

	/**
	 * Label displaying information about this panels parameter group.
	 */
	protected JLabel helpLabel = null;

	/**
	 * Reference to the EditOneParameterPanel which will be edited.
	 */
	protected EditOneParameterPanel me;

	/**
	 * Reference to the EditJobPanel this panel is part of.
	 */
	protected EditJobPanel editJobPanel;

	/**
	 * Reference to the workflow panel.
	 */
	protected WorkflowPanel workflowPanel;

	/**
	 * Checkbock used to choose if this panels parameter group is enabled.
	 */
	protected JCheckBox enableCheckBox = null;

	/**
	 * Checkbock used to choose if this panels parameter group is visible.
	 */
	protected JCheckBox visibleCheckBox = null;

	/**
	 * Parameter group assigned to this panel.
	 */
	protected ParameterGroup myParameterGroup;

	/**
	 * Node which this panels assigned parameter group belongs to.
	 */
	protected JobNode node;

	/**
	 * Stores whether the Parameter group is visible or not.
	 */
	private boolean showVisibleCheckBox;

	/**
	 * Creates and initializes the panel.
	 */
	public EditOneParameterPanel(JobNode node, EditJobPanel editJobPanel,
			WorkflowPanel workflowPanel, boolean showVisibleCheckBox) {
		this.editJobPanel = editJobPanel;
		this.me = this;
		this.node = node;
		this.workflowPanel = workflowPanel;
		this.showVisibleCheckBox = showVisibleCheckBox;
	}

	public int getDesiredHeight() {
		return this.desiredHeight;
	}

	/**
	 * Display the default values of this parameter group on this panel.
	 */
	public abstract void showDefaultValues();

	/**
	 * Display the values currently assigend to this parameter group on this
	 * panel.
	 */
	public abstract void showLastValues();

	/**
	 * Creates and returns a HistoryObject representing all changes the user
	 * made to the values displayed by this panel.
	 */
	public abstract HistoryObject createHistoryObjectToInheritTemporaryValues();

	/**
	 * Initializes all components used by this panel and adds them to its
	 * content pane.
	 */
	protected void initialize() {
		this.setSize(440, 73);
		this.setLayout(null);
		this.add(this.getHelpLabel(), null);
		this.add(this.getEnableCheckBox(), null);
		this.add(this.getVisibleCheckBox(), null);
		this.setBorder(BorderFactory.createTitledBorder(
				BorderFactory.createLineBorder(Color.black, 1),
				this.myParameterGroup.getName(), TitledBorder.CENTER,
				TitledBorder.TOP));

	}

	/**
	 * This method creates the HelpLabel which will display information about
	 * the parameter group.
	 */
	private JLabel getHelpLabel() {
		if (this.helpLabel == null) {
			this.helpLabel = new JLabel() {
				private static final long serialVersionUID = 1L;

				@Override
				public Point getToolTipLocation(MouseEvent e) {
					return new Point(e.getX() - 145, e.getY());
				}
			};
			this.helpLabel.setBounds(new Rectangle(334, 13, 13, 16));
			this.helpLabel.setText("?");
			this.helpLabel.setToolTipText(this.myParameterGroup
					.getDescription());
			this.helpLabel.addMouseListener(new java.awt.event.MouseAdapter() {
				@Override
				public void mouseClicked(java.awt.event.MouseEvent e) {
					EditOneParameterPanel.this.myParameterGroup.getName();
					JOptionPane.showMessageDialog(
							EditOneParameterPanel.this.me,
							EditOneParameterPanel.this.myParameterGroup
									.getDescription(), "Parameter Information",
							JOptionPane.INFORMATION_MESSAGE);
					// System.out.println("mouseClicked()");
				}
			});
		}
		return this.helpLabel;
	}

	/**
	 * This method creates a checkbox. This checkbox is needed to let the user
	 * enable or disable a parameter group.
	 */
	private JCheckBox getEnableCheckBox() {
		if (this.enableCheckBox == null) {
			this.enableCheckBox = new JCheckBox();
			this.enableCheckBox.setSelected(this.myParameterGroup.isEnabled());
			this.enableCheckBox.setBounds(new Rectangle(197, 32, 77, 23));
		}
		return this.enableCheckBox;
	}

	/**
	 * This method creates a checkbox. This checkbox is needed to let the user
	 * change the visibility of the parameter group.
	 */
	private JCheckBox getVisibleCheckBox() {
		if (this.visibleCheckBox == null) {
			this.visibleCheckBox = new JCheckBox();
			this.visibleCheckBox.setText("Visible");
			this.visibleCheckBox.setSelected(this.myParameterGroup.isVisible());
			this.visibleCheckBox.setBounds(new Rectangle(307, 35, 76, 23));
			this.visibleCheckBox.setVisible(!this.showVisibleCheckBox);
		}
		return this.visibleCheckBox;
	}

	/**
	 * Updates the position of all components on this panel.
	 */
	protected abstract void updatePositions();

	public ParameterGroup getGroup() {
		return this.myParameterGroup;
	}
}
