/*
 * 
 * ProFiDo - The Processes Fitting Toolkit Dortmund
 * Copyright (C) 2010 - 2012 LS Informatik IV, TU Dortmund
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

package gui;

import java.awt.Choice;
import java.awt.Rectangle;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.util.Vector;

import javax.swing.JLabel;

import core.SelectParameterGroup;
import core.graph.JobNode;
import core.history.ChangeParameterGroupEnable;
import core.history.ChangeParameterGroupVisibility;
import core.history.ChangeSelectParameterGroupValuesHistoryObject;
import core.history.HistoryObject;

/**
 * This class defines the properties of the panel for editing one
 * SelectParameterGroup. It contains a select box showing all possible values.
 */

public class EditOneSelectParameterPanel extends EditOneParameterPanel {
	
	/**
	 * Label for displaying additional information.
	 */
	private JLabel infoLabel = null;

	/**
	 * Choice box displaying all possible values.
	 */
	private Choice choice = null;

	/**
	 * Creates and initializes the panel.
	 * 
	 * @param showVisibleCheckBox
	 */
	public EditOneSelectParameterPanel(JobNode node,
			SelectParameterGroup group, EditJobPanel editJobPanel,
			WorkflowPanel workflowPanel, boolean showVisibleCheckBox) {
		super(node, editJobPanel, workflowPanel, showVisibleCheckBox);
		this.myParameterGroup = group;

		this.initialize();
		this.writeChoice();
	}

	/**
	 * Initializes all components used by this panel and adds them to its
	 * content pane.
	 */
	@Override
	public void initialize() {
		super.initialize();
		this.infoLabel = new JLabel();
		this.infoLabel.setBounds(new Rectangle(11, 14, 304, 16));
		this.updateInfoLabelText();
		this.add(this.infoLabel, null);
		this.add(this.getChoiceField(), null);

		this.enableCheckBox.setText("Enable");

		this.addComponentListener(new ComponentAdapter() {
			@Override
			public void componentResized(ComponentEvent evt) {
				EditOneSelectParameterPanel.this.updatePositions();
			}
		});
		this.updatePositions();
	}

	/**
	 * This method creates the Choice box which will display all possible values.
	 */
	public Choice getChoiceField() {
		if (this.choice == null
				&& this.myParameterGroup instanceof SelectParameterGroup) {
			this.choice = new Choice();
			this.choice.setBounds(new Rectangle(22, 37, 174, 28));
			this.proceedChoice();

		}
		return this.choice;
	}

	/**
	 * This method prints all possible file types to the choice box.
	 */
	public void proceedChoice() {
		Vector<String> list = ((SelectParameterGroup) this.myParameterGroup)
				.getChoices();

		for (int i = 0; i < list.size(); i++) {
			this.choice.add(list.get(i));
		}

		this.updateInfoLabelText();
	}

	/**
	 * This method updates the info label text of the parameter panel.
	 */
	private void updateInfoLabelText() {
		this.infoLabel.setText("Format: "
				+ ((SelectParameterGroup) this.myParameterGroup).getFormat()
				+ " (Result: "
				+ ((SelectParameterGroup) this.myParameterGroup)
						.getCommandlineCallString() + ")");
	}

	/**
	 * Updates the position of all components on this panel.
	 */
	@Override
	public void updatePositions() {
		int topMargin = 13;
		int bottomMargin = 4;

		int leftMargin = 6;
		int rightMargin = 6;
		int componentLeftSpacing = 2;
		int componentTopSpacing = 2;

		int contentWidth = this.getWidth() - leftMargin - rightMargin;
		int labelPositionCorrection = 4;

		// Define desired component sizes
		int helpLabelWidth = 10;
		int enableCheckBoxWidth = 74;
		int visibleCheckBoxWidth = 76;

		int labelHeight = 16;
		int textFieldHeight = 28;
		int checkBoxHeight = 23;

		// Calculate derived sizes and positions
		int checkBoxTopCorrection = (textFieldHeight - checkBoxHeight) / 2;

		if (this.infoLabel != null) {
			this.infoLabel.setBounds(new Rectangle(leftMargin
					+ labelPositionCorrection, topMargin, contentWidth
					- componentLeftSpacing - helpLabelWidth, labelHeight));
		}

		if (this.helpLabel != null) {
			this.helpLabel.setBounds(new Rectangle(this.infoLabel.getX()
					+ this.infoLabel.getWidth() + componentLeftSpacing,
					topMargin, helpLabelWidth, labelHeight));
		}

		int secondRowTopPosition = topMargin + labelHeight
				+ componentTopSpacing;

		if (this.visibleCheckBox != null) {
			this.visibleCheckBox.setBounds(new Rectangle(this.getWidth()
					- rightMargin - visibleCheckBoxWidth, secondRowTopPosition
					+ checkBoxTopCorrection, visibleCheckBoxWidth,
					checkBoxHeight));
		}

		if (this.enableCheckBox != null) {
			this.enableCheckBox.setBounds(new Rectangle(this.visibleCheckBox
					.getX() - componentLeftSpacing - enableCheckBoxWidth,
					secondRowTopPosition + checkBoxTopCorrection,
					enableCheckBoxWidth, checkBoxHeight));
		}

		if (this.choice != null) {
			this.choice.setBounds(new Rectangle(leftMargin,
					secondRowTopPosition, contentWidth - 2
							* componentLeftSpacing - enableCheckBoxWidth
							- visibleCheckBoxWidth, textFieldHeight));
		}

		this.desiredHeight = this.choice.getY() + this.choice.getHeight()
				+ bottomMargin;

	}

	/**
	 * This method sets the values of the text field and the check boxes to the
	 * default values.
	 */
	@Override
	public void showDefaultValues() {
		this.choice.select(((SelectParameterGroup) this.myParameterGroup)
				.getDefaultChoice());
		this.enableCheckBox
				.setSelected(((SelectParameterGroup) this.myParameterGroup)
						.getDefaultValueEnabled());
		this.visibleCheckBox
				.setSelected(((SelectParameterGroup) this.myParameterGroup)
						.getDefaultVisibility());
	}

	/**
	 * This method sets the values of the text field and the check boxes to the
	 * last stored values.
	 */
	@Override
	public void showLastValues() {
		this.choice.select(((SelectParameterGroup) this.myParameterGroup)
				.getChoiceValue());
		this.enableCheckBox
				.setSelected(((SelectParameterGroup) this.myParameterGroup)
						.isEnabled());
		this.visibleCheckBox
				.setSelected(((SelectParameterGroup) this.myParameterGroup)
						.isVisible());

	}

	/**
	 * This method determines the check boxes whose values has been changed and
	 * creates and return a history object to inherit these changes.
	 */
	public Vector<HistoryObject> createHistoryObjectToInheritTemporarySelectParameterCheckBoxValues() {

		Vector<HistoryObject> curPanelHistoryObject = new Vector<HistoryObject>();

		if (this.enableCheckBox.isSelected() != ((SelectParameterGroup) this.myParameterGroup)
				.isEnabled()
				&& this.visibleCheckBox.isSelected() != ((SelectParameterGroup) this.myParameterGroup)
						.isVisible()) {

			curPanelHistoryObject.add(new ChangeParameterGroupEnable(this.node
					.getName(), this.myParameterGroup, this.enableCheckBox
					.isSelected()));
			curPanelHistoryObject.add(new ChangeParameterGroupVisibility(
					this.node.getName(), this.myParameterGroup,
					this.visibleCheckBox.isSelected()));
			// System.out.println("E & V");

		}

		else if (this.enableCheckBox.isSelected() != ((SelectParameterGroup) this.myParameterGroup)
				.isEnabled()
				&& !(this.visibleCheckBox.isSelected() != ((SelectParameterGroup) this.myParameterGroup)
						.isVisible())) {

			curPanelHistoryObject.add(new ChangeParameterGroupEnable(this.node
					.getName(), this.myParameterGroup, this.enableCheckBox
					.isSelected()));
			// System.out.println("E & !V");
		}

		else if (!(this.enableCheckBox.isSelected() != ((SelectParameterGroup) this.myParameterGroup)
				.isEnabled())
				&& this.visibleCheckBox.isSelected() != ((SelectParameterGroup) this.myParameterGroup)
						.isVisible()) {

			curPanelHistoryObject.add(new ChangeParameterGroupVisibility(
					this.node.getName(), this.myParameterGroup,
					this.visibleCheckBox.isSelected()));
			// System.out.println("!E & V");
		}

		return curPanelHistoryObject;
	}

	@Override
	public HistoryObject createHistoryObjectToInheritTemporaryValues() {
		String newChoice = this.choice.getSelectedItem();
		String oldChoice = ((SelectParameterGroup) this.myParameterGroup)
				.getChoiceValue();

		if (!oldChoice.equals(newChoice))
			return new ChangeSelectParameterGroupValuesHistoryObject(this.node,
					(SelectParameterGroup) this.myParameterGroup, newChoice,
					oldChoice);
		else
			return null;

	}

	/**
	 * Select the current choice within the choices box.
	 */
	private void writeChoice() {
		this.choice.select(((SelectParameterGroup) this.myParameterGroup)
				.getChoiceValue());
	}

}
