/*
 * 
 * ProFiDo - The Processes Fitting Toolkit Dortmund
 * Copyright (C) 2010 - 2012 LS Informatik IV, TU Dortmund
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

package gui;

import gui.helper.ProFiDoTextField;

import java.awt.Rectangle;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.util.Vector;

import javax.swing.JLabel;
import javax.swing.JTextField;

import core.TextParameterGroup;
import core.graph.JobNode;
import core.history.ChangeParameterGroupEnable;
import core.history.ChangeParameterGroupVisibility;
import core.history.ChangeTextParameterGroupValuesHistoryObject;
import core.history.HistoryObject;

/**
 * This class defines the properties of the panel for editing one
 * TextParameterGroup. It contains a text box in order to display and edit the
 * current value.
 */
public class EditOneTextParameterPanel extends EditOneParameterPanel {

	private static final long serialVersionUID = 1L;

	/**
	 * Label for displaying additional information.
	 */
	private JLabel infoLabel = null;

	/**
	 * Textfield containing the current value.
	 */
	private JTextField valueTextField = null;

	/**
	 * Creates and initializes the panel.
	 * 
	 * @param showVisibleCheckBox
	 * @param showViewCheckBoxes
	 */
	public EditOneTextParameterPanel(JobNode node, TextParameterGroup group,
			EditJobPanel editJobPanel, WorkflowPanel workflowPanel,
			boolean showVisibleCheckBox) {
		super(node, editJobPanel, workflowPanel, showVisibleCheckBox);
		this.myParameterGroup = group;
		this.initialize();
	}

	/**
	 * Initializes all components used by this panel and adds them to its
	 * content pane.
	 */
	@Override
	public void initialize() {
		super.initialize();
		this.infoLabel = new JLabel();
		this.infoLabel.setBounds(new Rectangle(11, 14, 304, 16));
		this.updateInfoLabelText();
		this.add(this.infoLabel, null);
		this.add(this.getValueTextField(), null);

		this.enableCheckBox.setText("Enable");

		this.addComponentListener(new ComponentAdapter() {
			@Override
			public void componentResized(ComponentEvent evt) {
				EditOneTextParameterPanel.this.updatePositions();
			}
		});
		this.updatePositions();
	}

	/**
	 * Updates the text displayed on the info label
	 */
	private void updateInfoLabelText() {
		this.infoLabel.setText("Format: "
				+ ((TextParameterGroup) this.myParameterGroup).getFormat()
				+ " (Result: "
				+ ((TextParameterGroup) this.myParameterGroup)
						.getTemporaryValueString() + ")");
	}

	@Override
	public void updatePositions() {
		int topMargin = 13;
		int bottomMargin = 4;

		int leftMargin = 6;
		int rightMargin = 6;
		int componentLeftSpacing = 2;
		int componentTopSpacing = 2;

		int contentWidth = this.getWidth() - leftMargin - rightMargin;
		int labelPositionCorrection = 4;

		// Define desired component sizes
		int helpLabelWidth = 10;
		int enableCheckBoxWidth = 74;
		int visibleCheckBoxWidth = 76;

		int labelHeight = 16;
		int textFieldHeight = 28;
		int checkBoxHeight = 23;

		// Calculate derived sizes and positions
		int checkBoxTopCorrection = (textFieldHeight - checkBoxHeight) / 2;

		if (this.infoLabel != null) {
			this.infoLabel.setBounds(new Rectangle(leftMargin
					+ labelPositionCorrection, topMargin, contentWidth
					- componentLeftSpacing - helpLabelWidth, labelHeight));
		}

		if (this.helpLabel != null) {
			this.helpLabel.setBounds(new Rectangle(this.infoLabel.getX()
					+ this.infoLabel.getWidth() + componentLeftSpacing,
					topMargin, helpLabelWidth, labelHeight));
		}

		int secondRowTopPosition = topMargin + labelHeight
				+ componentTopSpacing;

		if (this.visibleCheckBox != null) {
			this.visibleCheckBox.setBounds(new Rectangle(this.getWidth()
					- rightMargin - visibleCheckBoxWidth, secondRowTopPosition
					+ checkBoxTopCorrection, visibleCheckBoxWidth,
					checkBoxHeight));
		}

		if (this.enableCheckBox != null) {
			this.enableCheckBox.setBounds(new Rectangle(this.visibleCheckBox
					.getX() - componentLeftSpacing - enableCheckBoxWidth,
					secondRowTopPosition + checkBoxTopCorrection,
					enableCheckBoxWidth, checkBoxHeight));
		}

		if (this.valueTextField != null) {
			this.valueTextField.setBounds(new Rectangle(leftMargin,
					secondRowTopPosition, contentWidth - 2
							* componentLeftSpacing - enableCheckBoxWidth
							- visibleCheckBoxWidth, textFieldHeight));
		}

		this.desiredHeight = this.valueTextField.getY()
				+ this.valueTextField.getHeight() + bottomMargin;
	}

	/**
	 * This method creates a JTextField which will contain the current value.
	 */
	private JTextField getValueTextField() {
		if (this.valueTextField == null
				&& this.myParameterGroup instanceof TextParameterGroup) {
			this.valueTextField = new ProFiDoTextField();
			this.valueTextField.setBounds(new Rectangle(22, 37, 174, 28));
			this.valueTextField.setText(this.myParameterGroup
					.getDisplayFriendlyValueString());
			this.valueTextField
					.addCaretListener(new javax.swing.event.CaretListener() {
						@Override
						public void caretUpdate(javax.swing.event.CaretEvent e) {
							EditOneTextParameterPanel.this
									.proceedTextfieldInput();
						}
					});
			this.proceedTextfieldInput();
		}
		return this.valueTextField;
	}

	/**
	 * This method passes the changed value of the text field as temporary value
	 * to the parameter group and updates the info label afterwards.
	 */
	private void proceedTextfieldInput() {
		((TextParameterGroup) this.myParameterGroup)
				.parseInputToTemporaryValues(this.valueTextField.getText());
		this.updateInfoLabelText();

	}

	/**
	 * This method sets the values of the text field and the check boxes to the
	 * default values.
	 */
	@Override
	public void showDefaultValues() {
		this.valueTextField
				.setText(((TextParameterGroup) this.myParameterGroup)
						.getDefaultValue());
		this.enableCheckBox
				.setSelected(((TextParameterGroup) this.myParameterGroup)
						.getDefaultValueEnabled());
		this.visibleCheckBox
				.setSelected(((TextParameterGroup) this.myParameterGroup)
						.getDefaultVisibility());
	}

	/**
	 * This method sets the values of the text field and the check boxes to the
	 * last stored values.
	 */
	@Override
	public void showLastValues() {
		this.valueTextField.setText(this.myParameterGroup
				.getDisplayFriendlyValueString());
		this.enableCheckBox
				.setSelected(((TextParameterGroup) this.myParameterGroup)
						.isEnabled());
		this.visibleCheckBox
				.setSelected(((TextParameterGroup) this.myParameterGroup)
						.isVisible());

	}

	/**
	 * This method determines the check boxes whose values has been changed and
	 * returns and creates a history object to inherit these changes.
	 */
	public Vector<HistoryObject> createHistoryObjectToInheritTemporaryTextParameterCheckBoxValues() {
		Vector<HistoryObject> curPanelHistoryObject = new Vector<HistoryObject>();

		if (this.enableCheckBox.isSelected() != ((TextParameterGroup) this.myParameterGroup)
				.isEnabled()
				&& this.visibleCheckBox.isSelected() != ((TextParameterGroup) this.myParameterGroup)
						.isVisible()) {

			curPanelHistoryObject.add(new ChangeParameterGroupEnable(this.node
					.getName(), this.myParameterGroup, this.enableCheckBox
					.isSelected()));
			curPanelHistoryObject.add(new ChangeParameterGroupVisibility(
					this.node.getName(), this.myParameterGroup,
					this.visibleCheckBox.isSelected()));
		} else if (this.enableCheckBox.isSelected() != ((TextParameterGroup) this.myParameterGroup)
				.isEnabled()
				&& !(this.visibleCheckBox.isSelected() != ((TextParameterGroup) this.myParameterGroup)
						.isVisible())) {

			curPanelHistoryObject.add(new ChangeParameterGroupEnable(this.node
					.getName(), this.myParameterGroup, this.enableCheckBox
					.isSelected()));
			// System.out.println("E & !V");
		} else if (!(this.enableCheckBox.isSelected() != ((TextParameterGroup) this.myParameterGroup)
				.isEnabled())
				&& this.visibleCheckBox.isSelected() != ((TextParameterGroup) this.myParameterGroup)
						.isVisible()) {

			curPanelHistoryObject.add(new ChangeParameterGroupVisibility(
					this.node.getName(), this.myParameterGroup,
					this.visibleCheckBox.isSelected()));
			// System.out.println("!E & V");
		}

		return curPanelHistoryObject;
	}

	@Override
	public HistoryObject createHistoryObjectToInheritTemporaryValues() {
		Vector<HistoryObject> changes = ((TextParameterGroup) this.myParameterGroup)
				.createHistoryObjectsToAdoptNewValues();
		if (changes.size() > 0)
			return new ChangeTextParameterGroupValuesHistoryObject(
					this.node.getName(),
					(TextParameterGroup) this.myParameterGroup, changes);
		else
			return null;

	}

}