/*
 * 
 * ProFiDo - The Processes Fitting Toolkit Dortmund
 * Copyright (C) 2010 - 2012 LS Informatik IV, TU Dortmund
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

package gui;

import gui.helper.DialogManager;
import gui.helper.Grid;
import gui.helper.InstantViewManager;
import gui.helper.RestoreToolTipTimingsTimerTask;
import gui.helper.status;
import helper.ConverterSignature;
import helper.SmallHelpers;

import java.awt.Component;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.awt.event.InputEvent;
import java.awt.event.MouseEvent;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.Timer;
import java.util.Vector;

import javax.swing.AbstractButton;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.SwingUtilities;
import javax.swing.ToolTipManager;

import core.TypedFile;
import core.graph.BasicNode;
import core.graph.Edge;
import core.graph.FileNode;
import core.graph.Graph;
import core.graph.GraphNode;
import core.graph.InputFileNode;
import core.graph.JobNode;
import core.graph.OutputFileNode;
import core.graph.PointNode;
import core.history.ChangeNodeNameHistoryObject;
import core.history.ChangePointNodeUserPositioned;
import core.history.ConnectNodeHistoryObject;
import core.history.MoveNodeHistoryObject;
import core.history.NewNodeHistoryObject;
import core.history.NewPointNodeHistoryObject;
import core.history.RemoveConnectionHistoryObject;
import core.history.RemoveNodeHistoryObject;
import core.history.SetConverterIDHistoryObject;
import core.history.SetExternalFileNameHistoryObject;
import core.storage.ConfigurationXMLReader;
import core.storage.GraphLoader;

import java.awt.Toolkit;

/**
 * This class defines the properties of the workspace. It displays the workflow
 * and manages all mouse interactions with it.
 */

public class WorkflowPanel extends JPanel {
	private static final long serialVersionUID = 1L;

	/**
	 * Stores the minimum width of the WorkflowPanel.
	 */
	private static final int MINIMUM_WIDTH = 300;

	/**
	 * Stores the minimum height of the WorkflowPanel.
	 */
	private static final int MINIMUM_HEIGHT = 200;

	/**
	 * Stores the X coordinate of the last user click
	 */
	private int mousePressedX;

	/**
	 * Stores the Y coordinate of the last user click
	 */
	private int mousePressedY;

	/**
	 * Timer used to restore tooltip settings when needed.
	 */
	private Timer toolTipRestoreTimer;

	/**
	 * Reference to a JLabel where status information can be written on.
	 */
	private JLabel statusjLabel;

	/**
	 * Reference to the graph representing the workflow.
	 */
	private Graph graph;

	/**
	 * Reference to the currently marked node.
	 */
	private BasicNode markedNode;

	/**
	 * Reference to the node located at the current mouse position.
	 */
	private BasicNode nodeAtMouse;

	/**
	 * Manages the status of this panel in order to allow more sophisticated and
	 * multi-click user interactions.
	 */
	private status curStatus = status.FREE;

	/**
	 * Reference to the grid used to align this graph.
	 */
	private Grid grid;

	/**
	 * Reference to the xmlReader used for accessing the configuration file.
	 */
	private ConfigurationXMLReader xmlReader;

	/**
	 * Stores the font used for output.
	 */
	public static Font FONT = new Font("Arial", Font.PLAIN, 12);

	/**
	 * Reference to the current WorkflowPanel.
	 */
	private WorkflowPanel me;

	/**
	 * Reference to the deleteEdgeMenuItem.
	 */
	private JMenuItem deleteEdgeMenuItem;

	/**
	 * Checkbox to determinate/change whether pointNodes should be positioned
	 * automatically.
	 */
	private JCheckBoxMenuItem pointNodeAutoPositionCheckBoxMenuItem;

	/**
	 * Menu Item to delete a PointNode.
	 */
	private JMenuItem deletePointNodeMenuItem;

	/**
	 * Menu item to access the edit menu for a Edge.
	 */
	private JMenuItem editEdgeMenuItem;

	/**
	 * Menu item to set/create a new PointNode.
	 */
	private JMenuItem setPointNodeMenuItem;

	/**
	 * Reference to a JPopupMenu called "jobNodePopupMenu" used to accessing a
	 * popup menu where all possible jobs are shown.
	 */
	private JPopupMenu jobNodePopupMenu;

	/**
	 * Menu item to delete a FileNode.
	 */
	private JMenuItem deleteFileNodeMenuItem;

	/**
	 * Menu item to connect to FileNodes.
	 */
	private JMenuItem connectFileNodeMenuItem;

	/**
	 * Reference to a JPopupMenu called "fileNodePopupMenu"
	 */
	private JPopupMenu fileNodePopupMenu;

	/**
	 * Menu item to change the file represented by the clicked FileNode.
	 */
	private JMenuItem changeFileNodeFileMenuItem;

	/**
	 * Menu item to access the menu where you can edit the JobNode.
	 */
	private JMenuItem editJobNodeMenuItem;

	/**
	 * Menu item to change the name of a JobNode.
	 */
	private JMenuItem changeJobNodeNameMenuItem;

	/**
	 * Menu item to delete a JobNode.
	 */
	private JMenuItem deleteJobNodeMenuItem;

	/**
	 * Menu item to connect JobNodes.
	 */
	private JMenuItem connectJobNodeMenuItem;

	/**
	 * Menu item to show the command line call for a given JobNode.
	 */
	private JMenuItem showJobNodeCommandLineCallMenuItem;

	/**
	 * Reference to the showHistoryFrame.
	 */
	private ShowHistoryFrame showHistoryFrame;

	/**
	 * Stores the time till a toolTip gets dismissed.
	 */
	private int toolTipDismissDelay;

	/**
	 * Stores the initial delay for a tooltip.
	 */
	private int toolTipInitialDelay;

	/**
	 * Stores the time it takes to show tooltips again.
	 */
	private int toolTipReshowDelay;

	/**
	 * Reference to the current MainWindowFrame called mainFrame.
	 */
	private MainWindowFrame mainFrame;

	/**
	 * Menu item to change the file represented by a Node.
	 */
	private JMenuItem changeFileNodeFileTypeMenuItem;

	/**
	 * Stores whether tooltips should be displayed.
	 */
	private boolean tooltipVisible;

	/**
	 * Menu item to edit a node comment.
	 */
	private JMenuItem editCommentNodeMenuItem;

	/**
	 * Menu item to edit a file node comment.
	 */
	private JMenuItem editCommentFileNodeMenuItem;

	/**
	 * Creates and initializes the panel.
	 * 
	 * @param pStatusjLabel
	 *            the status label
	 * @param pGraphInfojLabel
	 *            the graph information label
	 * @param pXMLReader
	 *            the XML reader
	 */
	public WorkflowPanel(JLabel pStatusjLabel, JLabel pGraphInfojLabel,
			ConfigurationXMLReader xmlReader, MainWindowFrame mainFrame) {
		super();
		this.initialize();
		this.me = this;

		this.setSize(300, 300);

		this.statusjLabel = pStatusjLabel;
		this.mainFrame = mainFrame;
		this.xmlReader = xmlReader;
		this.newWorkflow("untitled");
		this.grid = new Grid(6, 12, 100, 50, false);
		this.toolTipDismissDelay = ToolTipManager.sharedInstance()
				.getDismissDelay();
		this.toolTipInitialDelay = ToolTipManager.sharedInstance()
				.getInitialDelay();
		this.toolTipReshowDelay = this.toolTipInitialDelay;
		ToolTipManager.sharedInstance().setReshowDelay(this.toolTipReshowDelay); // ToolTipManager.sharedInstance().getReshowDelay();
		ToolTipManager.sharedInstance().registerComponent(this);

		this.toolTipRestoreTimer = new Timer();
	}

	@Override
	public String getToolTipText(MouseEvent e) {
		e = this.normalizeMouseEvent(e);
		BasicNode tmpNode = this.graph.getBasicNodeAtPosition(e.getX(),
				e.getY(), this.grid);

		if (tmpNode != null && !tmpNode.getTooltipText().equals("")) {
			this.tooltipVisible = true;
			return tmpNode.getTooltipText();
		} else
			return null;
	}

	/**
	 * This method ensures that the coordinates of passed mouse event are
	 * within the context of this panel. If not, they are truncated accordingly. 
	 */
	private MouseEvent normalizeMouseEvent(MouseEvent e) {
		int mouseX = e.getX();
		int mouseY = e.getY();

		// If outside, move to border
		if (mouseX < 0) {
			mouseX = 0;
		}
		if (mouseX > this.getWidth()) {
			mouseX = this.getWidth();
		}
		if (mouseY < 0) {
			mouseY = 0;
		}
		if (mouseY > this.getHeight()) {
			mouseY = this.getHeight();
		}

		return new MouseEvent((Component) e.getSource(), e.getID(),
				e.getWhen(), e.getModifiers(), mouseX, mouseY,
				e.getXOnScreen(), e.getYOnScreen(), e.getClickCount(), false,
				e.getButton());
	}

	/**
	 * This method is used to change the current marked node visualized by the
	 * WorkflowPanel.
	 */
	private void changeMarkedNode(BasicNode newMarkedNode) {
		if (this.markedNode != null) {
			this.markedNode.setMarked(false);
		}
		this.markedNode = newMarkedNode;
		if (this.markedNode != null) {
			this.markedNode.setMarked(true);
		}
		this.repaint();
	}

	/**
	 * Initializes all components used by this panel and adds them to its
	 * content pane.
	 */
	private void initialize() {
		this.showHistoryFrame = new ShowHistoryFrame(null, this);

		// font = new Font("Arial", Font.PLAIN, 12);

		this.jobNodePopupMenu = this.getJobNodePopupMenu();
		this.fileNodePopupMenu = this.getFileNodePopupMenu();

		this.setLayout(null);

		this.addComponentListener(new ComponentAdapter() {
			@Override
			public void componentResized(ComponentEvent evt) {
				if (WorkflowPanel.this.grid != null) {
					WorkflowPanel.this.grid.setSize(
							WorkflowPanel.this.getWidth(),
							WorkflowPanel.this.getHeight());
					WorkflowPanel.this.graph
							.realignToGrid(WorkflowPanel.this.grid);
					WorkflowPanel.this.me.repaint();
				}
			}
		});

		// ---------------------------------------------------------------------------------------
		// * All action listeners for clicks and movements follow
		// ---------------------------------------------------------------------------------------

		this.addMouseListener(new java.awt.event.MouseAdapter() {
			/**
			 * This method reacts on pressing the mouse button. By pressing the
			 * left mouse button a new node can be placed at the clicked
			 * position. If the left button of the mouse is pressed and its
			 * position is in the click area of a node then this node is marked.
			 * If the right button of the mouse is pressed and its position is
			 * in the click area of an workflow element (node, edge) a pop up
			 * menu is displayed for this element.
			 */

			@Override
			public void mousePressed(java.awt.event.MouseEvent e) {
				e = WorkflowPanel.this.normalizeMouseEvent(e);

				if (e.getButton() == java.awt.event.MouseEvent.BUTTON1) {
					if (WorkflowPanel.this.curStatus == status.FREE) {
						// Currently no action is taking place
						// -> click marks node
						WorkflowPanel.this.me
								.changeMarkedNode(WorkflowPanel.this.graph
										.getBasicNodeAtPosition(e.getX(),
												e.getY(),
												WorkflowPanel.this.grid));
						// selectionRectangle = new Rectangle(0, 0, 0, 0);
						WorkflowPanel.this.mousePressedX = e.getX();
						WorkflowPanel.this.mousePressedY = e.getY();
					} else if (WorkflowPanel.this.curStatus == status.NEW) {
						// Currently waiting for positioning of a new node
						// -> Click set position of new node and finishes the
						// adding of it
						WorkflowPanel.this.markedNode.moveToPixelCoordinates(
								e.getX(), e.getY(), true);
						WorkflowPanel.this.repaint();

						WorkflowPanel.this.graph
								.addNewAction(new NewNodeHistoryObject(
										(GraphNode) WorkflowPanel.this.markedNode,
										null, WorkflowPanel.this.graph));
						WorkflowPanel.this.graph.performLastAction();

						WorkflowPanel.this.curStatus = status.FREE;
						WorkflowPanel.this.statusjLabel.setText("");
						if (WorkflowPanel.this.markedNode instanceof JobNode) {
							WorkflowPanel.this.editJob(
									(JobNode) WorkflowPanel.this.markedNode,
									null);
						}
					}
				} else if (e.getButton() == java.awt.event.MouseEvent.BUTTON3) {
					WorkflowPanel.this.nodeAtMouse = WorkflowPanel.this.graph
							.getBasicNodeAtPosition(e.getX(), e.getY(),
									WorkflowPanel.this.grid);
					if (WorkflowPanel.this.nodeAtMouse != null) {
						if (WorkflowPanel.this.nodeAtMouse instanceof JobNode) {
							WorkflowPanel.this.me.getJobNodePopupMenu().show(
									WorkflowPanel.this.me, e.getX(), e.getY());
						} else if (WorkflowPanel.this.nodeAtMouse instanceof FileNode) {
							WorkflowPanel.this.me.getFileNodePopupMenu().show(
									WorkflowPanel.this.me, e.getX(), e.getY());
						} else if (WorkflowPanel.this.nodeAtMouse instanceof PointNode) {
							WorkflowPanel.this.me.getEdgeAktionsPopupMenu(
									(PointNode) WorkflowPanel.this.nodeAtMouse)
									.show(WorkflowPanel.this.me, e.getX(),
											e.getY());
						}
					}
				}

				WorkflowPanel.this.repaint();
			}

			/**
			 * This method reacts on releasing the mouse button. The node which
			 * is dropped is placed at the position where the mouse button is
			 * released.
			 */
			@Override
			public void mouseReleased(java.awt.event.MouseEvent e) {
				e = WorkflowPanel.this.normalizeMouseEvent(e);

				if (e.getButton() == java.awt.event.MouseEvent.BUTTON1
						&& WorkflowPanel.this.curStatus == status.FREE) {
					if (WorkflowPanel.this.markedNode != null) {
						// Move selected node
						Point newGridPosition = WorkflowPanel.this.grid
								.getNearestCell(e.getX(), e.getY());
						Point oldGridPosition = WorkflowPanel.this.markedNode
								.getGridPosition();
						if (!newGridPosition.equals(oldGridPosition)
								&& (!(WorkflowPanel.this.markedNode instanceof PointNode) || ((PointNode) WorkflowPanel.this.markedNode)
										.isUserPositioned())) {
							WorkflowPanel.this.graph
									.addNewAction(new MoveNodeHistoryObject(
											WorkflowPanel.this.graph,
											WorkflowPanel.this.markedNode,
											oldGridPosition, newGridPosition));
							WorkflowPanel.this.graph.performLastAction();
						} else {
							// Node was not move to another cell but might be
							// not in center anymore, so move back
							WorkflowPanel.this.markedNode.realignToGrid();
						}
					} else {
						// selectionRectangle = null;
					}
					WorkflowPanel.this.repaint();
				}
			}
		});

		this.addMouseMotionListener(new java.awt.event.MouseMotionAdapter() {

			/**
			 * This method reacts on moving the mouse while the key is pressed.
			 * The node which is dragged can be moved on the workspace.
			 */
			@Override
			public void mouseDragged(java.awt.event.MouseEvent e) {
				e = WorkflowPanel.this.normalizeMouseEvent(e);
				// BUTTON1_MASK is needed instead of BUTTON1 in order
				// to work properly under windows.
				if ((e.getModifiers() & InputEvent.BUTTON1_MASK) != 0) {
					if (WorkflowPanel.this.markedNode != null) {
						// Move node without realignen it to the grid
						WorkflowPanel.this.markedNode.moveToPixelCoordinates(
								e.getX(), e.getY(), false);
					} /*
					 * else if (selectionRectangle != null) { int startX =
					 * mousePressedX; int startY = mousePressedY; int width =
					 * Math.abs(e.getX() - startX); int heigth =
					 * Math.abs(e.getY() - startY);
					 * 
					 * if (e.getX() < startX) startX = e.getX(); if (e.getY() <
					 * startY) startY = e.getY();
					 * 
					 * selectionRectangle.setBounds(startX, startY, width,
					 * heigth); }
					 */
					WorkflowPanel.this.repaint();
				}
			}

			@Override
			public void mouseMoved(java.awt.event.MouseEvent e) {
				e = WorkflowPanel.this.normalizeMouseEvent(e);

				// Ensure that all currently visible tooltip are dismissed,
				// when the mouse is moved, in order to preserve correct
				// timing when displaying further tooltips.
				if (WorkflowPanel.this.tooltipVisible) {
					WorkflowPanel.this.tooltipVisible = false;
					ToolTipManager.sharedInstance().setDismissDelay(1); // Dismiss
																		// tooltip
																		// instantly
					// Restore tooltip timing
					WorkflowPanel.this.toolTipRestoreTimer.schedule(
							new RestoreToolTipTimingsTimerTask(
									WorkflowPanel.this.me.toolTipInitialDelay,
									WorkflowPanel.this.me.toolTipReshowDelay,
									WorkflowPanel.this.me.toolTipDismissDelay),
							5);
				}
			}
		});

	}

	/**
	 * This method will create the EdgeAktionsPopupMenu for a given PointNode
	 * (which represents the Edge).
	 */
	private JPopupMenu getEdgeAktionsPopupMenu(final PointNode node) {
		// TODO Pass current edge directly to methods instead of dynamic determination

		JPopupMenu resultPopUpMenu;
		resultPopUpMenu = new JPopupMenu();

		this.deleteEdgeMenuItem = new JMenuItem();
		this.deleteEdgeMenuItem.setText("Delete Connection");
		this.deleteEdgeMenuItem
				.addActionListener(new java.awt.event.ActionListener() {
					@Override
					public void actionPerformed(java.awt.event.ActionEvent e) {
						WorkflowPanel.this.me.deleteEdge();
					}
				});

		this.editEdgeMenuItem = new JMenuItem();
		this.editEdgeMenuItem.setText("Edit Connection");
		this.editEdgeMenuItem
				.addActionListener(new java.awt.event.ActionListener() {
					@Override
					public void actionPerformed(java.awt.event.ActionEvent e) {
						WorkflowPanel.this.me.editEdge();
					}
				});

		this.setPointNodeMenuItem = new JMenuItem();
		this.setPointNodeMenuItem.setText("Add EdgePoint");
		this.setPointNodeMenuItem
				.addActionListener(new java.awt.event.ActionListener() {
					@Override
					public void actionPerformed(java.awt.event.ActionEvent e) {
						WorkflowPanel.this.me.addPointNode();

					}
				});

		this.deletePointNodeMenuItem = new JMenuItem();
		this.deletePointNodeMenuItem.setText("Delete EdgePoint");
		this.deletePointNodeMenuItem
				.addActionListener(new java.awt.event.ActionListener() {
					@Override
					public void actionPerformed(java.awt.event.ActionEvent e) {
						WorkflowPanel.this.me.deleteNode();
					}
				});

		this.pointNodeAutoPositionCheckBoxMenuItem = new JCheckBoxMenuItem();
		this.pointNodeAutoPositionCheckBoxMenuItem.setText("Auto Position");
		this.pointNodeAutoPositionCheckBoxMenuItem.setSelected(!node
				.isUserPositioned());
		this.pointNodeAutoPositionCheckBoxMenuItem
				.addActionListener(new java.awt.event.ActionListener() {
					@Override
					public void actionPerformed(java.awt.event.ActionEvent e) {
						WorkflowPanel.this.me
								.changeAutoPositionOfPointNode(WorkflowPanel.this.pointNodeAutoPositionCheckBoxMenuItem
										.isSelected());
					}
				});

		/*
		 * plotMenuItem = new JMenuItem(); plotMenuItem.setText("Plotter");
		 * plotMenuItem .addActionListener(new java.awt.event.ActionListener() {
		 * public void actionPerformed(java.awt.event.ActionEvent e) {
		 * me.openPlotter(); } });
		 */

		resultPopUpMenu.add(this.deleteEdgeMenuItem);
		resultPopUpMenu.add(this.editEdgeMenuItem);
		resultPopUpMenu.add(this.setPointNodeMenuItem);
		resultPopUpMenu.add(this.deletePointNodeMenuItem);
		resultPopUpMenu.add(this.pointNodeAutoPositionCheckBoxMenuItem);
		// resultPopUpMenu.add(plotMenuItem);

		// Add additional menu items
		final Vector<Integer> matchingViewers = this.xmlReader
				.getAllMatchingViewerIDsForType(node.getEdge()
						.getEdgeFileType(), ConfigurationXMLReader.EDGE_VIEWER);
		if (matchingViewers.size() > 0) {
			resultPopUpMenu.addSeparator();
		}
		for (final int curID : matchingViewers) {
			String curName = this.xmlReader.getViewerNameByID(curID,
					ConfigurationXMLReader.EDGE_VIEWER);
			JMenuItem curItem = new JMenuItem();
			curItem.setText(curName);
			curItem.addActionListener(new java.awt.event.ActionListener() {
				@Override
				public void actionPerformed(java.awt.event.ActionEvent e) {
					WorkflowPanel.this.me.showViewer(node, curID);
				}
			});
			resultPopUpMenu.add(curItem);
		}
		return resultPopUpMenu;
	}

	/**
	 * This method creates and shows a viewer as specified by its parameters to
	 * allow instant display of the files created within a workflow.
	 */
	protected void showViewer(PointNode node, Integer id) {
		// Get file to view
		Edge edgeToView = node.getEdge();

		InstantViewManager manager = new InstantViewManager(this,
				this.xmlReader);
		manager.viewEdge(edgeToView, id);
	}

	/**
	 * This method will be called if the user decides to change the option if a
	 * PointNode should be positioned automatically or manually.
	 */
	protected void changeAutoPositionOfPointNode(boolean selected) {
		if (this.nodeAtMouse instanceof PointNode) {
			boolean curState = ((PointNode) this.nodeAtMouse)
					.isUserPositioned();
			// If value changed
			if (curState == selected) {
				this.graph.addNewAction(new ChangePointNodeUserPositioned(
						(PointNode) this.nodeAtMouse, !selected));
				this.graph.performLastAction();
			}
			this.repaint();
		}
	}

	/**
	 * This Method will create the JobNodePopupMenu and returns it.
	 */
	private JPopupMenu getJobNodePopupMenu() {
		if (this.jobNodePopupMenu == null) {
			this.jobNodePopupMenu = new JPopupMenu();

			if (this.connectJobNodeMenuItem == null) {
				this.connectJobNodeMenuItem = new JMenuItem();
				this.connectJobNodeMenuItem
						.setText("Connect output from selected node to this");
				this.connectJobNodeMenuItem
						.addActionListener(new java.awt.event.ActionListener() {
							@Override
							public void actionPerformed(
									java.awt.event.ActionEvent e) {
								WorkflowPanel.this.me.connectNode();
								WorkflowPanel.this
										.changeMarkedNode(WorkflowPanel.this.nodeAtMouse);
							}
						});
			}

			if (this.deleteJobNodeMenuItem == null) {
				this.deleteJobNodeMenuItem = new JMenuItem();
				this.deleteJobNodeMenuItem.setText("Delete");
				this.deleteJobNodeMenuItem
						.addActionListener(new java.awt.event.ActionListener() {
							@Override
							public void actionPerformed(
									java.awt.event.ActionEvent e) {
								WorkflowPanel.this.me.deleteNode();
							}
						});
			}

			if (this.changeJobNodeNameMenuItem == null) {
				this.changeJobNodeNameMenuItem = new JMenuItem();
				this.changeJobNodeNameMenuItem.setText("Change Name");
				this.changeJobNodeNameMenuItem
						.addActionListener(new java.awt.event.ActionListener() {
							@Override
							public void actionPerformed(
									java.awt.event.ActionEvent e) {
								WorkflowPanel.this
										.changeMarkedNode(WorkflowPanel.this.nodeAtMouse);
								WorkflowPanel.this.me.changeNodeName();
							}
						});
			}

			if (this.editJobNodeMenuItem == null) {
				this.editJobNodeMenuItem = new JMenuItem();
				this.editJobNodeMenuItem.setText("Edit Parameters");
				this.editJobNodeMenuItem
						.addActionListener(new java.awt.event.ActionListener() {
							@Override
							public void actionPerformed(
									java.awt.event.ActionEvent e) {
								if (WorkflowPanel.this.nodeAtMouse != null) {
									WorkflowPanel.this.me
											.editJob(
													(JobNode) WorkflowPanel.this.nodeAtMouse,
													null);
								}
							}
						});
			}

			if (this.editCommentNodeMenuItem == null) {
				this.editCommentNodeMenuItem = new JMenuItem();
				this.editCommentNodeMenuItem.setText("Edit Comment");
				this.editCommentNodeMenuItem
						.addActionListener(new java.awt.event.ActionListener() {
							@Override
							public void actionPerformed(
									java.awt.event.ActionEvent e) {
								if (WorkflowPanel.this.nodeAtMouse != null) {
									WorkflowPanel.this.me
											.editComment((JobNode) WorkflowPanel.this.nodeAtMouse);
								}
							}
						});
			}

			if (this.showJobNodeCommandLineCallMenuItem == null) {
				this.showJobNodeCommandLineCallMenuItem = new JMenuItem();
				this.showJobNodeCommandLineCallMenuItem
						.setText("Show Commandline Call");
				this.showJobNodeCommandLineCallMenuItem
						.addActionListener(new java.awt.event.ActionListener() {
							@Override
							public void actionPerformed(
									java.awt.event.ActionEvent e) {
								if (WorkflowPanel.this.nodeAtMouse != null
										&& WorkflowPanel.this.nodeAtMouse instanceof GraphNode) {
									System.out
											.println(((GraphNode) WorkflowPanel.this.nodeAtMouse)
													.getCommandLineCall("", "",
															false));
								}
								WorkflowPanel.this
										.changeMarkedNode(WorkflowPanel.this.nodeAtMouse);
							}
						});
			}

			this.jobNodePopupMenu.add(this.connectJobNodeMenuItem);
			this.jobNodePopupMenu.add(this.deleteJobNodeMenuItem);
			this.jobNodePopupMenu.add(this.editJobNodeMenuItem);
			this.jobNodePopupMenu.add(this.editCommentNodeMenuItem);
			this.jobNodePopupMenu.add(this.changeJobNodeNameMenuItem);
			this.jobNodePopupMenu.add(this.showJobNodeCommandLineCallMenuItem);

		}
		return this.jobNodePopupMenu;
	}

	/**
	 * This Method creates the FileNodePopupMenu and returns it.
	 */
	private JPopupMenu getFileNodePopupMenu() {
		if (this.fileNodePopupMenu == null) {
			this.fileNodePopupMenu = new JPopupMenu();

			if (this.connectFileNodeMenuItem == null) {
				this.connectFileNodeMenuItem = new JMenuItem();
				this.connectFileNodeMenuItem
						.setText("Connect output from selected node to this");
				this.connectFileNodeMenuItem
						.addActionListener(new java.awt.event.ActionListener() {
							@Override
							public void actionPerformed(
									java.awt.event.ActionEvent e) {
								WorkflowPanel.this.me.connectNode();
								WorkflowPanel.this
										.changeMarkedNode(WorkflowPanel.this.nodeAtMouse);
							}
						});
			}

			if (this.deleteFileNodeMenuItem == null) {
				this.deleteFileNodeMenuItem = new JMenuItem();
				this.deleteFileNodeMenuItem.setText("Delete");
				this.deleteFileNodeMenuItem
						.addActionListener(new java.awt.event.ActionListener() {
							@Override
							public void actionPerformed(
									java.awt.event.ActionEvent e) {
								WorkflowPanel.this.me.deleteNode();
							}
						});
			}

			if (this.changeFileNodeFileMenuItem == null) {
				this.changeFileNodeFileMenuItem = new JMenuItem();
				this.changeFileNodeFileMenuItem
						.setText("Change External File Name");
				this.changeFileNodeFileMenuItem
						.addActionListener(new java.awt.event.ActionListener() {
							@Override
							public void actionPerformed(
									java.awt.event.ActionEvent e) {
								WorkflowPanel.this.me.changeFileNodeFileName();
								WorkflowPanel.this
										.changeMarkedNode(WorkflowPanel.this.nodeAtMouse);
							}
						});
			}

			if (this.changeFileNodeFileTypeMenuItem == null) {
				this.changeFileNodeFileTypeMenuItem = new JMenuItem();
				this.changeFileNodeFileTypeMenuItem
						.setText("Change External File Type");
				this.changeFileNodeFileTypeMenuItem
						.addActionListener(new java.awt.event.ActionListener() {
							@Override
							public void actionPerformed(
									java.awt.event.ActionEvent e) {
								WorkflowPanel.this.me.changeFileNodeFileType(
										true, false);
								WorkflowPanel.this
										.changeMarkedNode(WorkflowPanel.this.nodeAtMouse);
							}
						});
			}

			if (this.editCommentFileNodeMenuItem == null) {
				this.editCommentFileNodeMenuItem = new JMenuItem();
				this.editCommentFileNodeMenuItem.setText("Edit Comment");
				this.editCommentFileNodeMenuItem
						.addActionListener(new java.awt.event.ActionListener() {
							@Override
							public void actionPerformed(
									java.awt.event.ActionEvent e) {
								if (WorkflowPanel.this.nodeAtMouse != null) {
									WorkflowPanel.this.me
											.editComment((FileNode) WorkflowPanel.this.nodeAtMouse);
								}
							}
						});
			}

			/*
			 * if (changeFileNodeTypeMenuItem == null) {
			 * changeFileNodeTypeMenuItem = new JMenuItem();
			 * changeFileNodeTypeMenuItem.setText("Change Type");
			 * changeFileNodeTypeMenuItem .addActionListener(new
			 * java.awt.event.ActionListener() { public void actionPerformed(
			 * java.awt.event.ActionEvent e) { me.changeFileNodeType(); } }); }
			 */

			/*
			 * if (changeFileNodeNameMenuItem == null) {
			 * changeFileNodeNameMenuItem = new JMenuItem();
			 * changeFileNodeNameMenuItem.setText("Change name");
			 * changeFileNodeNameMenuItem .addActionListener(new
			 * java.awt.event.ActionListener() { public void actionPerformed(
			 * java.awt.event.ActionEvent e) { me.changeNodeName(); } }); }
			 */
			this.fileNodePopupMenu.add(this.connectFileNodeMenuItem);
			this.fileNodePopupMenu.add(this.deleteFileNodeMenuItem);
			this.fileNodePopupMenu.add(this.changeFileNodeFileMenuItem);
			this.fileNodePopupMenu.add(this.changeFileNodeFileTypeMenuItem);
			this.fileNodePopupMenu.add(this.editCommentFileNodeMenuItem);

		}
		return this.fileNodePopupMenu;
	}

	// -------------------------------------------------------------------------------
	// * Methods for drawing follow
	// -------------------------------------------------------------------------------

	@Override
	protected void paintComponent(Graphics g) {
		super.paintComponent(g);

		if (g != null) {
			this.drawAll(g);
		}
	}

	/**
	 * This method draws the complete panel and the graph on it on the passed
	 * graphics object.
	 */
	private void drawAll(Graphics g) {
		g.setFont(WorkflowPanel.FONT);

		// Draw graph and grid
		this.grid.draw(g);
		this.graph.draw(g, this.grid);
		/*
		 * // Draw selection rectangle if ((selectionRectangle != null) &&
		 * (selectionRectangle.getWidth() > 0) &&
		 * (selectionRectangle.getHeight() > 0)) { AlphaComposite ta =
		 * AlphaComposite.getInstance(AlphaComposite.SRC_OVER, 0.1f); Graphics2D
		 * g2d = (Graphics2D)g; g2d.setComposite(ta); g2d.setColor(Color.BLUE);
		 * g2d.fill(selectionRectangle);
		 * g2d.setComposite(AlphaComposite.SrcOver); g2d.setColor(Color.GRAY);
		 * g2d.draw(selectionRectangle); }
		 */
	}

	/**
	 * Notify this panel that a repaint is necessary.
	 */

	public void notify(Object caller) {
		// TODO Remove if possible
		System.out.println("Notify called...");
		this.repaint();
		
	}

	/**
	 * This method creates and displayes the file frame after a click on the
	 * menu item "Edit Connection".
	 */
	protected void editEdge() {
		if (this.nodeAtMouse instanceof PointNode) {
			Edge pointNodesEdge = ((PointNode) this.nodeAtMouse).getEdge();

			JFrame editFileFrame = new EditFileFrame(pointNodesEdge, this);

			editFileFrame.setLocationRelativeTo(me);
			Point r = editFileFrame.getLocation();
			r.move(r.x + 240, r.y + 180);
			editFileFrame.setLocation(r);

			// if(editFileFrame.getLocationOnScreen())

			editFileFrame.setVisible(true);
			Point s = editFileFrame.getLocationOnScreen();
		
			int h = Toolkit.getDefaultToolkit().getScreenSize().height;
			if(h-150<= s.getY()) {
				s.setLocation(s.x, s.y -280);
				editFileFrame.setLocation(s);
			}
		}
	}

	/**
	 * This method sets a further PointNode on the edge after a click on the
	 * menu item "Set Edge Point" and adds this action to the history.
	 */
	protected void addPointNode() {
		if (this.nodeAtMouse instanceof PointNode) {
			Edge pointNodesEdge = ((PointNode) this.nodeAtMouse).getEdge();
			this.graph.addNewAction(new NewPointNodeHistoryObject(
					pointNodesEdge, this.nodeAtMouse, this.graph));
			this.graph.performLastAction();
		}
	}

	/**
	 * This method checks if the connection between two nodes is valid.
	 */
	private boolean isConnectionIsValid(GraphNode from, GraphNode to) {
		boolean result = false;
		String message = "Connection between " + from.getName() + " and "
				+ to.getName() + " is not valid.";

		if (from == to) {
			result = false;
			message = message + " A Node should not be connected to itself.";
		} else {
			if (from instanceof JobNode) {

				// Two job nodes are fine
				if (to instanceof JobNode) {
					result = true;

				} else if (to instanceof OutputFileNode) {

					if (to.getIncomingEdges().size() < 1) {
						result = true;
					} else {
						result = false;
						message = message
								+ " An OutputFileNode can only have one input.";
					}
				} else {
					result = false;
					message = message
							+ " A JobNode can only be connected to another JobNode or an OutputFileNode.";
				}
			}

			else if (from instanceof FileNode) {
				if (to instanceof JobNode) {
					if (from instanceof InputFileNode) {
						result = true;
					} else {
						result = false;
					}
				} else {
					result = false;
					message = message
							+ " A FileNode can only be connected to a JobNode.";
				}
			}
		}

		// Display error if any
		if (!result) {
			JOptionPane.showMessageDialog(this, message, "Error",
					JOptionPane.ERROR_MESSAGE);
		}

		return result;
	}

	/**
	 * This methods shows a dialog prompting the user to select the input type
	 * (and the corresponding converter) for the passed fileName. By passing an
	 * additional filter the possible types can be restricted. If showAlways is
	 * unset, the dialog is only showed if several types match the extension of
	 * the passed fileName.
	 */
	private int selectInputConverterID(Vector<String> possibleTypes,
			int oldConverterID, String fileName, boolean showAlways) {
		String fileExtension = this.getExtension(new File(fileName));
		HashMap<Integer, ConverterSignature> inputConverterSignatures = this.xmlReader
				.getInputConverterSignatures(possibleTypes);
		int idOfFirstInputConverterWithMatchingInputSuffix = -1;
		int matchingInputConverterCount = 0;
		for (int curID : inputConverterSignatures.keySet()) {
			if (inputConverterSignatures.get(curID).getInputExtension()
					.equals(fileExtension)) {
				matchingInputConverterCount++;

				if (idOfFirstInputConverterWithMatchingInputSuffix == -1) {
					idOfFirstInputConverterWithMatchingInputSuffix = curID;
				}
			}
		}

		if (idOfFirstInputConverterWithMatchingInputSuffix != -1 && !showAlways
				&& matchingInputConverterCount == 1)
			return idOfFirstInputConverterWithMatchingInputSuffix;
		else {
			// Create entries for selection box
			Vector<String> inputTypes = new Vector<String>();
			for (int curID : inputConverterSignatures.keySet()) {
				inputTypes.add(inputConverterSignatures.get(curID)
						.getSignature());
			}

			// Determine default type
			String defaultType = "";
			if (oldConverterID != -1) {
				defaultType = inputConverterSignatures.get(oldConverterID)
						.getSignature();
			} else {
				if (idOfFirstInputConverterWithMatchingInputSuffix != -1) {
					defaultType = inputConverterSignatures.get(
							idOfFirstInputConverterWithMatchingInputSuffix)
							.getSignature();
				}
			}

			// Show dialog
			String selectedSignature = DialogManager.showSelectionDialog(
					inputTypes, defaultType, "Select Input Type",
					"Please select type of external (input) file.", this);
			if (selectedSignature != null) {
				// Find id of selected type
				for (int curID : inputConverterSignatures.keySet()) {
					if (selectedSignature.equals(inputConverterSignatures.get(
							curID).getSignature()))
						return curID;
				}
			}

			return -1;

		}
	}

	/**
	 * This methods shows a dialog prompting the user to select the output type
	 * (and the corresponding converter) for the passed fileName. By passing an
	 * additional filter the possible types can be restricted. If showAlways is
	 * unset, the dialog is only showed if several types match the extension of
	 * the passed fileName.
	 */
	private int selectOutputConverterID(Vector<String> possibleTypes,
			int oldConverterID, String fileName, boolean showAlways) {
		String fileExtension = this.getExtension(new File(fileName));
		HashMap<Integer, ConverterSignature> outputConverterSignatures = this.xmlReader
				.getOutputConverterSignatures(possibleTypes);
		int idOfFirstOutputConverterWithMatchingOutputSuffix = -1;
		int matchingOutputConverterCount = 0;
		for (int curID : outputConverterSignatures.keySet()) {
			if (outputConverterSignatures.get(curID).getOutputExtension()
					.equals(fileExtension)) {
				matchingOutputConverterCount++;

				if (idOfFirstOutputConverterWithMatchingOutputSuffix == -1) {
					idOfFirstOutputConverterWithMatchingOutputSuffix = curID;
				}
			}
		}

		if (idOfFirstOutputConverterWithMatchingOutputSuffix != -1
				&& !showAlways && matchingOutputConverterCount == 1)
			return idOfFirstOutputConverterWithMatchingOutputSuffix;
		else {
			// Collect possible type
			Vector<String> outputTypes = new Vector<String>();
			for (int curID : outputConverterSignatures.keySet()) {
				outputTypes.add(outputConverterSignatures.get(curID)
						.getSignature());
			}

			// Determine default type
			String defaultType = "";
			if (oldConverterID != -1) {
				defaultType = outputConverterSignatures.get(oldConverterID)
						.getSignature();
			} else {
				if (idOfFirstOutputConverterWithMatchingOutputSuffix != -1) {
					defaultType = outputConverterSignatures.get(
							idOfFirstOutputConverterWithMatchingOutputSuffix)
							.getSignature();
				}
			}

			// Show dialog
			String selectedSignature = DialogManager.showSelectionDialog(
					outputTypes, defaultType, "Select Output Type",
					"Please select type of external (output) file.", this);
			if (selectedSignature != null) {
				// Find id of selected type
				for (int curID : outputConverterSignatures.keySet()) {
					if (selectedSignature.equals(outputConverterSignatures.get(
							curID).getSignature()))
						return curID;
				}
			}

			return -1;

		}
	}

	/**
	 * This method is responsible for the creation of an new InputNode. It is
	 * called after a click on the main window frame menu item "Add Input File".
	 */
	public void newInputFile() {
		File selectedFile = DialogManager.showOpenDialog(
				this.me.getCurrentWorkingDirectory(), "", ".*", this.me, true);
		if (selectedFile != null) {
			int converterID = this.selectInputConverterID(null, -1,
					selectedFile.getName(), false);

			TypedFile externalFile = new TypedFile(selectedFile.getPath());
			if (converterID != -1) {
				this.newAbstractNode(new InputFileNode(0, 0, this.xmlReader,
						this.grid, converterID, this.graph, externalFile));
			}
		}
	}

	/**
	 * This method is responsible for the creation of an new OutputNode. It is
	 * called after a click on the main window frame menu item
	 * "Add Output File".
	 */
	public void newOutputFile() {
		File selectedFile = DialogManager.showSaveDialog(
				this.me.getCurrentWorkingDirectory(), "", ".*", this.me, false);
		if (selectedFile != null) {
			int converterID = this.selectOutputConverterID(null, -1,
					selectedFile.getName(), false);

			TypedFile externalFile = new TypedFile(selectedFile.getPath());
			if (converterID != -1) {
				this.newAbstractNode(new OutputFileNode(0, 0, this.xmlReader,
						this.grid, converterID, this.graph, externalFile));
			}
		}
	}

	/**
	 * This method changes the file type of a FileNode.
	 */
	protected void changeFileNodeFileType(boolean showAlways,
			boolean ignoreOldConverterID) {
		if (this.nodeAtMouse != null && this.nodeAtMouse instanceof FileNode) {
			// Select new file name
			int newConverterID = -1;
			int curConverterID = -1;
			TypedFile curFileName = ((FileNode) this.nodeAtMouse)
					.getExternalFileName();
			if (!ignoreOldConverterID) {
				curConverterID = ((FileNode) this.nodeAtMouse).getConverterID();
			}

			if (this.nodeAtMouse instanceof OutputFileNode) {
				newConverterID = this.selectOutputConverterID(
						((OutputFileNode) this.nodeAtMouse)
								.getCommonIncomingTypes(), curConverterID,
						curFileName.getName(), showAlways);
			} else if (this.nodeAtMouse instanceof InputFileNode) {
				newConverterID = this.selectInputConverterID(
						((InputFileNode) this.nodeAtMouse)
								.getCommonOutgoingTypes(), curConverterID,
						curFileName.getName(), showAlways);
			}

			if (newConverterID != -1 && newConverterID != curConverterID) {
				this.graph
						.addNewAction(new SetConverterIDHistoryObject(
								(FileNode) this.nodeAtMouse, newConverterID,
								this.graph));
				this.graph.performLastAction();
			}
		}
	}

	/**
	 * This method changes the file of a FileNode.
	 */
	protected void changeFileNodeFileName() {
		if (this.nodeAtMouse != null && this.nodeAtMouse instanceof FileNode) {
			// Select new file name
			File newFileName = null;
			TypedFile curFileName = ((FileNode) this.nodeAtMouse)
					.getExternalFileName();

			if (this.nodeAtMouse instanceof OutputFileNode) {
				newFileName = DialogManager
						.showSaveDialog(
								curFileName.getParent(),
								curFileName.getName(),
								this.xmlReader
										.getOutputConverterOutputSuffixByID(((FileNode) this.nodeAtMouse)
												.getReferenceID()), this.me,
								false);
			} else if (this.nodeAtMouse instanceof InputFileNode) {
				newFileName = DialogManager
						.showOpenDialog(
								curFileName.getParent(),
								curFileName.getName(),
								this.xmlReader
										.getInputConverterInputSuffixByID(((FileNode) this.nodeAtMouse)
												.getReferenceID()), this.me,
								true);
			}

			// If a file was selected, check type
			if (newFileName != null) {
				this.graph.addNewAction(new SetExternalFileNameHistoryObject(
						(FileNode) this.nodeAtMouse, newFileName.getPath()));
				this.graph.performLastAction();

				// If the extension changed, let the user select the new type
				if (!this.getExtension(newFileName).equals(
						this.getExtension(curFileName))) {
					this.changeFileNodeFileType(true, true);
				}
			}
		}

	}

	/**
	 * This method returns you the extension for a given File.
	 */
	private String getExtension(File curFileName) {
		String result = "";
		int dotPos = curFileName.getName().lastIndexOf(".");
		if (dotPos > 0) {
			result = curFileName.getName().substring(dotPos,
					curFileName.getName().length());
		}
		return result;
	}

	/**
	 * This method creates and displays the job frame after a click on the menu
	 * item "Edit". If an instantViewManager is passed, the changes will only
	 * affect the corresponding job and won't appear within the history nor will
	 * the currently edited node be marked red.
	 */
	public void editJob(JobNode nodeToEdit,
			InstantViewManager instantViewManager) {
		if (nodeToEdit != null && !nodeToEdit.hasEditDialog()) {
			if (instantViewManager == null) {
				this.changeMarkedNode(nodeToEdit);
			}
			EditJobFrame editJobFrame = new EditJobFrame(nodeToEdit, this,
					instantViewManager);
			editJobFrame.setLocationRelativeTo(this);

			
			Point r = editJobFrame.getLocation();
			r.move(r.x + 240, r.y + 180);
			editJobFrame.setLocation(r);

			// if(editFileFrame.getLocationOnScreen())

			editJobFrame.setVisible(true);
			Point s = editJobFrame.getLocationOnScreen();
		
			int h = Toolkit.getDefaultToolkit().getScreenSize().height;
			if(h-150<= s.getY()) {
				s.setLocation(s.x, s.y -280);
				editJobFrame.setLocation(s);
			}
			

			nodeToEdit.setEditDialog(editJobFrame);
		}
	}

	/**
	 * This method will be called if you want to edit a comment for a given
	 * GraphNode.
	 */
	protected void editComment(GraphNode nodeToEdit) {
		if (nodeToEdit != null) {
			EditCommentFrame enterTextFrame = new EditCommentFrame(300, 300,
					nodeToEdit, this.graph.getHistory(), this);
			enterTextFrame.setVisible(true);
		}

	}

	/**
	 * This method deletes the selected edge. This method is called after a
	 * click on the menu item "Delete Connection".
	 */
	protected void deleteEdge() {
		if (this.nodeAtMouse instanceof PointNode) {
			Edge pointNodesEdge = ((PointNode) this.nodeAtMouse).getEdge();
			this.graph.addNewAction(new RemoveConnectionHistoryObject(
					pointNodesEdge, this.graph));
			this.graph.performLastAction();
			this.repaint();
			pointNodesEdge = null;
		}
	}

	/**
	 * This method deletes a node with all its edges. This method is called
	 * after a click on the menu item "Delete".
	 */
	public void deleteNode() {
		// Delete node only if one is selected and, in case it is
		// a point node, it is not the only point node of the edge
		if (this.nodeAtMouse != null
				&& (!(this.nodeAtMouse instanceof PointNode) || !((PointNode) this.nodeAtMouse)
						.isOnlyPointNodeOfEdge())) {
			this.graph.addNewAction(new RemoveNodeHistoryObject(
					this.nodeAtMouse, this.graph));
			this.graph.performLastAction();

			if (this.nodeAtMouse == this.markedNode) {
				this.me.changeMarkedNode(null);
			}
			this.nodeAtMouse = null;

			this.repaint();
		}
	}

	/**
	 * This method changes the name of a node. This method is called after a
	 * click on the menu item "Change Name".
	 */
	protected void changeNodeName() {

		if (this.nodeAtMouse != null
				&& !(this.nodeAtMouse instanceof PointNode)) {

			String newName = InstantViewManager.createOptionPane(
					this.nodeAtMouse.getName(), "Edit Job Name",
					"Enter new Name:", this);

			if (newName != null) {

				this.graph.addNewAction(new ChangeNodeNameHistoryObject(
						(GraphNode) this.nodeAtMouse, newName));
				this.graph.performLastAction();

				this.repaint();
			}
		}
	}

	/**
	 * This method is responsible for the creation of a new job. The jobs type
	 * is determined by the passed id. This method is called after a click on
	 * one of the job nodes in the main window frame menu. The output types of
	 * the current job are read out from the XML and displayed in the
	 * "Select Output" dialog window where the user can determine the output
	 * type of the job.
	 */
	public void newJob(int pId) {
		/*
		 * Vector<String> types = xmlReader.getProgramOutputTypesByID(pId);
		 * String targetType = DialogManager.showSelectionDialog(types,
		 * types.get(0), "Select Output", "Select Output Type for Job " +
		 * xmlReader.getProgrammNameByID(pId), this);
		 */
		this.newAbstractNode(new JobNode(0, 0, this.xmlReader, this.grid, pId,
				this.graph));
	}

	/**
	 * This method is called to change the state of the panel in order to place
	 * the new created node which is passed to this method.
	 */
	public void newAbstractNode(GraphNode newAbstractNode) {
		if (this.curStatus == status.FREE) {
			this.curStatus = status.NEW;

			// Save new node
			this.changeMarkedNode(newAbstractNode);

			// statusjLabel.setText("Node created, please klick to position.");
			this.repaint();
		}
	}

	/**
	 * This method connects two nodes with each other. Before the connection is
	 * created, the method checks if the connection between these two nodes is
	 * valid.
	 */
	private void connectNode() {
		if (this.markedNode != null && this.nodeAtMouse != null) {
			GraphNode from = (GraphNode) this.markedNode;
			GraphNode to = (GraphNode) this.nodeAtMouse;

			if (this.isConnectionIsValid(from, to)) {
				Edge edge = new Edge(from, to, to.getNextFreeOrderKey());
				if (edge.getPossibleTypes().size() > 0) {
					this.graph.addNewAction(new ConnectNodeHistoryObject(edge,
							this.graph));
					this.graph.performLastAction();

					this.markedNode.setMarked(false);

					this.repaint();
				} else {
					String message = "Connection between " + from.getName()
							+ " and " + to.getName() + " is not valid.\n";

					message += "\n";
					message += from.getName()
							+ " has the following types as Output: \n";
					for (String curType : edge.getFrom().getOutputFileTypes()) {
						message += curType + "\n";
					}
					message += "\n";

					message += to.getName()
							+ " expects the following types as Input: \n";

					for (String curType : edge.getTo().getInputFileTypes()) {
						message += curType + "\n";
					}

					JOptionPane.showMessageDialog(this, message, "Error",
							JOptionPane.ERROR_MESSAGE);

				}
			}

		}
	}

	/**
	 * Sets if the workflow should be scaled on resize.
	 */
	public void setScaleOnResize(boolean state) {
		this.grid.scaleOnResize(state);
	}

	/**
	 * Sets if the grid should be visible.
	 */
	public void setShowGrid(boolean state) {
		this.grid.setVisible(state);
		this.repaint();
	}

	/**
	 * This method saves the workflow under the passed name.
	 */
	public void saveGraph(String pFileName) {
		try {
			BufferedWriter bw = new BufferedWriter(new FileWriter(new File(
					pFileName)));
			bw.write(this.graph.getXMLRepresentationOfGraph());
			bw.close();
			this.graph.setSaved(true);
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	/**
	 * Saves a snaphot of the current graph in the users home folder.
	 */
	public void saveSnapshot() {
		SimpleDateFormat dateTimeFormat = new SimpleDateFormat(
				"yyyy.MM.dd_HH.mm.ss");
		Date now = new Date();

		String targetFolder = SmallHelpers
				.replaceVariables("${HOME}/ProFiDo/Snapshots");

		File targetFile = new File(targetFolder);
		targetFile.mkdir();

		String fileName = "_snapshot_" + dateTimeFormat.format(now) + ".xml";

		if (this.graph.getName().equals("")) {
			fileName = "untitled" + fileName;
		} else {
			fileName = this.graph.getName() + fileName;
		}

		this.saveGraph(targetFolder + '/' + fileName);
	}

	/**
	 * Returns the current default working directory.
	 */
	public String getCurrentWorkingDirectory() {
		if (this.graph == null || this.graph != null
				&& this.graph.getWorkingDirectory().equals(""))
			// If no graph exists, use default location
			return System.getProperty("user.home") + "/ProFiDo";
		else
			// Otherwise read from graph
			return this.graph.getWorkingDirectory();
	}

	/**
	 * This method loads the graph from the location specified by the passed
	 * filename.
	 * 
	 * @param checkFilesOnLoad
	 * @param checkSchema
	 */
	public void loadGraph(String pFileName, boolean checkFilesOnLoad,
			boolean checkSchema) {
		String errorMessage = "";
		Graph newGraph = null;

		try {
			GraphLoader loader = new GraphLoader(pFileName, this.xmlReader,
					this.me, checkSchema);

			if ((graph != null) && (!graph.getWorkingDirectory().equals(""))) {
				newGraph = loader.loadGraph(checkFilesOnLoad,
						graph.getWorkingDirectory());
			} else {
				newGraph = loader.loadGraph(checkFilesOnLoad, "");
			}

			if (newGraph != null) {
				this.graph = newGraph;
				this.graph.realignToGrid(this.grid);

				this.nodeAtMouse = null;
				this.markedNode = null;
				this.curStatus = status.FREE;

				// Determine frame size
				Rectangle boundingBox = newGraph.getBoundingBox(
						this.grid.getCellWidth(), this.grid.getCellHeight());
				int topOffset = (int) SwingUtilities.convertPoint(this,
						new Point(0, 0), this.mainFrame).getY();

				this.grid.dontScaleOnNextResize();
				int width = boundingBox.x + boundingBox.width + 40;
				if (width < WorkflowPanel.MINIMUM_WIDTH) {
					width = WorkflowPanel.MINIMUM_WIDTH;
				}
				int height = boundingBox.y + boundingBox.height + topOffset
						+ 20;
				if (height < WorkflowPanel.MINIMUM_HEIGHT) {
					height = WorkflowPanel.MINIMUM_HEIGHT;
				}
				this.mainFrame.setSize(width, height);
				this.mainFrame.updateTitle(newGraph.getName());
				this.repaint();
			}
		} catch (Exception e) {
			if (e.getMessage() != null) {
				errorMessage = e.getMessage();
			}
		}

		if (!errorMessage.equals("") || newGraph == null) {
			JOptionPane.showMessageDialog(null,
					"The workflow could not be loaded.\n" + errorMessage,
					"Error", JOptionPane.ERROR_MESSAGE);
		}

	}

	public void setBatchName(String newName) {
		this.graph.setName(newName);
		this.mainFrame.updateTitle(this.graph.getName());
	}

	public String getBatchName() {
		return this.graph.getName();
	}

	/*
	 * public String getExpFolder(){ return graph.getExpFolder(); }
	 */

	/**
	 * This method checks if the workflow is consistent to export the script and
	 * shows a message box with further information about any problem.
	 * 
	 * @return true if the workflow is consistent else false.
	 */
	public boolean check() {
		String allEdgesConnectedMessage = this.graph.checkBuild();
		String cycleMessage = this.graph.checkForCycle();

		String message = allEdgesConnectedMessage + cycleMessage;

		if (message.equals(""))
			return true;
		else {
			JOptionPane
					.showMessageDialog(
							this,
							"The current workflow cannont be exported. Please solve the following problems:\n"
									+ message, "Incorrect Workflow",
							JOptionPane.ERROR_MESSAGE);
			return false;
		}
	}

	/**
	 * This method exports the workflow to the passed location.
	 */
	public void exportWorkflowScript(String targetFile, boolean withReport) {
		try {
			this.graph.createScripts(targetFile, this, withReport);
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	/**
	 * This method is called if you want to export the report.sh script.
	 */
	public void exportReportScript() {
		try {
			this.graph.createReport(this);
		} catch (IOException e) {
			e.printStackTrace();
		}

	}

	public void setWorkingDirectory(String outputFolder) {
		this.graph.setWorkingDirectory(outputFolder);
	}

	/**
	 * Creates a new workflow according to the passed values.
	 */
	public void newWorkflow(String name) {
		// If current graph has a working directory use it for
		// the new graph, otherwise initialize with empty folder
		if (this.graph != null && !this.graph.getWorkingDirectory().equals("")) {
			this.graph = new Graph(name, this.graph.getWorkingDirectory(), "",
					this.xmlReader, this, this.showHistoryFrame);
		} else {
			this.graph = new Graph(name, "", "", this.xmlReader, this,
					this.showHistoryFrame);
		}

		this.mainFrame.updateTitle(this.graph.getName());
		this.me.repaint();
	}

	/**
	 * Undoes the last change done to the workflow.
	 */
	public void undo() {
		this.graph.undo();
	}

	/**
	 * Redoes the last change done to the workflow.
	 */
	public void redo() {
		this.graph.redo();
	}

	public Graph getGraph() {
		return this.graph;
	}

	public boolean isGraphSaved() {
		return this.graph.isSaved();
	}

	public void clearHistory() {
		this.graph.clearHistory();
	}

	public Grid getGrid() {
		return this.grid;
	}

	/**
	 * Shows the current history in a ShowHistoryFrame.
	 */
	public void showHistory() {
		if (!this.showHistoryFrame.isVisible()) {
			// this.showHistoryFrame.setLocation(this.getX() + this.getWidth()
			// + 20, this.getY());
			this.showHistoryFrame.setLocationRelativeTo(this);
			Point r = this.showHistoryFrame.getLocation();
			r.move(r.x + 240, r.y + 180);
			this.showHistoryFrame.setLocation(r);
			this.showHistoryFrame.setVisible(true);
		}

	}

	public boolean getScaleOnResize() {
		return this.grid.getScaleOnResize();
	}

	public JMenuItem getDeleteJobMenuItem() {
		return this.deleteJobNodeMenuItem;
	}

	public AbstractButton getDeleteEdgeMenuItem() {
		return this.deleteEdgeMenuItem;
	}

	public void saveScreenshot(String fileName) {
		this.graph.saveScreenshot(fileName, this.grid);

	}

	public Vector<Point> getJobNodePositions() {
		return this.graph.getJobNodePositions();
	}

	public Vector<Point> getEdgePositions() {
		return this.graph.getEdgePositions();
	}

	/**
	 * Sets which information should be printed on the edges and nodes of the
	 * graph.
	 */
	public void setPrintInformation(boolean displayEdgeInformation,
			boolean displayNodeInformation, boolean alwaysDisplayOrderKey,
			boolean displayAllEdgeInformation, boolean displayAllNodeInformation) {
		Graph.DISPLAY_EDGE_INFORMATION = displayEdgeInformation;
		Graph.DISPLAY_NODE_INFORMATION = displayNodeInformation;
		Graph.ALWAYS_DISPLAY_ORDER_KEYS = alwaysDisplayOrderKey;
		Graph.DISPLAY_ALL_EDGE_INFORMATION = displayAllEdgeInformation;
		Graph.DISPLAY_ALL_NODE_INFORMATION = displayAllNodeInformation;
		this.repaint();

	}

	public ShowHistoryFrame getShowHistoryFrame() {
		return this.showHistoryFrame;
	}
	
	public MainWindowFrame getMainWindowFrame(){
		
		return this.mainFrame;
	}

}
