package gui.helper;

import gui.InstantViewFrame;
import gui.WorkflowPanel;

import java.io.File;
import java.util.HashMap;
import java.util.Vector;

import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;

import core.ParameterGroup;
import core.SelectParameterGroup;
import core.TypedFile;
import core.graph.Edge;
import core.graph.Graph;
import core.graph.JobNode;
import core.graph.OutputFileNode;
import core.storage.ConfigurationXMLReader;

/**
 * This class provides methods that are used to handle the edge view 
 * and report feature. 
 */
public class InstantViewManager {
	// TODO Ensure proper state handling. Members are used to store global information. Should be improved.

	
	/**
	 * Reference to the workflow Panel.
	 */
	private WorkflowPanel workflowPanel;

	/**
	 * Reference to the configuration xml reader.
	 */
	private ConfigurationXMLReader xmlReader;
	
	/**
	 * Reference to the Edge.
	 */
	private Edge edgeToView;

	/**
	 * Represents the viewer that will be used to display the file.
	 */
	private Integer viewerID;

	/**
	 * Represents the File.
	 */
	private String inputFileName;

	/**
	 * Creates the instant view manager and stores the passed objects.
	 */
	public InstantViewManager(WorkflowPanel workflowPanel,
			ConfigurationXMLReader xmlReader) {
		this.workflowPanel = workflowPanel;
		this.xmlReader = xmlReader;
	}

	/**
	 * This method creates all possible report objects for a 
	 * given Edge.
	 */
	public Vector<ReportObject> createReportObjects(Edge edgeToView) {
		Vector<ReportObject> result = new Vector<ReportObject>();

		// Find all possible reportObjectIDs
		Vector<Integer> viewerIDs = this.xmlReader
				.getAllMatchingViewerIDsForType(edgeToView.getEdgeFileType(),
						ConfigurationXMLReader.REPORT);

		// Create Job Node for each id
		for (int curViewerID : viewerIDs) {
			String viewerName = this.xmlReader.getViewerNameByID(curViewerID,
					ConfigurationXMLReader.REPORT);
			String viewerDescription = this.xmlReader.getViewerDescriptionByID(curViewerID,
					ConfigurationXMLReader.REPORT);
			// Prepare global name
			Vector<String> curName = new Vector<String>();
			curName.add(viewerName);

			// Prepare job
			int curJobID = this.xmlReader.getProgramIDOfViewer(curViewerID,
					ConfigurationXMLReader.REPORT);
			Vector<ParameterGroup> parameterGroups = this.xmlReader
					.generateProgramParameterGroups(curJobID);

			if (curJobID != -1) {
				// Prepare parameter groups
				HashMap<String, String> presetParameterGroups = this.xmlReader
						.getViewerPresetParameters(curViewerID,
								ConfigurationXMLReader.REPORT);
				Vector<ParameterGroup> unchangedGroups = ParameterGroup
						.setParameterValues(parameterGroups,
								presetParameterGroups);

				// Find all unset select groups
				Vector<SelectParameterGroup> unsetSelectGroups = new Vector<SelectParameterGroup>();
				for (ParameterGroup curGroup : unchangedGroups)
					if (curGroup instanceof SelectParameterGroup) {
						unsetSelectGroups.add((SelectParameterGroup) curGroup);
					}

				// Create cross product
				Vector<HashMap<String, String>> allCombinations = this
						.createProductOfAllPossibleValuesOfSelectParameter(unsetSelectGroups);

				// Create job nodes
				for (HashMap<String, String> curCombination : allCombinations) {
					// Create name
					Vector<String> localName = new Vector<String>();
					localName.addAll(curName);
					String contentInfo = "";
					for (String curKey : curCombination.keySet()) {
						contentInfo = contentInfo + " "
								+ curCombination.get(curKey);
						localName.add(curCombination.get(curKey));
					}
					contentInfo = contentInfo
							.substring(1, contentInfo.length());

					JobNode viewerJob = this.createJob(edgeToView.getFileName()
							.getName(), curViewerID, curCombination,
							ConfigurationXMLReader.REPORT);

					WorkflowScript workflow = this.createMiniWorkflow(
							"${RESULTDIR}/Report/Files", edgeToView,
							curViewerID, viewerJob,
							ConfigurationXMLReader.REPORT, false);
					result.add(new ReportObject(localName, workflow,
							viewerName, contentInfo, viewerDescription, edgeToView));
				}
			} else {
				WorkflowScript workflow = this.createMiniWorkflow(
						"${RESULTDIR}/Report/Files", edgeToView, curViewerID,
						null, ConfigurationXMLReader.REPORT, false);
				result.add(new ReportObject(curName, workflow, viewerName, "", viewerDescription,
						edgeToView));
			}

		}

		return result;
	}

	/**
	 * This method creates the script for a mini worklofw that will execute the
	 * passed job and convert the result according to the specified converter.
	 */
	private WorkflowScript createMiniWorkflow(String targetResultPath,
			Edge edgeToView, int viewerID, JobNode generatorJob, int section,
			boolean withInitialization) {
		String script = "";
		if (withInitialization) {
			script = script
					+ this.workflowPanel.getGraph().createScriptInitialization(
							this.workflowPanel.getGraph()
									.getLocationOfWorkflowScript(), null,
							false, true, false);
		}

		// Prepare file names
		TypedFile fileToView = edgeToView.getFileName();
		// TypedFile createdFile = fileToView;
		// Vector<String> createdFileTypes = fileToView.getTypes();

		// Involve generator job if existent
		if (generatorJob == null) {
			// Nothing to do
		} else {
			generatorJob.addIncomingEdge(edgeToView);
			fileToView = generatorJob.getXMLOutputFile();
		}

		// Involve converter if existent
		OutputFileNode outFileNode = null;
		int converterID = this.xmlReader.getConverterIDOFViewer(viewerID,
				section);
		if (converterID != -1) {
			this.workflowPanel.getGraph();
			// Create converter
			fileToView = new TypedFile(
					fileToView.getPath()
							+ Graph.getNextFreeID()
							+ this.xmlReader
									.getOutputConverterOutputSuffixByID(converterID),
					this.xmlReader
							.getOutputConverterOutputTypeByID(converterID));
			outFileNode = new OutputFileNode(-1, -1, this.xmlReader,
					this.workflowPanel.getGrid(), converterID,
					this.workflowPanel.getGraph(), fileToView);

			// Connect nodes
			if (generatorJob != null) {
				Edge outputConnection = new Edge(generatorJob, outFileNode, "");
				generatorJob.addOutgoingEdge(outputConnection);
				outFileNode.addIncomingEdge(outputConnection);
			} else {
				outFileNode.addIncomingEdge(edgeToView);
			}
		}

		// Create workflow script
		if (generatorJob != null || outFileNode != null) {
			// Create script
			// instantViewFileToDisplayName = workflowPanel.getExpFolder() +
			// "/Results/Tmp/" + generatedFileNameBaseName;

			// Note:
			// The file to view is located within the global ${RESULTDIR}. As
			// all
			// results created by the instant view feature should be located
			// within resultPath as this folder is used as result directory
			// when creating these calls.
			if (generatorJob != null) {
				// script = script + "\n# Display Job\n";
				script = script
						+ generatorJob.getCommandLineCall("${SOFTWAREDIR}/",
								targetResultPath, false) + "\n";
			}

			if (outFileNode != null) {
				// script = script + "\n# Output Converter\n";
				script = script
						+ outFileNode.getCommandLineCall("${SOFTWAREDIR}/",
								targetResultPath, false);
			}

			// Note:
			// As the file to view is located in ${RESULTDIR} the call as
			// created above won't point to the correct input location. In order
			// to solve this issue a number of possibilities exist. The most
			// simple one is to replace the corresponding path directly within
			// the call string as done below.
			TypedFile fileOfEdge = edgeToView.getFileName();
			script = script.replace(
					"\"" + fileOfEdge.getNameWithPath(targetResultPath, false)
							+ "\"",
					"\"" + fileOfEdge.getNameWithPath("${RESULTDIR}/", false)
							+ "\"");
		}

		if (generatorJob == null && outFileNode == null)
			return new WorkflowScript("", fileToView);
		else
			return new WorkflowScript(script, fileToView);

	}

	/**
 	 * Creates all possible permutations of the strings passed
 	 * to this method.
	 */
	private Vector<Vector<String>> createPermutations(
			Vector<Vector<String>> possibleValues) {
		Vector<Vector<String>> result = new Vector<Vector<String>>();

		if (possibleValues.size() > 0) {
			Vector<String> firstPositionPossibleValues = possibleValues
					.remove(0);
			Vector<Vector<String>> permutationsOfRemainingPossibleValues = this
					.createPermutations(possibleValues);

			for (String curValue : firstPositionPossibleValues) {
				if (permutationsOfRemainingPossibleValues.size() > 0) {
					for (Vector<String> curPermutataion : permutationsOfRemainingPossibleValues) {
						Vector<String> curResult = new Vector<String>();
						curResult.add(curValue);
						curResult.addAll(curPermutataion);
						result.add(curResult);
					}
				} else {
					Vector<String> curResult = new Vector<String>();
					curResult.add(curValue);
					result.add(curResult);
				}
			}
		}

		return result;
	}

	/**
	 * This method creates all permutations for the values of the passed
	 * selectParameterGroups.
	 */
	private Vector<HashMap<String, String>> createProductOfAllPossibleValuesOfSelectParameter(
			Vector<SelectParameterGroup> inputGroups) {
		Vector<HashMap<String, String>> result = new Vector<HashMap<String, String>>();

		// Collect all values
		Vector<Vector<String>> possibleValues = new Vector<Vector<String>>();
		for (SelectParameterGroup curGroup : inputGroups) {
			possibleValues.add(curGroup.getChoices());
		}

		// Create permutation
		Vector<Vector<String>> permutations = this
				.createPermutations(possibleValues);
		//System.out.println("-- Permutations for current viewer --");

		for (Vector<String> curPermutation : permutations) {
			// Create hash map for current permutation
			HashMap<String, String> curHashMap = new HashMap<String, String>();

			for (int curPos = 0; curPos < inputGroups.size(); curPos++) {
				SelectParameterGroup curParameter = inputGroups
						.get(curPos);
				String curParameterGroupValue = curPermutation.get(curPos);
				curHashMap.put(curParameter.getName(), curParameterGroupValue);
				//System.out.print(curParameter.getName() + " = "
				//		+ curParameterGroupValue + "; ");
			}
			//System.out.println();

			result.add(curHashMap);
		}

		return result;
	}

	/**
	 * Creates a job node that will create a viewable output for the passed file
	 * name.
	 */
	public JobNode createJob(String fileToViewName, int viewerID,
			HashMap<String, String> presetParameters, int section) {
		// Determine id of corresponding tool
		int jobID = this.xmlReader.getProgramIDOfViewer(viewerID, section);
		String viewerName = this.xmlReader.getNameOfViewer(viewerID, section);

		if (jobID != -1) {
			JobNode result = new JobNode(fileToViewName + "_" + viewerName, -1,
					-1, this.xmlReader, this.workflowPanel.getGrid(), jobID,
					this.workflowPanel.getGraph());

			// Apply pre set parameters
			presetParameters.putAll(this.xmlReader.getViewerPresetParameters(
					viewerID, section));
			result.setParameterValues(presetParameters);

			return result;
		} else
			return null;
	}

	/**
	 * This method is called in order to show the file represented by the passed
	 * edge.
	 * 
	 * @param edgeToView
	 *            Edge that represents the file to be showed.
	 * @param viewerID
	 *            ID of the viewer that should be used.
	 */
	public void viewEdge(Edge edgeToView, Integer viewerID) {
		// Store values
		this.edgeToView = edgeToView;
		this.viewerID = viewerID;
		this.inputFileName = edgeToView.getFileName().getNameWithPath(
				this.workflowPanel.getGraph().getLocationOfWorkflowScript()
						+ "/Results/", true);

		File inputFile = new File(this.inputFileName);

		if (this.workflowPanel.getGraph().getLocationOfWorkflowScript()
				.equals("")) {
			JOptionPane
					.showMessageDialog(
							this.workflowPanel,
							"The current workflow has not been exported yet.\nPlease export and execute the workflow fist.",
							"Error", JOptionPane.ERROR_MESSAGE);
		} else if (!inputFile.exists()) {
			JOptionPane
					.showMessageDialog(
							this.workflowPanel,
							"The file "
									+ inputFile.getName()
									+ " does not exist.\nPlease export and execute the workflow fist.",
							"Error", JOptionPane.ERROR_MESSAGE);
		} else {
			// Prepare job
			JobNode viewerJob = this.createJob(edgeToView.getFileName()
					.getName(), viewerID, new HashMap<String, String>(),
					ConfigurationXMLReader.EDGE_VIEWER);

			// Edit job if existent
			if (viewerJob != null) {
				viewerJob.setFixedParameterValues(this.xmlReader
						.getViewerFixedParameters(viewerID,
								ConfigurationXMLReader.EDGE_VIEWER));
				this.workflowPanel.editJob(viewerJob, this);
			} else {
				this.createAndExecuteEdgeViewWorkflow(viewerJob);
			}
		}

	}

	/**
	 * This method creates a mini workflow in order to execute the
	 * passed job node. After the workflow is created it is executed
	 * and the result is displayed according to the settings specified
	 * in the configuration xml file.
	 */
	public void createAndExecuteEdgeViewWorkflow(JobNode viewerJob) {
		// Create workflow
		WorkflowScript workflow = this.createMiniWorkflow("${RESULTDIR}/Tmp",
				this.edgeToView, this.viewerID, viewerJob,
				ConfigurationXMLReader.EDGE_VIEWER, true);

		String createdFileFileName = "";
		if (workflow.getWorkflowScript().equals("")) {
			createdFileFileName = this.inputFileName;
		} else {
			// Execute workflow
			String scriptName = this.workflowPanel.getGraph()
					.getLocationOfWorkflowScript() + "/Results/Tmp/edgeview.sh";
			this.workflowPanel.getGraph().writeFile(scriptName,
					workflow.getWorkflowScript(), true);
			createdFileFileName = workflow.getResultFile().getNameWithPath(
					this.workflowPanel.getGraph().getLocationOfWorkflowScript()
							+ "/Results/Tmp", true);
			try {
				Process p = Runtime.getRuntime().exec("sh " + scriptName);
				p.waitFor();
			} catch (Exception e) {
				System.out.println("Error! Could not execute mini workflow.");
			}
		}

		// View file
		try {
			// Determine if to show internal or external
			if (this.xmlReader.getIsViewerInternByID(this.viewerID)) {
				new InstantViewFrame(new TypedFile(createdFileFileName),
						this.xmlReader.getViewerInternTypeByID(this.viewerID),
						this.workflowPanel);
			} else if (this.xmlReader.getIsViewerExternByID(this.viewerID)) {
				String externalBinary = this.xmlReader
						.getViewerExternBinaryByID(this.viewerID);
				Runtime.getRuntime().exec(
						externalBinary + " " + createdFileFileName);
			}
		} catch (Exception e) {
			System.out
					.println("Error! Could not start external viewer for file.");
		}
	}

	/**
	 * Method to create and execute a JOptionPane dialog
	 * 
	 * @param contentText
	 *            text that is displayed in the textfield
	 * @param title
	 *            The title of the JOptionPane
	 * @param text
	 *            The text that is displayed in the OptionPane
	 * @param panel
	 *            parent panel
	 * @return user input String
	 */
	public static String createOptionPane(String contentText, String title,
			String text, JPanel panel) {

		final JTextField textpanel = new ProFiDoTextField(contentText);
		textpanel.selectAll();
		// textpanel.addKeyListener(new ProFiDoTextFieldKeyListener(textpanel));
		Object[] buttons = { "OK", "Cancel" };
		Object[] content = { text, textpanel };
		int option = JOptionPane.showOptionDialog(panel, content, title,
				JOptionPane.YES_OPTION, JOptionPane.PLAIN_MESSAGE, null,
				buttons, null);
		if (option == JOptionPane.YES_OPTION
				&& textpanel.getText().length() > 0)
			return textpanel.getText();
		else
			return null;
	}

}
