/*
 * 
 * ProFiDo - The Processes Fitting Toolkit Dortmund
 * Copyright (C) 2010 - 2012 LS Informatik IV, TU Dortmund
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

package gui.helper;

import java.awt.Point;

/**
 * This class defines the properties of a line.
 * 
 * A line has a start point (startX, startY) and an end point (endX, endY).
 */

public class Line {
	
	/**
	 * Represents the x coordinate where the line starts.
	 */
	private int startX;
	
	/**
	 * Represents the y coordinate where the line starts.
	 */
	private int startY;
	
	/**
	 * Represents the y coordinate where the line ends.
	 */
	private int endY;
	
	/**
	 * Represents the x coordinate where the line ends.
	 */
	private int endX;
	
	/**
	 * y-intercept of the line.
	 */
	private double n;
	
	/**
	 * Slope of the line.
	 */
	private double m;

	/**
	 * Indicates if this line is parallel to the y-axis.
	 */
	private boolean yParallel;

	public Line(int pStartX, int pStartY, int pEndX, int pEndY) {
		this.startX = pStartX;
		this.startY = pStartY;
		this.endX = pEndX;
		this.endY = pEndY;

		// For any line, the following equations hold:
		// (1) startY = m * startX + n
		// (2) endY = m * endX + n
		// Therefore:
		// (1) startY - (m * startX) = n
		// (2) endY = m * endX + startY - m * startX
		// <=> endY = m * endX - m * startX + startY
		// <=> endY - startY = m * (endX - startX)
		// <=> (endY - startY) / (endX - startX) = m
		if (this.endX == this.startX) {
			// In this case m is infinite
			this.yParallel = true;
			this.m = this.startX;
		} else {
			this.yParallel = false;
			double yDiff = this.endY - this.startY;
			double xDiff = this.endX - this.startX;
			this.m = yDiff / xDiff;
			this.n = this.startY - this.m * this.startX;
		}

		// Alter Code
		// pointVector = new Vector2d(pStartX, pStartY);
		// lineVector = new Vector2d(pEndX - pStartX, pEndY - pStartY);
	}

	/**
	 * This method returns the intersection with the passed edge.
	 */
	public Point getIntersection(Line edgeLine) {
		// For the intersection, the following holds:
		// (1) m1 * x + n1 = m2 * x + n2
		// <=> (m1 * x) - (m2 * x) = n2 - n1
		// <=> x * (m1 - m2) = (n2 - n1)
		// <=> x = (n2 - n1) / (m1 - m2)
		if (this.m == edgeLine.getM())
			return null;
		else {
			double intersectionX = 0;
			double intersectionY = 0;
			if (this.yParallel) {
				intersectionX = this.m;
				intersectionY = edgeLine.getM() * intersectionX
						+ edgeLine.getN();
			} else if (edgeLine.getYParallel()) {
				intersectionX = edgeLine.getM();
				intersectionY = this.m * intersectionX + this.n;

			} else {
				intersectionX = Math.round((edgeLine.getN() - this.n)
						/ (this.m - edgeLine.getM()));
				intersectionY = this.m * intersectionX + this.n;
			}

			if (this.isOnThisLine(intersectionX, intersectionY))
				return new Point((int) Math.round(intersectionX),
						(int) Math.round(intersectionY));
			else
				return null;
		}

		// Alter Code:
		// For 2ny point on the line the following holds:
		// (2) x = p + z * l (where x is the point, p 2 known point on the line
		// 2nd l 2 vector on the line)
		// For the intersection, with p2, p1 2nd l2, l1 referring to the p 2nd l
		// of the two lines we get:
		// (2) p2 + z2 * l2 = p1 + z1 * l1
		// Splitting into two equ2tions for e2ch component of the vector
		// (22) p2.x + z2 * l2.x = p1.x + z1 * l1.x
		// <=> z2 = (p1.x + z1 * l1.x - p2.x) / l2.x
		// (2b) p2.y + z2 * l2.y = p1.y + z1 * l1.y
		// <=> p2.y + ((p1.x + z1 * l1.x - p2.x) / l2.x) * l2.y - p1.y = z1 *
		// l1.y
		// <=> p2.y + (p1.x * l2.y) / l2.x + (z1 * l1.x * l2.y) / l2.x - (p2.x *
		// l2.y) / l2.x - p1.y = z1 * l1.y
		// <=> p2.y + (p1.x * l2.y) / l2.x - (p2.x * l2.y) / l2.x - p1.y = z1 *
		// l1.y - (z1 * l1.x * l2.y) / l2.x
		// <=> p2.y + (p1.x * l2.y) / l2.x - (p2.x * l2.y) / l2.x - p1.y = z1 *
		// (l1.y - (l1.x * l2.y) / l2.x)
		// <=> (p2.y * l2.x + p1.x * l2.y - p2.x * l2.y - p1.y * l2.x) / l2.x =
		// z1 * (l1.y * l2.x - l1.x * l2.y) / l2.x
		// <=> (p2.y * l2.x + p1.x * l2.y - p2.x * l2.y - p1.y * l2.x) = z1 *
		// (l1.y * l2.x - l1.x * l2.y)
		//
		// Vector2d p2 = pointVector;
		// Vector2d p1 = edgeLine.getPointVector();
		// Vector2d l2 = lineVector;
		// Vector2d l1 = edgeLine.getLineVector();
		//
		// double z1 = (p2.y * l2.x + p1.x * l2.y - p2.x * l2.y - p1.y * l2.x) /
		// (l1.y * l2.x - l1.x * l2.y);
		// double z2 = (p1.x + z1 * l1.x - p2.x) / l2.x;
		//
		// System.out.println(z1 + " " + l2.x);
		//
		// if (z1 <= 1) {
		// int resultX = (int) Math.round(p2.x + (z2 * l2.x));
		// int resultY = (int) Math.round(p2.y + (z2 * l2.y));
		//
		// return new Point(resultX, resultY);
		// } else
		// return null;

	}

	/**
	 * This method determines if the delivered coordinates are on the line.
	 */
	private boolean isOnThisLine(double intersectionX, double intersectionY) {
		return intersectionX >= this.startX && intersectionX <= this.endX
				&& intersectionY >= this.startY && intersectionY <= this.endY;
	}

	private boolean getYParallel() {
		return this.yParallel;
	}

	public double getN() {
		return this.n;
	}

	public double getM() {
		return this.m;
	}

	/**
	 * This method returns the angle between the line and the positive x-axis.
	 * This method is called by draw to determine the direction of the arrow to
	 * draw it in a correct way.
	 * 
	 * @return the angel at which the positive x-axis has to be turned counter
	 *         clockwise till it meets the arrow.
	 */
	public double getAngle() {
		double winkel = Math.atan(this.m);

		if (winkel < -1 * Math.toRadians(1)) {
			winkel = winkel + Math.PI;
		}

		if (this.startY == this.endY)
			if (this.startX < this.endX)
				return winkel;
			else
				return winkel + Math.PI;
		else if (this.startY > this.endY)
			return winkel + Math.PI;
		else
			return winkel;
	}

	public double f(double pX) {
		return pX * this.m + this.n;
	}

	public int getStartX() {

		return this.startX;
	}

	public int getEndX() {

		return this.endX;
	}

	public int getEndY() {

		return this.endY;
	}

}
