package gui.helper;

import helper.SmallHelpers;

import java.util.Vector;

import core.TypedFile;
import core.graph.Edge;
import core.graph.Graph;

/**
 * This class represents one single object in the report. Report
 * objects can be used to display arbitrary information about any 
 * file in the workflow. This class contains all informations needed 
 * to created and include this particular object. 
 * 
 * Report objects can be of any type, but typically they
 * are either graphic or text.
 */

public class ReportObject {
	/**
	 * Name of the variable used to enable / disable
	 * this object within scripts.
	 */
	
	private String controlVariableName;
	
	/**
	 * String describing this report object.
	 */
	
	private String viewerDescription;
	
	/**
	 * Bash script that will create the file system
	 * representation of this object.
	 */
	private WorkflowScript script;
	
	/**
	 * Name of the viewer that is used to create this object. 
	 */
	private String viewerName;
	
	/**
	 * Informations about the content of this report object.
	 */
	private String contentInfo;
	
	/**
	 * Edge that contains the file within the workflow that
	 * is represented by this report object.
	 */
	private Edge edgeToView;

	public ReportObject(Vector<String> controlVariable, WorkflowScript script,
			String viewerName, String contentInfo, String viewerDescription, Edge edgeToView) {
		// Parse control variable
		this.controlVariableName = "";
		this.viewerDescription = viewerDescription + " (";

		for (String curNamePart : controlVariable) {
			this.controlVariableName = this.controlVariableName + curNamePart;
			this.viewerDescription = this.viewerDescription + " " + curNamePart;
			// controlVariableName = controlVariableName.substring(0,
			// controlVariableName.length() - 1);
		}

		this.viewerDescription = this.viewerDescription + ")";
		this.viewerName = viewerName;
		this.contentInfo = contentInfo;
		this.script = script;
		this.edgeToView = edgeToView;
	}

	/*
	 * public String getFileToViewName() { return
	 * script.getResultFile().getName(); }
	 */

	public String getFileToViewContentInfo() {
		return this.contentInfo;
	}

	public String getViewerName() {
		return this.viewerName;
	}

	public String getControlVariableName() {
		return this.controlVariableName;
	}
	
	public String getViewerDescription() {
		return this.viewerDescription;
	}

	public String getFileToViewNameWithPath() {
		return this.script.getResultFile().getNameWithPath("Files/", true);
	}

	/**
	 * Creates a name to label this report object within the report. 
	 */
	public String getLabelName() {
		// Create label
		String label = this.edgeToView.getFrom().getName() + "_"
				+ this.edgeToView.getFileName().getName() + "_"
				+ this.viewerName + "_" + this.contentInfo;
		return label;
	}

	/**
	 * Creates a shell script that creates a file representing this
	 * report object.
	 */
	public String getWorkflowScript() {
		String result = "";
		result = result + "# Representation for "
				+ this.getEdgeToViewFileName() + " (" + this.getLabelName()
				+ ")\n";
		result = result + "if [[ \"$" + this.getControlVariableName()
				+ "\" == 1 ]] ; then\n";
		result = result + this.script.getWorkflowScript() + "\n";
		result = result + "fi\n";

		return result;
	}

	/** 
	 * Creates the latex code needed to include this report object
	 * in the report.
	 */
	public String getLatexInclude(String prefix) {
		String result = "";
		// Set latex variables
		result = result + prefix + "% Representation for file "
				+ this.getEdgeToViewFileName() + " (" + this.getLabelName()
				+ ")\n";
		result = result + prefix + "\\renewcommand{\\filename}{"
				+ SmallHelpers.escapeLatex(this.getEdgeToViewFileName())
				+ "}\n";
		result = result + prefix + "\\renewcommand{\\filecontent}{"
				+ SmallHelpers.escapeLatex(this.getFileToViewContentInfo())
				+ "}\n";
		result = result + prefix + "\\renewcommand{\\viewername}{"
				+ SmallHelpers.escapeLatex(this.getViewerName()) + "}\n";
		result = result + prefix + "\\renewcommand{\\labelname}{"
				+ SmallHelpers.escapeLatex(this.getLabelName()) + "}\n";
		result = result + prefix + "\\renewcommand{\\localID}{"
				+ Graph.getNextFreeID() + "}\n";

		// Set variable check
		result = result + prefix + "\\ifdefined\\"
				+ this.getControlVariableName() + "\n";
		result = result + prefix + "\\profidoinput{"
				+ this.getFileToViewNameWithPath() + "}\n";
		result = result + prefix + "\\fi\n";
		return result;
	}

	public String getEdgeToViewFileName() {
		return this.edgeToView.getFileName().getName();
	}

	public TypedFile getEdgeToViewFile() {
		return this.edgeToView.getFileName();
	}



}
